/**
 * \file mgui/tree.h
 * \brief Definitions for MGUI::CTRL_TREE class.
 *
 * \if NODOC
 * $Id: tree.h_v 1.28 2011/12/28 14:15:03 mju Exp $
 *
 * $Log: tree.h_v $
 * Revision 1.28  2011/12/28 14:15:03  mju
 * Remove unneeded hdr.
 *
 * Revision 1.27  2011/11/09 16:06:27  mju
 * Remove mfc defns.
 *
 * Revision 1.26  2010/02/12 22:26:17  mju
 * Add setDelegateOnSize.
 *
 * Revision 1.25  2009/11/25 15:58:49  mju
 * Fix doxygen comment.
 *
 * Revision 1.24  2008/05/23 16:15:03  mju
 * Add isRedrawDisabled.
 *
 * Revision 1.23  2008/04/28 19:39:36  mju
 * Add ctrlstyle_ImageIdxIsIconId.
 * Add setItemImageNormal.
 *
 * Revision 1.22  2008/03/03 21:32:45  mju
 * Add setAutoSelectOnClick.
 *
 * Revision 1.21  2008/03/03 17:27:16  mju
 * Remove setDelegateOnLeft/RightButtonDown/Up as not used.
 *
 * Revision 1.20  2008/03/03 17:11:16  mju
 * Add override and delegates for drag-and-drop reordering.
 *
 * Revision 1.19  2030/02/18 18:57:05  mju
 * Add ctrlstyle_SizeInPixels.
 *
 * Revision 1.18  2007/06/28 17:43:53  dwilliss
 * Added delegate support.
 * Added support for double-click and keypress (only via delegates)
 *
 * Revision 1.17  2007/02/15 15:24:06  mju
 * Add scrollToShow parm to ensureVisible.
 *
 * Revision 1.16  2007/02/08 16:21:15  mju
 * Add 'recursive' parm to findChildByID.
 *
 * Revision 1.15  2006/09/28 13:16:25  mju
 * Use mistring hdr.
 *
 * Revision 1.14  2006/02/16 18:59:32  mju
 * Change tooltip mechanism to include hitresult and location.
 *
 * Revision 1.11  2005/10/14 17:41:32  mju
 * Add setItemBold.
 *
 * Revision 1.10  2005/09/28 14:25:22  mju
 * Add drawitem and redraw methods.
 *
 * Revision 1.9  2005/09/21 21:33:45  mju
 * Pass modifier key state to v_on...Button... methods.
 *
 * Revision 1.8  2005/09/21 17:51:48  mju
 * Add setItemImageState.
 *
 * Revision 1.7  2005/09/19 16:56:46  mju
 * In docs for setBold indicate must have valid status.
 *
 * Revision 1.6  2005/09/06 20:33:13  mju
 * Add v_onPreInsertItem.
 *
 * Revision 1.5  2005/09/01 13:09:44  mju
 * Add v_onDestroyItem.
 *
 * Revision 1.3  2004/07/29 17:44:38  mju
 * Keep track of which values are valid in iteminfo so when set don't clobber previous.
 *
 * Revision 1.1  2004/05/05 20:26:04  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_TREE_H
#define	INC_MGUI_TREE_H

#ifndef	INC_MGUI_CTRL_H
#include <mgui/ctrl.h>
#endif

#ifndef	INC_MGUI_KEYCODE_H
#include <mgui/keycode.h>
#endif

#ifndef	INC_MGUI_POINTERBUTTON_H
#include <mgui/pointerbutton.h>
#endif

#ifndef	INC_MGUI_CUSTOMDRAW_H
#include <mgui/customdraw.h>
#endif

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace MGUI {
	class CTRL_TOOLTIP;
	class CUSTOMDRAW_TREE;
	class IMAGELIST;
	}
#endif


namespace MGUI {

//! Tree control with support for expand/collapse.
class CTRL_TREE : public CTRL {
	public:

		//! Style for control specified when created.
		enum CTRLSTYLE {
			CTRLSTYLE_Default =					0x0000,	//!< 
			CTRLSTYLE_ShowLines =				0x0001,	//!< Show lines linking child items to corresponding parent item
			CTRLSTYLE_ShowLinesAtRoot =		0x0002,	//!< Show lines linking root items, required if want buttons for root items
			CTRLSTYLE_NoButtons =				0x0004,	//!< Don't show buttons for expand/collapse
			CTRLSTYLE_SingleExpand =			0x0008,	//!< Only expand one item at a time, collapsing previously expanded item
			CTRLSTYLE_NoToolTips =				0x0010,	//!< Disable automatic tool tips for text items when not entirely visible
			CTRLSTYLE_EditableText =			0x0020,	//!< Allow in-place editing of text items
			CTRLSTYLE_NoScrollHorizontal =	0x0040,	//!< Disallow horizontal scrolling and don't show horizontal scrollbar
			CTRLSTYLE_SizeInPixels =			0x0080,	//!< Size parameters specified in pixels
			CTRLSTYLE_ImageIdxIsIconID =		0x0100,	//!< Index for "normal" image is an ICONID
			};

		//! Style for item specified when created.
		enum ITEMSTYLE {
			ITEMSTYLE_Default =					0x0000,	//!< Default style
			ITEMSTYLE_OwnerText =				0x0001,	//!< Call v_OnGetItemInfo to obtain text and v_OnSetDispInfo when text edited
			ITEMSTYLE_OwnerImages =				0x0002,	//!< Call v_OnGetItemInfo to obtain normal/selected images
			ITEMSTYLE_AllowChildReorder =		0x0004,	//!< Allow reordering of child items via drag-and-drop within parent
			};

		//! 'Handle' used to reference items in tree.
		enum ITEMHND {
			ITEMHND_Last =							-2,		//!< Indicates insertion as last item in child list
			ITEMHND_None =							-1,		//!< Indicates insertion as first child item, as well as non-existent item
			ITEMHND_Root =							0,			//!< Indicates root of tree
			};

		//! Item status values.
		enum ITEMSTATUS {
			ITEMSTATUS_None =						0x00,
			ITEMSTATUS_Bold =						0x01,		//!< Item text shown in bold
			ITEMSTATUS_DropHighlighted =		0x10,		//!< Item is highlighted as target for drag-and-drop operation
			ITEMSTATUS_Expanded =				0x02,		//!< Item is presently expanded
			ITEMSTATUS_ExpandedOnce =			0x04,		//!< Item has been expanded at least once
			ITEMSTATUS_Selected =				0x08,		//!< Item is 'selected'
			};

		//! Type of search or relationship, used in FindItem().
		enum FIND {
			FIND_Selected,										//!< Currently selected item
			FIND_DropHighlight,								//!< Item which is target of drag-and-drop operation
			FIND_Parent,										//!< Parent of item
			FIND_FirstChild,									//!< First child item
			FIND_NextSibling,									//!< Next sibling item
			FIND_PreviousSibling,							//!< Previous sibling item
			FIND_TopParent,									//!< Topmost parent (child of root)
			};

		//! Indicator for presence or absence of children for item.
		enum CHILDPRESENCE {
			CHILDPRESENCE_No =	0,							//!< Item has no children
			CHILDPRESENCE_Yes =	1,							//!< Item has children
			CHILDPRESENCE_Ask =	2							//!< Determine children via v_OnGetItemInfo when first expand item
			};

		//! Child visibility setting.
		enum CHILDVISIBILITY {
			CHILDVISIBILITY_Expand,							//!< Expand to show children
			CHILDVISIBILITY_Collapse,						//!< Collapse to hide children
			CHILDVISIBILITY_CollapseReset,				//!< Collapse and remove children, resets to CHILDPRESENCE_Ask
			CHILDVISIBILITY_Toggle							//!< Toggle visibility
			};

		//! Image list type, used in SetImageList().
		enum IMAGETYPE {
			IMAGETYPE_Primary,
			IMAGETYPE_State
			};

		//! Values to set or request for item.
		enum ITEMVALUES {
			ITEMVALUES_None =						0x0000,
			ITEMVALUE_ChildPresence =			0x0001,
			ITEMVALUE_ImageNormal =				0x0002,
			ITEMVALUE_ImageSelected =			0x0004,
			ITEMVALUE_ImageState =				0x0008,
			ITEMVALUE_Status =					0x0010,
			ITEMVALUE_Text =						0x0020,
			ITEMVALUE_ID =							0x0040,
			ITEMVALUE_Style =						0x0080,
			ITEMVALUE_HeightMult =				0x0100,
			ITEMVALUES_All =						0x0FFF
			};

		//! Results of HitTest.
		// (Values chosen to match Windows.)
		enum HITRESULT {
			HITRESULT_Nowhere =					0x0001,	//!< Not on any item
			HITRESULT_OnItemIcon =				0x0002,	//!< On icon (primary image) for item
			HITRESULT_OnItemText =				0x0004,	//!< On text for item
			HITRESULT_OnItemIndent =			0x0008,	//!< On indentation for item
			HITRESULT_OnItemButton =			0x0010,	//!< On visibility toggle for item
			HITRESULT_OnItemRight =				0x0020,	//!< To right of item
			HITRESULT_OnItemStateIcon =		0x0040,	//!< On state icon
			HITRESULT_OnItem =					(HITRESULT_OnItemIcon | HITRESULT_OnItemText | HITRESULT_OnItemStateIcon)	//!< On item icons or text
			};

		#if !defined(GENERATING_DOXYGEN_OUTPUT)
			class PRIV;
		#endif

		//! Container for individual item information.
		//! Used for efficient setting an retrieval of various item information.
		class ITEMINFO {
			public:

				//! Default constructor.
				ITEMINFO (
					);

				// Construct with handle and ID.
				ITEMINFO (
					ITEMHND itemhnd,
					ID id
					);

				//! Construct with specified text and ID.
				explicit ITEMINFO (
					const MISTRING& text,
					ID id = 0
					);

				//! Destructor.
				~ITEMINFO (
					);

				//! Get child presence.
				CHILDPRESENCE GetChildPresence (
					) const { return (m_ChildPresence); }

				//! Get item handle.
				ITEMHND GetHandle (
					) const { return (m_itemhnd); }

				//! Get height multiplier.
				int GetHeightMult (
					) const { return (m_HeightMult); }

				//! Get ID for attaching external information to item.
				ID GetID (
					) const { return (m_id); }

				//! Get index for 'normal' image.
				int GetImageNormal (
					) const { return (m_ImageNormal); }

				//! Get primary image index based on current status.
				//! @return Index in primary image list or -1 if none
				int GetImagePrimary (
					) const { return ((m_status & ITEMSTATUS_Selected) ? GetImageSelected() : GetImageNormal()); }

				//! Get index for 'selected' image.
				int GetImageSelected (
					) const { return ((m_ImageSelected == -2) ? m_ImageNormal : m_ImageSelected); }

				//! Get 'state' image index.
				//! @return Index in 'state' image list (1-15) or 0 for none
				int GetImageState (
					) const { return (m_ImageState); }

				//! Get item status.
				ITEMSTATUS GetStatus (
					) const { return (m_status); }

				//! Get item style.
				ITEMSTYLE GetStyle (
					) const { return (m_style); }

				//! Get item text.
				//! If item text retained by owner the returned string will be empty.
				const MISTRING& GetText (
					) const { return (m_text); }

				//! Get enumeration of valid values.
				ITEMVALUES GetValidValues (
					) const { return (static_cast<ITEMVALUES>(m_ValidValues)); }

				//! Set 'externally settable' members from ITEMINFO.
				void Set (
					const ITEMINFO& iteminfo
					);

				//! Set item 'bold' status.
				//! Status must already be valid for this to have any effect.
				void SetBold (
					bool bold = true							//!< true to make bold, false to not
					);

				//! Set whether item should indicate whether it has children.
				//! This setting has no effect if item already has children.
				void SetChildPresence (
					CHILDPRESENCE ChildPresence			//!< Child presence indication
					) { m_ValidValues |= ITEMVALUE_ChildPresence; m_ChildPresence = ChildPresence; }

				//! Set height multiplier.
				//! Actual item height will be base height * multiplier.
				void SetHeightMult (
					int HeightMult								//!< Height multiplier
					) { m_ValidValues |= ITEMVALUE_HeightMult; m_HeightMult = HeightMult; }

				//! Set ID for attaching external information to item.
				void SetID (
					ID id
					) { m_ValidValues |= ITEMVALUE_ID; m_id = id; }

				//! Set primary image index for 'normal' status.
				void SetImageNormal (
					int IdxNormal								//!< Index in primary image list for 'normal' status, -1 for none
					);

				//! Set primary image index for 'selected' status.
				void SetImageSelected (
					int IdxSelected							//!< Index in primary image list for 'selected' status, -2 to use same as 'normal'
					);

				//! Set 'state' image index.
				void SetImageState (
					unsigned int IdxState				//!< Index in 'state' image list (1-15) or 0 for none
					) { m_ValidValues |= ITEMVALUE_ImageState; m_ImageState = (IdxState < 16) ? IdxState : 0; }

				//! Set to retain returned images so not requested again.
				//! Used in v_OnGetItemInfo().
				void SetRetainImages (
					);

				//! Set to retain returned text so not requested again.
				//! Used in v_OnGetItemInfo().
				void SetRetainText (
					);

				//! Set item status.
				void SetStatus (
					ITEMSTATUS status
					) { m_ValidValues |= ITEMVALUE_Status; m_status = status; }

				//! Set item style for insertion.
				//! Has no effect for existing items.
				void SetStyle (
					ITEMSTYLE style
					) { m_ValidValues |= ITEMVALUE_Style; m_style = style; }

				//! Set item text.
				void SetText (
					const MISTRING& text
					) { m_ValidValues |= ITEMVALUE_Text; m_text = text; }

				#ifndef GENERATING_DOXYGEN_OUTPUT
				// Internal methods used by implementation.
				void SetHandle (
					ITEMHND itemhnd
					) { m_itemhnd = itemhnd; }
				#endif

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				unsigned int m_ValidValues;
				ITEMHND m_itemhnd;
				ID m_id;
				ITEMSTATUS m_status;
				ITEMSTYLE m_style;
				CHILDPRESENCE m_ChildPresence;
				MISTRING m_text;
				int m_HeightMult;
				int m_ImageNormal;
				int m_ImageSelected;
				int m_ImageState;

				friend class TREE;			//!< Give TREE access to ITEM
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Delegate for pointer messages.
		//! Delegate should return 'true' to accept the default behaviour or false to prevent it.
		//! The default behaviour for left click is to select the item clicked.
		typedef fastdelegate::FastDelegate<bool(ITEMHND, HITRESULT, const LPOINT2D&, MGUI::POINTERBUTTON, MGUI::KEYSTATE)> DELEGATE_ONPOINTER;

		//! Delegate for mouse clicks.
		//! Delegate should return 'true' to accept the default behaviour,
		//! or false to prevent it.
		//! The default behaviour for left click is to select the item clicked.
		typedef fastdelegate::FastDelegate<bool(ITEMHND, HITRESULT, const LPOINT2D&, MGUI::KEYSTATE)> DELEGATE_ONMOUSE;

		//! Delegate for Key presses.
		typedef fastdelegate::FastDelegate<void(ITEMHND, HITRESULT, const LPOINT2D&, MGUI::KEYSTATE, MGUI::KEYCODE)> DELEGATE_ONKEY;

		//! Delegate for tooltips.
		typedef fastdelegate::FastDelegate<bool(ITEMHND, HITRESULT, const LPOINT2D&, MGUI::CTRL_TOOLTIP&)> DELEGATE_ONTOOLTIPREQUEST;

		//! Delegate called when the visibility of a child has changed
		typedef fastdelegate::FastDelegate<void(ITEMHND, CHILDVISIBILITY)> DELEGATE_ONCHILDVISIBILITYCHANGED;

		//! Delegate called when the visibility of a child is about to change.
		//! Return 'false' to prevent action for occuring.  Must be used to 
		//! create chiltd items if item has CHILDPRESENCE_Ask
		typedef fastdelegate::FastDelegate<bool(ITEMHND, CHILDVISIBILITY)> DELEGATE_ONCHILDVISIBILITYCHANGING;
		
		//! Called to perform custom drawing.
		typedef fastdelegate::FastDelegate<MGUI::CUSTOMDRAW::RETVALUE(CUSTOMDRAW_TREE&)> DELEGATE_ONCUSTOMDRAW;

		//! Delegate for various item events.
		//! OnDestroyItem, OnItemDeselected, OnItemSelected all
		//! use this delegate
		typedef fastdelegate::FastDelegate<void(ITEMHND)> DELEGATE_ONITEM;

		//! Delegate called before an item is created
		typedef fastdelegate::FastDelegate<void(ITEMINFO&)> DELEGATE_ONPREINSERTITEM;
		//! Delegate called to obtain info for item stored externally to control
		typedef fastdelegate::FastDelegate<void(CTRL_TREE::ITEMVALUES, CTRL_TREE::ITEMINFO&)> DELEGATE_ONGETITEMINFO;

		//! Delegate called to obtain info for item stored externally to control.
		//! Only applies to text value and only if CTRLSTYLE_EditableText 
		//! specified when control created
		typedef fastdelegate::FastDelegate<void(CTRL_TREE::ITEMVALUES, const CTRL_TREE::ITEMINFO&)> DELEGATE_ONSETITEMINFO;

		//! Constructor.
		CTRL_TREE (
			);

		//! Destructor.
		~CTRL_TREE ();

		//! Create control.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			int height = 20,								//!< Height in rows (or pixels if CTRLSTYLE_SizeInPixels)
			int width = 40,								//!< Width in 'em' characters (or pixels if CTRLSTYLE_SizeInPixels)
			CTRLSTYLE style = CTRLSTYLE_Default,	//!< Control style
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand
			);

		//! Delete all children of specified parent.
		void DeleteChildren (
			ITEMHND hndParent						//!< Handle to parent item
			);

		//! Delete item and all children.
		void DeleteItem (
			ITEMHND hndItem						//!< Item handle
			);

		//! Draw single item.
		void DrawItem (
			ITEMHND hndItem,						//!< Item handle
			bool erase = true						//!< Erase existing item
			);

		//! Ensure that specified item is visible, expanding or scrolling as needed.
		void EnsureVisible (
			ITEMHND hndItem,						//!< Item handle
			bool ScrollToShow = true			//!< Scroll if needed to show item
			);

		//! Find child using ID.
		ITEMHND FindChildByID (
			ITEMHND hndParent,					//!< Handle to parent item
			ID id,									//!< Programmer-specified ID
			bool recursive = false				//!< Search descendents recursively if not found directly under parent
			) const;

		//! Find item matching specified criteria or relationship.
		ITEMHND FindItem (
			FIND find,								//!< Relationship for finding item
			ITEMHND hndItem = ITEMHND_None	//!< Item to search from if needed
			) const;

		//! Get base item height.
		//! Items must be integer multiple of the base height.
		//! @return Base height in pixels.
		int GetBaseItemHeight (
			) const;

		//! Get child presence for item
		CHILDPRESENCE GetChildPresence (
			ITEMHND hndItem						//!< Item handle
			) const;

		//! Get first child for given parent.														
		ITEMHND GetFirstChild (
			ITEMHND hndParent						//!< Handle to parent item
			) const { return (FindItem(FIND_FirstChild,hndParent)); }

		//! Get item ID.
		ID GetItemID (
			ITEMHND hndItem						//!< Item handle
			) const;

		//! Get information for item.
		void GetItemInfo (
			ITEMHND hndItem,						//!< Item handle
			ITEMVALUES ItemValues,				//!< Values to retrieve
			ITEMINFO& ItemInfo					//!< Item information returned
			) const;

		//! Get item status.
		ITEMSTATUS GetItemStatus (
			ITEMHND hndItem						//!< Item handle
			) const;

		//! Get next "sibling" item.
		ITEMHND GetNextSibling (
			ITEMHND hndItem						//!< Item handle
			) const { return (FindItem(FIND_NextSibling,hndItem)); }

		//! Get parent of item.
		ITEMHND GetParent (
			ITEMHND hndChild						//!< Child handle
			) const { return (FindItem(FIND_Parent,hndChild)); }

		//! get 'selected' item.
		//! @return Handle to selected item, ITEMHND_None if none selected.
		ITEMHND GetSelectedItem (
			) const { return (FindItem(FIND_Selected)); }

		//! Insert simple text item into tree.
		//! @return Handle to ITEM added.
		ITEMHND InsertItem (
			ITEMHND hndParent,					//!< Parent item, ITEMHND_Root to insert as root item
			ITEMHND hndInsertAfter,				//!< Item to insert after, ITEMHND_None to insert at beginning, ITEMHND_Last to insert at end
			const MISTRING text,					//!< Item text
			ID id = 0								//!< Item ID
			);

		//! Insert item into tree.
		//! @return Handle to ITEM added.
		ITEMHND InsertItem (
			ITEMHND hndParent,					//!< Parent item, ITEMHND_Root to insert as root item
			ITEMHND hndInsertAfter,				//!< Item to insert after, ITEMHND_None to insert at beginning, ITEMHND_Last to insert at end
			ITEMINFO& ItemInfo					//!< Item information, will be updated to contain handle and actual status on return
			);

		//! Determine if redraw is disabled.
		bool IsRedrawDisabled (
			) const;

		//! Redraw entire control.
		void Redraw (
			bool erase = true						//!< Erase before redraw
			);

		//! Disable redraw, incrementing disable count.
		void RedrawDisable (
			);

		//! Enable redraw, decrementing disable count.
		//! Will redraw the control if was disabled and is no longer.
		void RedrawEnable (
			);

		//! Set whether to autoselect item when click on.
		void SetAutoSelectOnClick (
			bool AutoSelectOnClick
			);

		//! Set background color.
		void SetBackgroundColor (
			const COLOR& color					//!< Color to set
			);

		//! Set base item height.
		//! Individual Items will be integer multiple of the base height.
		void SetBaseItemHeight (
			int baseheight							//!< Base height in pixels
			);

		//! Set visibility for children of item.
		void SetChildVisibility (
			ITEMHND hndParent,					//!< Parent handle
			CHILDVISIBILITY visibility			//!< Child visibility
			);

		//! Delegate called when the visibility of a child has changed
		void SetDelegateOnChildVisibilityChanged(DELEGATE_ONCHILDVISIBILITYCHANGED);

		//! Set the delegate to call when the visibility of a child is about to change.
		//! Return 'false' to prevent action for occuring.  Must be used to 
		//! create chiltd items if item has CHILDPRESENCE_Ask
		void SetDelegateOnChildVisibilityChanging(DELEGATE_ONCHILDVISIBILITYCHANGING);
		
		//! Set delegate to perform custom drawing.
		void SetDelegateOnCustomDraw(DELEGATE_ONCUSTOMDRAW);

		void SetDelegateOnDestroyItem (DELEGATE_ONITEM);

		//! Delegate called to obtain info for item stored externally to control
		void SetDelegateOnGetItemInfo (DELEGATE_ONGETITEMINFO);

		void SetDelegateOnItemChangePosition (
			DELEGATE_ONITEM delegate
			) { m_DelegateOnItemChangePosition = delegate; }

		void SetDelegateOnItemDeselected (
			DELEGATE_ONITEM delegate
			) { m_DelegateOnItemDeselected = delegate; }

		void SetDelegateOnItemSelected (
			DELEGATE_ONITEM delegate
			) { m_DelegateOnItemSelected = delegate; }

		//! Set the delegate to call on a key press
		void SetDelegateOnKeyPress(DELEGATE_ONKEY);

		//! Set the delegate to call on left mouse button double-click
		//! The return value of the delegate is ignored (there is no default action)
		void SetDelegateOnLeftButtonDoubleClick(DELEGATE_ONMOUSE);

		//! Set the delegate to call on pointer button down
		//! Delegate should return to accept the default action.
		void SetDelegateOnPointerButtonDown (
			DELEGATE_ONPOINTER delegate
			) { m_DelegateOnPointerButtonDown = delegate; }

		//! Set the delegate to call on pointer button up
		//! Delegate should return to accept the default action.
		void SetDelegateOnPointerButtonUp (
			DELEGATE_ONPOINTER delegate
			) { m_DelegateOnPointerButtonUp = delegate; }

		void SetDelegateOnPreInsertItem (DELEGATE_ONPREINSERTITEM);

		//! Delegate called to obtain info for item stored externally to control.
		//! Only applies to text value and only if CTRLSTYLE_EditableText 
		//! specified when control created
		void SetDelegateOnSetItemInfo (DELEGATE_ONSETITEMINFO);

		//! Set delegate to call when control resized.
		void SetDelegateOnSize (DELEGATE_ONSIZE delegate);

		//! Set the delegate for tooltip requests
		void SetDelegateOnToolTipRequest (DELEGATE_ONTOOLTIPREQUEST);

		//! Set primary or 'state' image list for control.
		//! The 'state' image list may contain at most 16 entries, and the first entry is unused.
		void SetImageList (
			IMAGELIST& ImageList,				//!< Image list
			IMAGETYPE ImageType					//!< Image type
			);

		//! Set item to be bold or not.
		void SetItemBold (
			ITEMHND hndItem,						//!< Item handle
			bool bold								//!< True to make bold, false for normal
			);

		//! Set index for "normal" image.
		void SetItemImageNormal (
			ITEMHND hndItem,						//!< Item handle
			int IdxNormal							//!< Image index, -1 for none
			);

		//! Set item image for state.
		void SetItemImageState (
			ITEMHND hndItem,						//!< Item handle
			unsigned int IdxState				//!< Index in 'state' image list (1-15) or 0 for none
			);

		//! Set item information for specified ITEMHND.
		void SetItemInfo (
			ITEMHND hndItem,						//!< Item handle
			const ITEMINFO& ItemInfo			//!< Item information to set
			);

		//! Set item information using handle from ITEMINFO.
		void SetItemInfo (
			const ITEMINFO& ItemInfo			//!< Item information to set
			) { SetItemInfo(ItemInfo.GetHandle(),ItemInfo); }

		//! Set item position in list.
		void SetItemPosition (
			ITEMHND hndItem,						//!< Item handle
			ITEMHND hndInsertAfter				//!< Handle for item to insert after
			);

		//! Set text for item.
		void SetItemText (
			ITEMHND hndItem,						//!< Item handle
			const MISTRING& text					//!< Text string to set
			);

		//! Set color for drawing lines linking children to parent.
		void SetLineColor (
			const COLOR& color					//!< Color to set
			);

		//! Set selected item.
		void SetSelectedItem (
			ITEMHND hndItem						//!< Item handle
			);

	protected:

		//! Determine location of point relative to tree control.
		//! @return Handle for item point is on or ITEMHND_None if none.
		ITEMHND HitTest (
			const LPOINT2D& point,					//!< Point to test
			HITRESULT *pHitResult = 0				//!< Result of test, NULL if not needed
			) const;

		//! Called when item position changed.
		//! Called after position changed before redrawing.
		//! Default behavior is to call delegate.
		virtual void v_OnItemChangePosition (
			ITEMHND hndItem							//!< Item handle
			);

		//! Called when left mouse button pressed.
		//! Default behavior is to 'select' item if press anywhere in item.
		virtual void v_OnLeftButtonDown (
			ITEMHND hndItem,							//!< Item button event occurred over
			HITRESULT HitResult,						//!< Location where event occurred
			const LPOINT2D& point,					//!< Point where event occurred
			KEYSTATE keystate							//!< Modifier key state at time of event
			);

		//! Called when left mouse button released.
		//! Default behavior supports visibility toggle, text editing, drag and drop, etc.
		virtual void v_OnLeftButtonUp (
			ITEMHND hndItem,							//!< Item button event occurred over
			HITRESULT HitResult,						//!< Location where event occurred
			const LPOINT2D& point,					//!< Point where event occurred
			KEYSTATE keystate							//!< Modifier key state at time of event
			);

		//! Called when pointer button pressed.
		//! Default behavior is to 'select' item if press anywhere in item.
		virtual void v_OnPointerButtonDown (
			ITEMHND hndItem,							//!< Item button event occurred over
			HITRESULT HitResult,						//!< Location where event occurred
			const LPOINT2D& point,					//!< Point where event occurred
			POINTERBUTTON button,					//!< Button that was pressed
			KEYSTATE keystate							//!< Modifier key state at time of event
			);

		//! Called when pointer button released.
		//! Default behavior supports visibility toggle, text editing, drag and drop, etc.
		virtual void v_OnPointerButtonUp (
			ITEMHND hndItem,							//!< Item button event occurred over
			HITRESULT HitResult,						//!< Location where event occurred
			const LPOINT2D& point,					//!< Point where event occurred
			POINTERBUTTON button,					//!< Button that was released
			KEYSTATE keystate							//!< Modifier key state at time of event
			);

		//! Called when right mouse button pressed.
		//! Default behavior is to 'select' item if press anywhere in item.
		virtual void v_OnRightButtonDown (
			ITEMHND hndItem,							//!< Item button event occurred over
			HITRESULT HitResult,						//!< Location where event occurred
			const LPOINT2D& point,					//!< Point where event occurred
			KEYSTATE keystate							//!< Modifier key state at time of event
			);

		//! Called when right mouse button released.
		//! Default behavior is to toggle expanded/collapsed if press/release on appropriate button.
		virtual void v_OnRightButtonUp (
			ITEMHND hndItem,							//!< Item button event occurred over
			HITRESULT HitResult,						//!< Location where event occurred
			const LPOINT2D& point,					//!< Point where event occurred
			KEYSTATE keystate							//!< Modifier key state at time of event
			);

		//! Called to obtain ToolTip.
		//! Will not be called if CTRLSTYLE_NoToolTips specified in creation.
		//! Default implementation sets ToolTip text to match item text for HITRESULT_OnItem and return true,
		//! otherwise returns false.
		//! @return True to show tooltip, false to not and use ToolTipCtrl methods to set text or graphic for tooltip.
		virtual bool v_OnToolTipRequest (
			ITEMHND hndItem,							//!< Item cursor was over
			HITRESULT HitResult,						//!< Location where event occurred
			const LPOINT2D& point,					//!< Point where event occurred
			MGUI::CTRL_TOOLTIP& ToolTipCtrl		//!< ToolTip control
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		PRIV *m_pPriv;
		DELEGATE_ONPOINTER m_DelegateOnPointerButtonDown;
		DELEGATE_ONPOINTER m_DelegateOnPointerButtonUp;
		DELEGATE_ONITEM m_DelegateOnItemChangePosition;
		DELEGATE_ONITEM m_DelegateOnItemDeselected;
		DELEGATE_ONITEM m_DelegateOnItemSelected;

		void NotifyPointerButtonPress (ITEMHND hnditem, HITRESULT HitResult, const LPOINT2D& point, POINTERBUTTON button, KEYSTATE keystate);
		void NotifyPointerButtonRelease (ITEMHND hnditem, HITRESULT HitResult, const LPOINT2D& point, POINTERBUTTON button, KEYSTATE keystate);

		CTRL_TREE (const CTRL_TREE&);
		CTRL_TREE& operator= (const CTRL_TREE&);
		#endif // GENERATING_DOXYGEN_OUTPUT

		// Overridables not callable from derived classes.

		//! Called when child visibility has changed.
		virtual void v_OnChildVisibilityChanged (
			ITEMHND hndItem,							//!< Handle to item
			CHILDVISIBILITY visibility				//!< New child visibility
			);

		//! Called when child visibility is about to change.
		//! Return 'false' to prevent action from occurring.
		//! Must be overridden to create child items if item has CHILDPRESENCE_Ask.
		virtual bool v_OnChildVisibilityChanging (
			ITEMHND hndItem,							//!< Handle to item
			CHILDVISIBILITY visibility				//!< New child visibility
			);

		//! Called at end of create.
		virtual void v_OnCreateEnd (
			);

		//! Called to perform custom drawing.
		//! Default implementation simply returns RETVALUE_DoDefault.
		virtual MGUI::CUSTOMDRAW::RETVALUE v_OnCustomDraw (
			CUSTOMDRAW_TREE& CustomDraw			//!< Custom drawing parameters
			);

		//! Called when item about to be destroyed.
		virtual void v_OnDestroyItem (
			ITEMHND hndItem							//!< Item handle
			);

		//! Called to obtain info for item stored externally to control.
		virtual void v_OnGetItemInfo (
			ITEMVALUES RequestedValue,				//!< ITEMVALUE_ChildPresence, ITEMVALUE_ImageNormal, ITEMVALUE_ImageSelected or ITEMVALUE_Text
			ITEMINFO& ItemInfo						//!< Item information passed/returned
			);

		//! Called when item becomes deselected.
		//! Called after status changed before redrawing item.
		virtual void v_OnItemDeselected (
			ITEMHND hndItem							//!< Item handle
			);

		//! Called when item becomes selected.
		//! Called after status changed before redrawing item.
		virtual void v_OnItemSelected (
			ITEMHND hndItem							//!< Item handle
			);

		//! Called before item created.
		virtual void v_OnPreInsertItem (
			ITEMINFO& ItemInfo						//!< Item information passed/returned
			);

		//! Called when change made to value stored external to control.
		//! Only applies to text value and only if CTRLSTYLE_EditableText specified when control created.
		virtual void v_OnSetItemInfo (
			ITEMVALUES ChangedValue,				//!< ITEMVALUE_Text
			const ITEMINFO& ItemInfo				//!< Item information passed/returned
			);

	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(CTRL_TREE::CTRLSTYLE)
DEFINE_ENUM_OP_BITWISE(CTRL_TREE::ITEMSTYLE)
DEFINE_ENUM_OP_BITWISE(CTRL_TREE::ITEMSTATUS)
DEFINE_ENUM_OP_BITWISE(CTRL_TREE::ITEMVALUES)
#endif

//! Custom drawing parameters for CTRL_TREE.
class CUSTOMDRAW_TREE : public MGUI::CUSTOMDRAW {
	public:
		//! Default constructor.
		CUSTOMDRAW_TREE ();
		//! Destructor.
		~CUSTOMDRAW_TREE ();
		//! Get handle for item being drawn if applicable.
		CTRL_TREE::ITEMHND GetItemHnd (
			) const;
		//! Get background pixel value.
		MGD::PIXEL GetBackgroundPixel (
			) const { return (m_PixelBG); }
		//! Get level (depth) of item being drawn.
		//! @return Level of item, root is level 0.
		int GetLevel (
			) const { return (m_level); }
		//! Get text color.
		MGD::PIXEL GetTextPixel (
			) const { return (m_PixelText); }
		//! Set background color.
		void SetBackgroundColor (
			const COLOR& color				//!< Color to set
			);
		//! Set background pixel value.
		void SetBackgroundPixel (
			MGD::PIXEL pixel					//!< Pixel value to set
			) { m_PixelBG = pixel; }
		//! Set text color.
		void SetTextColor (
			const COLOR& color				//!< Color to set
			);
		//! Set text pixel value.
		void SetTextPixel (
			MGD::PIXEL pixel					//!< Pixel value to set
			) { m_PixelText = pixel; }
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MGD::PIXEL m_PixelBG;
		MGD::PIXEL m_PixelText;
		int m_level;
		CUSTOMDRAW_TREE (
			DRAWSTAGE drawstage,
			const LRECT2D& rect
			);
		void SetupItem (
			const CTRL_TREE::ITEMINFO& iteminfo,
			const LRECT2D& rect,
			int level,
			MGD::CONTEXT *pDC
			);
		friend class CTRL_TREE::PRIV;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}	// End namespace MGUI

#endif
