/****
 *	\file filefind.h <mi32/filefind.h>
 *	\brief Class FILEFIND - Methods to locate files
 *
 * Path searching management class, replaces MioSearchList...() functions
 *
 *	1.  This class is not thread safe.  Most of the time the instance is specific to a 
 *		thread therefore the recouse sharing issues are not involved.  If this class is
 *		a shared resource, it is up to the user of the class to guard it using the 
 *		appropriate semaphore method.
 *
 *	\if NODOC
 *	$Id: filefind.h_v 1.45 2012/05/09 21:04:14 mju Exp $
 *
 * $Log: filefind.h_v $
 * Revision 1.45  2012/05/09 21:04:14  mju
 * Add some private methods, remove some long-obsolete ones.
 *
 * Revision 1.44  2010/03/23 15:26:35  mju
 * Rename createFile as createPath so not changed by windows macro.
 *
 * Revision 1.42  2008/07/24 21:15:21  mju
 * Add location_installation and add to locations_reference and locations_executable.
 *
 * Revision 1.41  2007/10/09 21:46:08  dwilliss
 * Added optional extension parameter to make temp file
 *
 * Revision 1.40  2007/05/10 12:44:13  mju
 * Fwd-declare filepath/list so don't need header.
 *
 * Revision 1.38  2007/01/24 23:44:03  dwilliss
 * Added static method to GetDefaultTempFolder
 *
 * Revision 1.36  2007/01/03 22:56:40  mju
 * Add locations_Temporary so can setup in ctor.
 * Add needWritable parm to addEnv/Path/String.
 *
 * Revision 1.34  2006/10/05 17:39:27  mju
 * Add registerMountDelegate.
 *
 * Revision 1.33  2006/10/05 14:59:37  mju
 * Add missingFilePromptDisable/Enable and missingfilepromptdisabler class.
 *
 * Revision 1.31  2006/10/05 12:57:14  mju
 * Add clearMissingFileList.
 *
 * Revision 1.30  2006/10/04 21:49:29  mju
 * Add locateRelatedPath and relatedorder enum.
 * Add v_prompt to filter.
 *
 * Revision 1.18  2003/05/22 21:28:18  mju
 * MakeSettings now requires one or more of listed locations to be specified.
 *
 * Revision 1.15  2003/05/19 16:51:06  mju
 * In LocateSettingsFile allow to specifed legacy locations separately.
 *
 * Revision 1.13  2003/05/16 17:50:29  mju
 * Add LOCATION_Settings and Locate/MakeSettingsFile methods.
 *
 * Revision 1.10  2003/04/10 14:06:27  mju
 * Search locale folders first if LOCATION_Locale specified.
 *
 * Revision 1.7  2003/02/21 20:53:06  mju
 * Add AddLocale method.
 * Expand and clarify documentation on search order.
 *
 * Revision 1.2  2002/01/09 13:48:35  mju
 * Add default parm for LocateReferenceFile.
 *
 * Revision 1.1  2001/12/06 17:43:44  scowan
 * Initial revision
 * \endif
****/
 
#ifndef	INC_MI32_FILEFIND_H
#define	INC_MI32_FILEFIND_H

#ifndef	INC_MI32_DELEGATE_H
	#include <mi32/delegate.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class FILEPATH;
class FILEPATHLIST;
#endif


//! Class used to search for files.
class CLASSLIBEXPORT FILEFIND {
	public:

		//! Placement of path in the search list
		enum ADDPOSN {
			ADDPOSN_ToFront,
			ADDPOSN_ToBack
			};

		enum LOCATIONS {
			LOCATION_None =								0x0000,
			LOCATION_Locale = 							0x0001,	//!< Active and default locale folders, if any, in that order
			LOCATION_Home =								0x0002,	//!< Home folder, if any
			LOCATION_Current =							0x0004,	//!< Current working directory
			LOCATION_Process =							0x0008,	//!< Folder containing process executable
			LOCATION_PATH =								0x0010,	//!< PATH environment variable
			LOCATION_Installation =						0x0020,	//!< Installation folder
			LOCATION_Settings_CurInst_CurUser =		0x0100,	//!< Settings for current 'installation', current user
			LOCATION_Settings_CurInst_AllUsers =	0x0200,	//!< Settings for current 'installation', all users
			LOCATION_Settings_AllInst_CurUser =		0x0400,	//!< Settings for all 'installations', current user
			LOCATION_Settings_AllInst_AllUsers =	0x0800,	//!< Settings for all 'installations', all users
			LOCATIONS_Executable =	(LOCATION_Installation | LOCATION_Process | LOCATION_Current | LOCATION_PATH),	//! Executable file locations
			LOCATIONS_Reference =	(LOCATION_Installation | LOCATION_Process | LOCATION_Current),	//! Reference file locations
			LOCATIONS_Temporary =						0x1000,	//!< Temporary locations
			#ifndef GENERATING_DOXYGEN_OUTPUT
			#ifdef MISYSTEMDLL
			LOCATIONS_Settings_Mask =					0x0F00,
			#endif
			#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Searching order for LocateRelatedPath.
		//! Search order consists of up to 4 components.
		//!  "Source" indicates the folder referred to by the SourcePath.
		//!  "Original" indicates the original value of RelatedPath.
		//!  "Media" replaces the media/filesystem component(s) of RelatedPath with those from SourcePath.
		//!  "Tree" searches all subfolders of the SourcePath.
		enum RELATEDORDER {
			#ifndef GENERATING_DOXYGEN_OUTPUT
			SUBORDER_Original =	0x01,
			SUBORDER_Media =		0x02,
			SUBORDER_Source =		0x03,
			SUBORDER_Tree =		0x04,
			SUBORDER_Mask =		0x0F,
			#endif
			RELATEDORDER_SourceOriginalMediaTree = ((SUBORDER_Source) | (SUBORDER_Original << 4) | (SUBORDER_Media << 8) | (SUBORDER_Tree << 12)),
			RELATEDORDER_OriginalSourceMediaTree = ((SUBORDER_Original) | (SUBORDER_Source << 4) | (SUBORDER_Media << 8) | (SUBORDER_Tree << 12)),
			RELATEDORDER_SourceMediaOriginalTree = ((SUBORDER_Source) | (SUBORDER_Media << 4) | (SUBORDER_Original << 8) | (SUBORDER_Tree << 12)),
			RELATEDORDER_MediaSourceOriginalTree = ((SUBORDER_Media) | (SUBORDER_Source << 4) | (SUBORDER_Original << 8) | (SUBORDER_Tree << 12)),
			RELATEDORDER_MediaTreeSourceOriginal = ((SUBORDER_Media) | (SUBORDER_Tree << 4) | (SUBORDER_Source << 8) | (SUBORDER_Original << 12)),
			RELATEDORDER_NoPrompt =	0x80000000	//!< Don't prompt user for file if not found
			};
			
		//! Base FILTER class used in LocatePath() and LocateRelatedFile() to select the correct path if multiple possible.
		class CLASSLIBEXPORT FILTER {
			public:

				//! Determine if path suitable.
				virtual bool v_Filter (
					const FILEPATH& path
					) = 0;

				//! Prompt user for path.
				//! Default implementation calls the delegate specified in RegisterPromptDelegate(), if any.
				//! @return true if path found/located, false if not.
				virtual bool v_Prompt (
					FILEPATH& path,							//!< Path being prompted for
					const MISTRING *pPromptStr				//!< Optional pointer to "prompt" string, NULL for default
					);
	 		};

		//! Class to automatically disable prompt for missing file on
		//! creation and enable prompt on destruction.
		class MISSINGFILEPROMPTDISABLER {
			public:
				//! Constructor - Disables missing file prompting.
				MISSINGFILEPROMPTDISABLER (
					) { FILEFIND::MissingFilePromptDisable(); }
				//! Destructor - Enables missing file prompting.
				~MISSINGFILEPROMPTDISABLER (
					) { FILEFIND::MissingFilePromptEnable(); }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				MISSINGFILEPROMPTDISABLER (const MISSINGFILEPROMPTDISABLER&);
				MISSINGFILEPROMPTDISABLER& operator= (const MISSINGFILEPROMPTDISABLER&);
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Delegate to prompt user for file.
		//! @return true if path found/located, false if not.
		typedef FastDelegate<bool(FILEPATH&, FILEFIND::FILTER*, const MISTRING*)> DELEGATE_PROMPT;

		//! Delegate to mount file.
		typedef FastDelegate<void(FILEPATH&)> DELEGATE_MOUNT;

		//! Clear "missing file" list so next use of LocateRelatedFile may prompt user.
		static void ClearMissingFileList ();

		//! Get the default temporay folder location.
		//! This is currently only useful on Macintosh, where we put a
		//! "TNT Temporary Files" folder in the install folder along with
		//! the start icons.  This function will return a path to that folder
		//! and optionally create it if missing.
		static ERRVALUE GetDefaultTempFolder (
			FILEPATH& Folder,
			bool bCreateIfMissing
			);

		//! Locate an executable file.
		//! Searches LOCATIONS_Executable folders in the following order:
		//!		1. Current working directory.
		//!		2. Process executable path.
		//!		3. PATH environment variable.
		//! @return 'True' if file located
		static bool LocateExecutableFile (
			const char* ExecutableFileName,				//!< Name of file to search for
			FILEPATH& ExecutableFilePath					//!< Executable file RETURNED
			);

		//! Locate a reference file using char* name.
		//! Searches folders in the following order:
		//!		1. locale/??? (Current locale) folder if LOCATION_Locale specified and not using 'enu' locale.
		//!		2. locale/enu (English) folder if LOCATION_Locale specified.
		//!		3. Current working directory.
		//!		4. Process executable path.
		//!		5. Product installation folder
		//!		6. Home path if LOCATION_Home specified.
		//! @return 'True' if file located
		static bool LocateReferenceFile (
			const char* ReferenceFileName,				//!< Name of file to search for
			FILEPATH& ReferenceFilePath,					//!< Reference file RETURNED
			LOCATIONS locations = LOCATION_None			//!< Locations to search in addition to LOCATIONS_Reference
			);

		//! Locate a reference file using MISTRING name.
		//! Searches folders in the following order:
		//!		1. locale/??? (Current locale) folder if LOCATION_Locale specified and not using 'enu' locale.
		//!		2. locale/enu (English) folder if LOCATION_Locale specified.
		//!		3. Current working directory.
		//!		4. Process executable path.
		//!		5. Product installation folder
		//!		6. Home path if LOCATION_Home specified.
		//! @return 'True' if file located
		static bool LocateReferenceFile (
			const MISTRING& ReferenceFileName,			//!< Name of file to search for
			FILEPATH& ReferenceFilePath,					//!< Reference file RETURNED
			LOCATIONS locations = LOCATION_None			//!< Locations to search in addition to LOCATIONS_Reference
			);

		//! Locate related file from another file.
		//! @return 'true' if file located, false if not
		static bool LocateRelatedFile (
			const FILEPATH& SourcePath,					//!< Path to source file or starting folder
			FILEPATH& RelatedPath,							//!< Related file to search for passed/returned
			RELATEDORDER order,								//!< Folder search order
			FILTER *pFilter = 0								//!< Optional filter
			);

		//! Locate a 'settings' file using char* name.
		//! Searches specified locations in the following order:
		//!		1. Settings_CurInst_CurUser (if specified or no other listed locations specified)
		//!		2. Settings_CurInst_AllUsers
		//!		3. Settings_AllInst_CurUser
		//!		4. Settings_AllInst_AllUsers
		//!		5. Current.
		//!		6. Process.
		//!	In most cases only a single 'preferred' location should be specified.
		//! If the file is not found in the preferred location and any 'legacy' locations are
		//! specified, they will be searched, and if the file is found there, it will be copied
		//! to the preferred location.
		//! @return 'True' if file located
		static bool LocateSettingsFile (
			const char* SettingsFileName,					//!< Name of file to search for
			FILEPATH& SettingsFilePath,					//!< Settings file RETURNED
			LOCATIONS PreferredLocation,					//!< Preferred location(s) to search
			LOCATIONS LegacyLocations = LOCATION_None	//!< Legacy locations to search
			);
			
		//! Locate the available truetype font files on the current platform
		static ERRVALUE LocateTrueTypeFontFiles (
			FILEPATHLIST& TrueTypeFiles					//!< Locations of the truetype files on the platform RETURNED
			);

		//! Make 'settings' file path in first available location specified.
		//! The first writable location NOT already containing a file of the specified name
		//! will be used to generate the file path.  One or more of the following locations
		//! must be specified, which are tried in the following order:
		//!		1. Settings_CurInst_CurUser
		//!		2. Settings_CurInst_AllUsers
		//!		3. Settings_AllInst_CurUser
		//!		4. Settings_AllInst_AllUsers
		//!		5. Current
		//!		6. Process
		//!	In most cases only a single specific location should be specified
		//! Does not actually create the file itself.
		static ERRVALUE MakeSettingsFilePath (
			const char* SettingsFileName,					//!< Name of file
			FILEPATH& SettingsFilePath,					//!< New reference file location RETURNED
			LOCATIONS locations								//!< Locations to search, must be one or more listed above
			);

		//! Make a temporary file.
		//! An empty unopened file will be created using the returned path.
		static ERRVALUE MakeTempFile (
			FILEPATH& TempFilePath,							//!< Path of created temp file
			INT64 MinSize,										//!< Minimum file size for the temp file
			bool LargestPath,									//!< Select largest path to create temp file in
			const MISTRING& basename,						//!< Basename for the temp file
			const char *extn = "tmp"						//!< Default extension to use (no dot)
			);

		//! Make a temporary file using default 'temp' base name.
		//! An empty unopened file will be created using the returned path.
		static ERRVALUE MakeTempFile (
			FILEPATH& TempFilePath,							//!< Path of created temp file
			INT64 MinSize,										//!< Minimum file size for the temp file
			bool LargestPath									//!< Select largest path to create temp file in
			);

		//! Disable prompt for missing file.
		//! Increments count, match with MissingFilePromptEnable().
		static void MissingFilePromptDisable ();

		//! Enable prompt for missing file.
		//! Decrements count.
		static void MissingFilePromptEnable ();

		//! Register delegate to call to request mount of file in LocateRelatedFile().
		static void RegisterMountDelegate (
			DELEGATE_MOUNT delegate						//!< Delegate to call
			);

		//! Register delegate to call when file can't be located in LocateRelatedFile().
		static void RegisterPromptDelegate (
			DELEGATE_PROMPT delegate						//!< Delegate to call
			);

		//! Construct with specified locations in standard order.
		//! Specified locations will occur in the list in the following order.
		//!		1. Locale
		//!		2. Current
		//!		3. Process
		//!		4. PATH environment
		//!		5. Settings_CurInst_CurUser
		//!		6. Settings_AllInst_CurUser
		//!		7. Settings_CurInst_AllUsers
		//!		8. Settings_AllInst_AllUsers
		//!		9. Home
		//!	Note that this order may be different than the search order used in the static Locate... methods.
		explicit FILEFIND (
			LOCATIONS locations = LOCATION_None			//!< Locations to add to list
			);

		//! Copy constructor
		FILEFIND (
			const FILEFIND& rhs
			);

		//! Destructor
		~FILEFIND (
			);

		//! Assignment
		FILEFIND& operator= (
			const FILEFIND& rhs
			);
			
		//! Add environment variable items to the search list, uses 
		//! system environment separator for multiple paths
		void AddEnvVar (
			const char* EnvTag,
			ADDPOSN AddPosn = ADDPOSN_ToBack,
			bool NeedWritable = false
			);

		//! Add locale paths to search list.
		//! Regardless of position, the active locale will be searched before the default (English) locale.
		void AddLocale (
			bool IncludeDefault = true,				//!< Include default (enu) locale
			ADDPOSN AddPosn = ADDPOSN_ToBack
			);

		//! Add a path to the search list
		void AddPath (
			const FILEPATH& path,
			ADDPOSN AddPosn = ADDPOSN_ToBack,
			bool NeedWritable = false
			);

		//! Add multiple separated strings to the search list
		void AddString (
			const MISTRING& mpath,
			ADDPOSN AddPosn = ADDPOSN_ToBack,
			bool NeedWritable = false,
			MIUNICODE Separator = 0xFFFF
			);

		//! Create file path using the search list.
		//! The first writable location in the list NOT already containing a file
		//! of the specified name will be used to generate the file path.
		//! @return 'True' if the file was created, 'false' if not
		bool CreatePath (							
			const MISTRING& name,
			FILEPATH& path									//!< FILEPATH set to made file
			) const;

		//! Create file path using the search list.
		//! The first writable location in the list NOT already containing a file
		//! of the specified name will be used to generate the file path.
		//! @return 'True' if the file was created, 'false' if not
		bool CreatePath (							
			const char* name,
			FILEPATH& path									//!< FILEPATH set to made file
			) const;

		//! Get list of paths to search.
		const FILEPATHLIST& GetPathList (
			) const { return (*m_pPathList); }
			
		//! Does the search list have any entries
		//! @return 'True' if no entries in the list
		bool IsEmpty (
			) const;

		//! Locate file using search list
		//! @return 'True' if the path was found, 'false' if not
		bool LocateFile (							
			const char* name,								//!< Name of file to locate
			FILEPATH& path									//!< Set to path if found
			) const;

		//! Locate file using search list and a possible relative path
		//! @return 'True' if the path was found, 'false' if not
		bool LocateFile (
			const MISTRING& relfile,					//!< Relative path to locate
			FILEPATH& path									//!< Set to path if found
			) const;

		//! Locate files using search list and pattern
		//! @return 'True' if the path was found, 'false' if not
		bool LocateFiles (
			const MISTRING& pattern,
			FILEPATHLIST& PathList						//!< List of files found
			) const;

		//! Locate path using filter
		//! @return 'True' if the path was found, 'false' if not
		bool LocatePath (
			FILTER& filter,
			FILEPATH& path								//!< Set to path if found
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		FILEPATHLIST *m_pPathList;				// The list of FILEPATH's to search
		static const MIUNICODE s_SepMark;
		static FILEPATH& GetLocaleDir();
		static DELEGATE_MOUNT s_DelegateMount;
		static DELEGATE_PROMPT s_DelegatePrompt;
		static UINT32 s_PromptDisabledCount;

		void AddTempEnvVar (const char *EnvTag);

		bool CreatePathLow (
			const MISTRING& name,
			FILEPATH& path,
			MISTRING& errdetails
			) const;

		bool CreatePathLow (
			const char *name,
			FILEPATH& path,
			MISTRING& errdetails
			) const;

		void MoveOldTemp (
			const FILEPATH& FolderPathNew,
			const FILEPATH& FolderPathOld,
			const char *name
			);

		friend class FILEFIND::FILTER;

		#endif // GENERATING_DOXYGEN_OUTPUT
		
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FILEFIND::LOCATIONS);
DEFINE_ENUM_OP_BITWISE(FILEFIND::RELATEDORDER);
#endif

#undef	CLASSLIBEXPORT
#endif 	// INC_MI32_FILEFIND_H
