/**
 * \file genericrw.h <mi32/genericrw.h>
 * \brief GENERICRW class for simple file-like read/write interface.
 *
 * \if NODOC
 * $Id: genericrw.h_v 1.14 2011/11/14 17:22:26 mju Exp $
 *
 * $Log: genericrw.h_v $
 * Revision 1.14  2011/11/14 17:22:26  mju
 * Add readAll.
 *
 * Revision 1.13  2011/10/25 17:52:22  mju
 * Remove redundant hdr.
 *
 * Revision 1.12  2006/07/07 22:49:32  scowan
 * Make dtor virtual.
 *
 * Revision 1.11  2005/10/25 21:22:37  dwilliss
 * document the fact that Read and SeekAndRead return number of bytes read
 *
 * Revision 1.10  2005/07/20 20:10:55  dfriberg
 * Added mapped memory methods
 *
 * Revision 1.7  2003/12/31 22:07:47  dwilliss
 * Added a new capability flag to indicate that we can determine a size for
 * whatever it is we have.
 *
 * Revision 1.6  2003/12/22 17:58:57  dwilliss
 * Added some new capability flags for sockets
 *
 * Revision 1.1  2003/07/10 19:28:11  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_GENERICRW_H
#define	INC_MI32_GENERICRW_H

#ifndef	INC_MI32_SIMPLEAR_H
	#include <mi32/simplear.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

//===================================================================================================================

//! Base class for generic read/write data source/target.
// This class is exported from DLL.
// Adding virtual methods will require complete rebuild of all dependent DLLs and EXEs.
class CLASSLIBEXPORT GENERICRW {
	public:

		//! Enumeration for where to seek from.
		enum SEEKFROM {
			SEEKFROM_Beginning = 0,
			SEEKFROM_Current,
			SEEKFROM_End
			};

		//! Enumeration for instance capabilities.
		enum CAPABILITIES {
			CAPABILITY_None =				0x0000,
			CAPABILITY_Read =				0x0001,
			CAPABILITY_Write =			0x0002,
			CAPABILITY_Seek =				0x0004,
			CAPABILITY_Resize =			0x0008,
			CAPABILITY_Accept	=			0x0010,		//!< Can accept incoming connections (Sockets implement this)
			CAPABILITY_Bind	=			0x0020,		//!< Can bind to a specific port (Sockes implement this)
			CAPABILITY_NonBlocking	=	0x0040,		//!< Can be set to have read/write return immediately if not ready.
			CAPABILITY_Size	=			0x0080,		//!< We can find the size
			CAPABILITIES_All =			0x00FF
			};

		//! Destructor.
		virtual ~GENERICRW ();
			
		//! Get instance capabilities.
		//! @return Capabilities
		CAPABILITIES GetCapabilities (
			) const;

		//! Get current size of data.
		//! Note, if instance does not support seeking then size returned may be less than ultimate
		//! amount of data which may be read.
		//! @return Current size or error < 0
		INT64 GetSize (
			) const;

		//! Get maximum possible size of data.
		//! @return Maximum possible size in bytes.
		INT64 GetSizeLimit (
			) const;

		//! Get memory-mapped buffer containing data.
		//! @return Pointer to mapped memory, or null if operation is not supported.
		void* GetMappedMemory (
			INT64& size,						//!< size of buffer returned
			INT64 offset = 0					//!< offset into data
			);

		//! Read data into buffer.
		//! @return Number of bytes read (may be less than requested), 0 if no more available at present, -1 if end of data, < -1 if error
		INT32 Read (
			void* buffer,								//!< Buffer to read into
			INT32 numbytes,							//!< Number of bytes to read
			bool AllowPartial = false				//!< Allow partial reads, if false will return error if less than requested number of bytes were read.
			);

		//! Read entire contents into buffer.
		ERRVALUE ReadAll (
			SIMPLE_ARRAY<UINT8>& buffer			//!< Buffer to hold contents
			);

		//! Resize data.
		//! May be used to truncate or extend the data, if used to extend, the contents
		//! beyond the old end of data and the new size are undefined.
		ERRVALUE Resize (
			INT64 newsize								//!< New size in bytes
			);

		//! Seek to specified location.
		//! @return Offset from beginning of data, < 0 if error
		INT64 Seek (
			INT64 offset,								//!< Location to seek to
			SEEKFROM from = SEEKFROM_Beginning	//!< Relative position to seek from
			);

		//! Seek to specified position and read.
		//! @return Number of bytes read (may be less than requested), 0 if no more available at present, -1 if end of data, < -1 if error
		ERRVALUE SeekAndRead (
			INT64 offset,								//!< Offset from beginning of data stream
			void* buffer,								//!< Buffer to contain data
			INT32 numbytes								//!< Number of bytes to read
			);

		//! Seek to specified position and write.
		ERRVALUE SeekAndWrite (
			INT64 offset,								//!< Offset from beginning of data stream
			const void* buffer,						//!< Buffer containing data to write
			INT32 numbytes								//!< Number of bytes to write
			);

		//! Return current read/write position.
		//! @return Current offset from start of data or error < 0.
		INT64 Tell (
			) const;

		//! Write data from buffer.
		ERRVALUE Write (
			const void* buffer,						//!< Buffer to write from
			INT32 numbytes								//!< Number of bytes to write
			);

	private:
		// Overridables.

		//! Override to get instance capabilities.
		virtual CAPABILITIES v_GetCapabilities () const = 0;

		//! Override to get mapped memory.
		virtual void* v_GetMappedMemory (INT64& size, INT64 offset);

		//! Override to get data size.
		virtual INT64 v_GetSize () const = 0;

		//! Override to get maximum possible data size.
		virtual INT64 v_GetSizeLimit () const = 0;

		//! Override to read data.
		//! @return Number of bytes read (may be less than requested), 0 if no more available at present, -1 if end of data, < -1 if error
		virtual INT32 v_Read (void* buffer, INT32 numbytes, bool AllowPartial) = 0;

		//! Override to resize.
		virtual ERRVALUE v_Resize (INT64 newsize) = 0;

		//! Override to seek.
		virtual INT64 v_Seek (INT64 offset, SEEKFROM from) = 0;

		//! Override to return current position.
		virtual INT64 v_Tell () const = 0;

		//! Override to write data
		virtual ERRVALUE v_Write (const void* buffer, INT32 numbytes) = 0;
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(GENERICRW::CAPABILITIES)
#endif

//===================================================================================================================

#undef	CLASSLIBEXPORT

#endif	// INC_MI32_GENERICRW_H
