/**
 * \file geocat.h <mi32/geocat.h>
 * \brief Geographic Catalog (GEOCAT) class
 *
 * \if NODOC
 * $Id: geocat.h_v 1.31 2011/10/13 12:47:58 mju Exp $
 *
 * $Log: geocat.h_v $
 * Revision 1.31  2011/10/13 12:47:58  mju
 * Remove redundant hdrs.
 *
 * Revision 1.30  2009/09/29 19:07:13  bcolombe
 * Use separate scan log for last file
 *
 * Revision 1.29  2009/07/15 21:03:09  bcolombe
 * Add changes for scanner program
 *
 * Revision 1.28  2009/05/15 14:39:52  bcolombe
 * Added BuildIndex
 *
 * Revision 1.27  2009/05/06 23:03:54  bcolombe
 * Added bool m_HasError
 *
 * Revision 1.26  2009/04/29 19:34:52  bcolombe
 * Changes for media table
 *
 * Revision 1.25  2009/04/20 20:42:35  bcolombe
 * Add separate table for folders to exclude from all catalogs
 *
 * Revision 1.24  2009/04/17 21:14:57  bcolombe
 * Added m_HasNewFields
 *
 * Revision 1.23  2009/04/15 23:21:50  bcolombe
 * Add option on open for write or read open
 *
 * Revision 1.22  2009/04/15 20:09:46  bcolombe
 * Geocatalog rewrite
 *
 * Revision 1.21  2008/08/25 13:04:37  mju
 * Remove rvcdbase hdr.
 *
 * Revision 1.20  2007/04/03 12:59:37  mju
 * Fix comment, remove redundant miunicode methods.
 *
 * Revision 1.19  2007/02/20 16:54:23  mju
 * Can't copy recorditerator.
 *
 * Revision 1.18  2007/02/20 16:42:02  mju
 * Declare copy ctor/op= for iterator.
 *
 * Revision 1.17  2007/02/20 15:03:00  mju
 * Use rvc classes.
 *
 * Revision 1.16  2006/09/28 19:41:48  scowan
 * Added forward decl of objitem.
 *
 * Revision 1.15  2005/03/31 16:57:11  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.14  2004/06/30 13:31:09  mju
 * Use spatref.
 *
 * Revision 1.13  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.12  2000/10/13 14:57:47  dwilliss
 * Must use "friend class NAME", not "friend NAME"
 *
 * Revision 1.11  2000/09/20 17:47:51  mju
 * Add GetProjection() static method.
 *
 * Revision 1.10  2000/09/13 17:08:28  mju
 * Add "findnow" flag to GetObject().
 * Add iterator::showprogress().
 *
 * Revision 1.9  2000/08/16 16:48:02  mju
 * *** empty log message ***
 *
 * Revision 1.8  2000/08/07 20:35:29  mju
 * Redesign to always use regions.
 *
 * Revision 1.5  2000/07/19 22:05:06  dwilliss
 * Constructors don't need a GEOCAT passed any more.
 *
 * Revision 1.4  2000/07/19 17:38:56  dwilliss
 * ScanDirectory() now takes an optional StatusContext
 *
 * Revision 1.3  2000/07/19 16:22:08  dwilliss
 * Implementation mostly done.  API nearly final
 *
 * Revision 1.1  2000/07/17 14:16:42  dwilliss
 * Initial revision
 * \endif
**/

#ifndef INC_MI32_GEOCAT_H
#define INC_MI32_GEOCAT_H

#ifndef INC_RVC_DBTABLE_H
#include <rvc/dbtable.h>
#endif

#ifndef INC_MI32_FILEPATHLIST_H
#include <mi32/filepathlist.h>
#endif

#ifndef INC_MI32_REGION2D_H
#include <mi32/region2d.h>
#endif

#ifndef INC_MI32_FILETEXT_H
#include <mi32/filetext.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class STATUSCONTEXT;
namespace RVC {
	class OBJITEM;
	}
#endif


//! GeoSpatial Catalog class.
//!
//! This class handles a catalog file.
class GEOCAT {
	public:

		//! A record in the catalog.
		class RECORD {
			public:

				//! default constructor
				RECORD (
					);

				//! Copy constructor
				RECORD (
					const RECORD& rhs
					);

				//! Destructor
				~RECORD (
					);

				//! Assignment from another RECORD
				RECORD& operator= (
					const RECORD& rhs
					);

				//! Return ID
				UINT32 GetID (
					) const {
					return (m_ID);
					}

				//! Return accuracy
				double GetAccuracy (
					) const {
					return (m_Accuracy);
					}

				//! Return CRS
				const MISTRING& GetCRS (
					) const {
					return (m_CRS);
					}

				//! Return raster datatype
				const MISTRING& GetDataType (
					) const {
					return (m_DataType);
					}

				//! Return the description
				const MISTRING& GetDescription (
					) const {
					return (m_Description);
					}

				//! Return the format
				const MISTRING& GetFormat (
					) const {
					return (m_Format);
					}

				//! Get extents in latlon
				void GetLatLonExtents (
					DRECT2D& extents
					) const;

				//! Return north bounding latitude
				double GetLatitudeNorth (
					) const {
					return (m_LatitudeNorth);
					}

				//! Return south bounding latitude
				double GetLatitudeSouth (
					) const {
					return (m_LatitudeSouth);
					}

				//! Return the location
				const MISTRING& GetLocation (
					) const {
					return (m_Location);
					}

				//! Return east bounding longitude
				double GetLongitudeEast (
					) const {
					return (m_LongitudeEast);
					}

				//! Return west bounding longitude
				double GetLongitudeWest (
					) const {
					return (m_LongitudeWest);
					}

				//! Return raster number of columns
				UINT32 GetNumCols (
					) const {
					return (m_NumCols);
					}

				//! Return raster number of lines
				UINT32 GetNumLins (
					) const {
					return (m_NumLins);
					}

				//! Return the OBJTYPE of the object.
				RVC::OBJTYPE GetObjectType (
					) const {
					return (m_objtype);
				 	}

				//! Get extents in native crs
				void GetProjectionExtents (
					DRECT2D& extents
					) const;

				//! Return MinX bounding
				double GetProjectionMinX (
					) const {
					return (m_ProjectionMinX);
					}

				//! Return MaxX bounding
				double GetProjectionMaxX (
					) const {
					return (m_ProjectionMaxX);
					}

				//! Return MinY bounding
				double GetProjectionMinY (
					) const {
					return (m_ProjectionMinY);
					}

				//! Return MaxY bounding
				double GetProjectionMaxY (
					) const {
					return (m_ProjectionMaxY);
					}

				//! Return time modified
				const DATETIME& GetTimeObjModified (
					) const {
					return (m_TimeObjModified);
					}

				//! Return time modified
				const DATETIME& GetTimeRecCreated (
					) const {
					return (m_TimeRecCreated);
					}

				//! Return time modified
				const DATETIME& GetTimeRecModified (
					) const {
					return (m_TimeRecModified);
					}

				//! Return the title
				const MISTRING& GetTitle (
					) const {
					return (m_Title);
					}

				//! Return x scale (or raster cellsize)
				double GetXScale (
					) const {
					return (m_XScale);
					}

				//! Return y scale (or raster cellsize)
				double GetYScale (
					) const {
					return (m_YScale);
					}

				//! Set accuracy
				void SetAccuracy (
					double accuracy
					) { m_Accuracy = accuracy; }

				//! Set the CRS definition
				void SetCRS (
					const MISTRING& crs
					) { m_CRS = crs; }

				//! Set datatype
				void SetDataType (
					const MISTRING& datatype
					) { m_DataType = datatype; }

				//! Set the object description.
				void SetDescription (
					const MISTRING& desc
					) { m_Description = desc; }

				//! Set the format
				void SetFormat (
					const MISTRING& format
					) { m_Format = format; }

				//! Set north bounding latitude
				void SetLatitudeNorth (
					double val
					) { m_LatitudeNorth = val; }

				//! Set south bounding latitude
				void SetLatitudeSouth (
					double val
					) { m_LatitudeSouth = val; }

				//! Set the location
				void SetLocation (
					const MISTRING& location
					) { m_Location = location; }

				//! Set east bounding longitude
				void SetLongitudeEast (
					double val
					) { m_LongitudeEast = val; }

				//! Set west bounding longitude
				void SetLongitudeWest (
					double val
					) { m_LongitudeWest = val; }

				//! Set raster number of columns
				void SetNumCols (
					UINT32 NumCols
					) { m_NumCols = NumCols; }

				//! Set raster number of lines
				void SetNumLins (
					UINT32 NumLins
					) { m_NumLins = NumLins; }

				//! Set the OBJTYPE of the object
				void SetObjectType (
					RVC::OBJTYPE objtype
					) { m_objtype = objtype; }

				//! Set MinX bounding
				void SetProjectionMinX (
					double val
					) { m_ProjectionMinX = val; }

				//! Set MaxX bounding
				void SetProjectionMaxX (
					double val
					) { m_ProjectionMaxX = val; }

				//! Set MinY bounding
				void SetProjectionMinY (
					double val
					) { m_ProjectionMinY = val; }

				//! Set MaxY bounding
				void SetProjectionMaxY (
					double val
					) { m_ProjectionMaxY = val; }

				//! Set the title (displayed name)
				void SetTitle (
					const MISTRING& title
					) { m_Title = title; }

				//! Set x scale
				void SetXScale (
					double xscale
					) { m_XScale = xscale; }

				//! Set y scale
				void SetYScale (
					double yscale
					) { m_YScale = yscale; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				//! Used by GEOCAT::Write().
				void SetID (
					UINT32 id
					) {
					m_ID = id;
					return;
					}

				//! Set timestamp, used by GEOCAT::Write().
				void SetTimeObjModified (
					) {
					time_t modtime;
					RVC::OBJITEM objitem;
					objitem.SetCombinedPath(m_Location);
					objitem.GetFilePath().GetTimeModified(&modtime);
					m_TimeObjModified = modtime;
					return;
					}

				UINT32 m_ID;
				MISTRING m_Location;
				MISTRING m_Format;
				RVC::OBJTYPE m_objtype;
				MISTRING m_Title;
				MISTRING m_Description;
				DATETIME m_TimeRecCreated;
				DATETIME m_TimeRecModified;
				DATETIME m_TimeObjModified;
				double m_LongitudeWest;
				double m_LongitudeEast;
				double m_LatitudeSouth;
				double m_LatitudeNorth;
				MISTRING m_CRS;
				double m_ProjectionMinX;
				double m_ProjectionMaxX;
				double m_ProjectionMinY;
				double m_ProjectionMaxY;
				double m_Accuracy;
				UINT32 m_NumCols;
				UINT32 m_NumLins;
				double m_XScale;
				double m_YScale;
				MISTRING m_DataType;
				
				friend class GEOCAT;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Iterator for searching the catalog.
		class CONST_ITERATOR {
			public:
				//! Default constructor
				explicit CONST_ITERATOR (
					const GEOCAT *cat = 0
					):
					m_cat(cat)
					{ }

				//! Destructor
				~CONST_ITERATOR (
					);

				//! Dereference operator.  Returns reference to item
				const RECORD& operator* (
					) const {
					return (m_record);
					}

				//! Dereference operator.  Returns pointer to item
				const RECORD* operator-> (
					) const {
					return (&m_record);
					}

				//! Pre-increment operator.
				CONST_ITERATOR& operator++ (
					);

				//! Equality operator
				bool operator== (
					const CONST_ITERATOR& rhs
					) const {
					return (m_cat == rhs.m_cat && (m_itRecObject.Done() == rhs.m_itRecObject.Done() || m_record.GetID() == rhs.m_record.GetID()));
					}
					
				//! Inequality operator
				bool operator!= (
					const CONST_ITERATOR& rhs
					) const {
					return (!operator==(rhs));
					}

				//! Get "object" associated with current record.
				ERRVALUE GetObject (
					RVC::OBJITEM& objitem,
					bool findnow = false
					) const;

				bool IsDone (
					) const { return (m_itRecObject.Done()); }

				//! Show progress within catalog.
				//! Also checks for "cancel" if enabled in status dialog/form.
				ERRVALUE ShowProgress (
					STATUSCONTEXT& scontext,				//!< Status context
					UINT16 line = 0							//!< Line in progress dialog
					);
					
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				const GEOCAT* m_cat;
				RVC::DBTABLE::CONST_RECORDITERATOR m_itRecObject;
				RECORD m_record;

				friend class GEOCAT;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		// XXX If need non-const ITERATOR insert it here...
		
		//! Get CRS for objects used in catalogs.
		static const SPATREF::COORDREFSYS& GetCoordRefSys (
			);

		//! Default Constructor.
		//! Doesn't open anything.  Use Open or Make methods below.
		GEOCAT (
			);

		//! Destructor
		~GEOCAT (
			);

		//! Add a media description to catalog
		ERRVALUE AddMedia (
			const MISTRING& desc
			);

		//! Build index for location field
		ERRVALUE BuildIndex (
			);

		//! Close the catalog
		ERRVALUE Close (
			);

		//! Add folder to be excluded from searching
		ERRVALUE ExcludeFolders (
			const FILEPATHLIST& list,
			bool AddExcludeFile
			);

		//! Get list of folders excluded for this catalog
		void GetExcludeFolders (
			FILEPATHLIST& list
			) {
			list = m_ExcludeList;
			}

		//! Get list of folders that the geocatexlude file was found in
		void GetExcludeAllFolders (
			FILEPATHLIST& list
			) {
			list = m_ExcludeAllList;
			}

		//! Get path to geocatalog file.
		const FILEPATH& GetFilePath (
			) const {
			return (m_filepath);
			}

		ERRVALUE GetMediaList (
			MISTRINGLIST& list
			);

		//! Retrieve the record for a given file given it's ID.
		//! Returns -1 if record not found.
		ERRVALUE GetRecord (
			UINT32 id,
			RECORD& FileRecord
			) const;

		//! Retrieve the record for a given file given a location string pointing to the file.
		//! Returns -1 if not found.
		ERRVALUE GetRecord (
			const MISTRING& filepath,
			RECORD& record
			) const;

		void GetScanFolders (
			FILEPATHLIST& list
			)  {
			list = m_ScanList;
			}

		//! Get iterator pointing to first item in the catalog.
		void IteratorBegin (
			CONST_ITERATOR& iterator
			) const;

		//! Create a catalog file.
		ERRVALUE Make (
			const FILEPATH& catalog
			);

		//! Open a catalog file.
		//! Catalog files have the extention ".gsc" for GeoSpatial Catalog
		//! May skip open mode parameter.  Just open for read and the scan
		//! methods can change it to write as needed.
		ERRVALUE Open (
			const FILEPATH& catalog,
			bool bCreateIfNotFound = false,
			bool bOpenForWrite = false
			);
		
		//! Does a prescan of the directory counting files to index
		ERRVALUE PrescanDirectory (
			const FILEPATH& dir,
			STATUSCONTEXT& StatusContext,
			bool bScanSubDirs = true
			) const;

		//! Remove all references to given file from the catalog.
		//! If the file is really a directory, it will remove references
		//! to all files in the given directory.
		ERRVALUE Remove (
			const FILEPATH& file
			);

		//! Remove all references to given file from the catalog.
		ERRVALUE Remove (
			const RECORD& record
			);

		//! Scan all RVC files in a given directory.
		//! Also scans the catalog for any files which were in this
		//! directory but are not there any more and deletes them from
		//! the catalog.
		//! If you want to provide status, call PrescanDirectory() to
		//! initialize the StatusContext
		ERRVALUE ScanDirectory (
			const FILEPATH& dir,
			STATUSCONTEXT* StatusContext = 0,
			bool bScanSubDirs = true
			);

		//! Scans an RVC file and adds or updates it's records in the catalog.
		//! If already cataloged and time stamp agrees, it won't do anyting.
		ERRVALUE ScanFile (
			const FILEPATH& file
			);

		ERRVALUE SetScanFolders (
			const FILEPATHLIST& list
			);

		//! Write a record.
		//! If a record for the given object's ID already exists, it is updated.
		//! If the record does not exist, a new record is created, and the record's
		//! ID is updated.
		ERRVALUE Write (
			RECORD& record
			);
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		ERRVALUE ExtractRecord (
			const RVC::DBTABLE::RECORD& RecObject,
			RECORD& record
			) const;

		ERRVALUE Read (
			INT32 RecNum,
			RECORD& record
			) const;

		ERRVALUE Read (
			CONST_ITERATOR& it,
			RECORD& record
			) const;

		ERRVALUE ScanFolder (
			RVC::OBJECT& ObjParent
			);

		ERRVALUE ScanObject (
			RVC::OBJECT& Object
			);

		UINT32 m_OpenCount;
		FILE_TEXT m_errorlog;
		FILE_TEXT m_scanlog;
		FILEPATH m_filepath;
		FILEPATH m_lastfilepath;
		FILEPATHLIST m_ExcludeList;
		FILEPATHLIST m_ExcludeAllList;
		FILEPATHLIST m_ScanList;
		RVC::DBASE m_ObjDb;
		RVC::DBTABLE m_Table;
		RVC::DBTABLE m_MediaTable;
		RVC::DBTABLE m_ExcludeTable;
		RVC::DBTABLE m_ExcludeAllTable;
		RVC::DBTABLE m_ScanTable;
		bool m_HasNewFields;
		bool m_HasError;
		mutable RVC::DBTABLE::RECORD m_Record;
		mutable RVC::DBTABLE::RECORD m_ExcludeRecord;
		mutable RVC::DBTABLE::RECORD m_ExcludeAllRecord;
		mutable RVC::DBTABLE::RECORD m_ScanRecord;

		// UNIMPLEMENTED / UNSUPPORTED
		GEOCAT (const GEOCAT&);
		GEOCAT& operator= (const GEOCAT&);

		//! Iterator gets access to private Read() method.
		friend class CONST_ITERATOR;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


#endif
