/******************************************************************************
 * \file mi32/mg2mem8g.h
 * \brief A RAM-based implementation of MGD::DEVICE for 8-bit Gray visuals
 *
 * \if NODOC
 * $Log: mg2mem8g.h_v $
 * Revision 1.12  2011/10/26 17:58:07  mju
 * Frame buffer support.
 *
 * Revision 1.11  2005/09/15 21:40:14  mju
 * Fix unref parm warning.
 *
 * Revision 1.5  2003/05/19 16:26:31  dwilliss
 * Added GetColorPixel/GetPixelColor methods
 *
 * Revision 1.4  2002/10/17 15:22:32  mju
 * COLORSPACE enum values changed.
 *
 * Revision 1.2  2002/05/29 22:22:58  dwilliss
 * Added Filter and FilterLowPass method
 *
 * Revision 1.1  2002/05/29 17:30:12  fileserver!dwilliss
 * Initial revision
 * \endif
**/

#ifndef INC_MI32_MG2MEM8G_H
#define INC_MI32_MG2MEM8G_H

#ifndef INC_MI32_MGD2_H
#include <mi32/mgd2.h>
#endif


namespace MGD {

class CONTEXT_MEM8G;

class DEVICE_MEM8G : public MGD::DEVICE {

	public:

		//! constructor
		DEVICE_MEM8G();

		//! Destructor
		virtual ~DEVICE_MEM8G();

		ERRVALUE Create (
			INT32 height,
			INT32 width
			);

		//! Run a generic filter over the whole image.
		ERRVALUE Filter (
			const SIMPLE_ARRAY<double*> kernel,
			int kheight,
			int kwidth
			);

		ERRVALUE FilterLowPass (
			int radius
			);

	protected:
		UINT32 m_BytesPerLine;

		virtual CAPABILITY v_GetCapabilities () const;

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		// Copy constructor private and unimplemented
		DEVICE_MEM8G (const DEVICE_MEM8G&);

		UINT8* m_buf;

		friend class MGD::CONTEXT_MEM8G;	// Only MGD_DEVICE::CreateGC() can call our constructor
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Create a GC for drawing.  
		//!
		//! Derived classes may override this and make it return anything
		//! derived from MGD_GC.  MGD_GC's constructor is private, so this
		//! is the only way for calling code to create a GC.
		virtual CONTEXT* v_CreateContext (
			);

		//! Fill a horizontal span.
		//!
		//! Many of the default method implementations call this to do all the work
		virtual ERRVALUE v_FillSpan (
			MGD::CONTEXT* vgc,
			INT32 x1,
			INT32 x2,
			INT32 y
			);

		//! Given a color, return the pixel value that matches or is as close
		//! as possible
		virtual PIXEL v_GetColorPixel (
			const COLOR& color
			);

		//! Return the "native" COLORSPACE used by the display
		virtual COLORSPACE v_GetColorSpace (
			) const;

		//! Return a pointer to a buffer holding part of the frame buffer
		virtual UINT8* v_GetImagePointer (
			INT32 x,
			INT32 y,
			INT32 len,
			UINT16 FrameBuffer
			);

		virtual void v_GetPixelColor (
			PIXEL pixel,
			COLOR& color
			) const;

	};

		
class CONTEXT_MEM8G : public MGD::CONTEXT {
	public:
		// Note, default Constructor is protected

		virtual ~CONTEXT_MEM8G();



	protected:

		//! Default Constructor 
		CONTEXT_MEM8G (
			MGD::DEVICE_MEM8G* disp
			);

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT

		MGD::DEVICE_MEM8G* m_DevMem8g;

		//! Copy constructor private and unimplemented
		CONTEXT_MEM8G(const CONTEXT_MEM8G&);

		friend class MGD::DEVICE_MEM8G;	// Only MGD_DEVICE::CreateGC() can call our constructor
		#endif // GENERATING_DOXYGEN_OUTPUT

		virtual ERRVALUE v_PutImageSpanDirect (
			INT32 x,
			INT32 y,
			INT32 len,
			const void* data,
			const UINT8* mask
			);

	};


}	//! End of namespace

#endif	// INC_MI32_MG2MEM8G_H


