/**
 * \file mi32/mimetype.h
 * \brief MIMETYPE class definitions.
 *
 * \if NODOC
 * $Id: mimetype.h_v 1.2 2008/09/02 17:55:46 mju Exp $
 *
 * $Log: mimetype.h_v $
 * Revision 1.2  2008/09/02 17:55:46  mju
 * Fix doxygen warning.
 *
 * Revision 1.1  2007/01/29 18:26:38  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_MIMETYPE_H
#define	INC_MI32_MIMETYPE_H

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif


//! Container for MIME content/media;parameters specification.
class MIMETYPE {
	public:

		//! Enumeration of supported content types.
		enum CONTENT {
			CONTENT_Unknown =			0,
			CONTENT_Application =	1,
			CONTENT_Audio =			2,
			CONTENT_Image =			3,
			CONTENT_Message =			4,
			CONTENT_Model =			5,
			CONTENT_Multipart =		6,
			CONTENT_Text =				7,
			CONTENT_Video =			8,
			CONTENT_NUMTYPES
			};

		//! Enumeration of supported media types.
		enum MEDIA {
			MEDIA_Unknown =						0,
			MEDIA_Unknown_XML = 					1,		//!< Unknown, but contains +xml
			MEDIA_GIF =								2,		//!< gif
			MEDIA_JP2 =								3,		//!< jp2
			MEDIA_JPEG =							4,		//!< jpeg
			MEDIA_JPX =								5,		//!< jpx
			MEDIA_PNG =								6,		//!< png
			MEDIA_TIFF =							7,		//!< tiff
			MEDIA_SVG_XML =						8,		//!< svg+xml
			MEDIA_Plain =							9,		//!< plain
			MEDIA_XML =								10,	//!< xml
			MEDIA_CSV =								11,	//!< csv
			MEDIA_HTML =							12,	//!< html
			MEDIA_VND_GoogleEarth_KML_XML =	13,	//!< vnd.google-earth.kml+xml
			MEDIA_NUMTYPES
			};

		//! Convert content enumeration to string.
		static const char* ContentToString (
			CONTENT content
			);

		//! Convert media enumeration to string.
		static const char* MediaToString (
			MEDIA media
			);

		//! Default constructor.
		MIMETYPE ();

		//! Copy constructor.
		MIMETYPE (
			const MIMETYPE& rhs
			);

		//! Construct with specified content, media and optional parameters.
		MIMETYPE (
			CONTENT content,
			MEDIA media,
			const char *parameters = 0
			);

		//! Construct from MIME type string in form "content/media;parameters" (implicit).
		MIMETYPE (
			const char *mimestr
			);

		//! Destructor.
		~MIMETYPE ();

		//! Assign from MIMETYPE instance.
		MIMETYPE& operator= (
			const MIMETYPE& rhs
			);

		//! Assign from string.
		MIMETYPE& operator= (
			const char *mimestr
			);

		//! Get content enumeration.
		CONTENT GetContent (
			) const { return (m_content); }

		//! Get media enumeration.
		MEDIA GetMedia (
			) const { return (m_media); }

		//! Get parameters string.
		//! @return Pointer to string, will be empty string (not NULL) if no parameters.
		const char* GetParameters (
			) const { return (m_parameters); }

		//! Get full MIME type string.
		const char* GetString (
			) const { return (m_mimestr); }

		//! Set from MIME type string in form "content/media;parameters".
		//! @return true if content/media recognized, false if not.
		bool Set (
			const char *mimestr
			);

		//! Set content, media and optional parameters
		void Set (
			CONTENT content,
			MEDIA media,
			const char *parameters = 0
			);

		//! Set parameters without changing content/media.		
		void SetParameters (
			const char *parameters				//! Parameters to set, may be NULL to clear existing
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		SIMPLE_ARRAY<char> m_mimestr;
		CONTENT m_content;
		MEDIA m_media;
		char *m_parameters;						// Pointer to start of parameters in mimestr

		void UpdateParameters ();

		#endif
	};

#endif	// INC_MI32_MIMETYPE_H
