/**************************************************************************
 * \file mi32/mxgraph.h
 * \brief Graphical display widget functions
 *
 * \if NODOC
 * $Id: mxgraph.h_v 1.61 2013/01/15 17:51:46 mju Exp $
 *
 * $Log: mxgraph.h_v $
 * Revision 1.61  2013/01/15 17:51:46  mju
 * Add defaults for setAxisNames.
 *
 * Revision 1.60  2011/11/04 12:55:27  mju
 * Remove redundant hdrs.
 *
 * Revision 1.59  2011/05/17 13:55:34  mju
 * Use const for lineid.
 *
 * Revision 1.58  2008/11/19 18:00:00  mju
 * Change to use delegate for drawing callback so can eliminate transparm usage.
 *
 * Revision 1.57  2007/09/19 17:55:14  mju
 * Add cursor tracking support.
 *
 * Revision 1.56  2006/11/09 00:08:04  bcolombe
 * Changed MxGraphHistogramSetFromRaster to use objitem instead of filepath and inode
 *
 * Revision 1.52  2004/03/18 20:05:48  vdronov
 * integer x labels for XYPlot
 *
 * Revision 1.1  1997/06/10  22:07:15  dfrolov
 * Initial revision
 *
 * \endif
*************************************************************************/

#ifndef	INC_MI32_MXGRAPH_H
#define	INC_MI32_MXGRAPH_H

#ifndef	INC_MI32_MGD2_H
	#include <mi32/mgd2.h>
#endif

#ifndef	INC_MI32_MXTOOLS_H
	#include <mi32/mxtools.h>
#endif


/* Callback reasons for MxGraph widget */

#define MxGraph_DestroyReason					0x00000001
#define MxGraph_LeftPressReason				0x00000002
#define MxGraph_LeftReleaseReason			0x00000004
#define MxGraph_RightPressReason				0x00000008
#define MxGraph_RightReleaseReason			0x00000010
#define MxGraph_MotionLeftNotifyReason		0x00000020
#define MxGraph_MotionRightNotifyReason	0x00000040
#define MxGraph_RedrawReason					0x00000080
#define MxGraph_ExposeReason					0x00000100
#define MxGraph_ResizeReason					0x00000200
#define MxGraph_ClientMessageReason			0x00000400
#define MxGraph_DataChangeReason				0x00000800
#define MxGraph_ColorChangeReason			0x00001000
#define MxGraph_CursorEnterReason			0x00002000
#define MxGraph_CursorLeaveReason			0x00004000
#define MxGraph_CursorMotionReason			0x00008000

#define MxGraph_NDViewClearScreen			0x02000000
#define MxGraph_NDViewRestoreScreen			0x04000000

struct POINT_ND {
	DOUBLE *coord;								//!<  coordinates of the point 
	UINT16 cluster;							//!<  class id 
	INT32 pointid;								//!<  user supplied id-data for selection callbacks 
	};

struct MxGraphCallbackStruct {
	UINT32 reason;
	Widget widget;	 							//!<  histogram widget 
	Display *display;							//!<  histogram display 
	XEvent event;
	int x;										//!<  cursor position 
	int y;
	DPOINT2D Pos;								//!<  tool or cursor position in graph coordinates
	const MIUNICODE *lineid;				//!<  line name used as id 
	int vertexnum;								//!<  closest vertex number 
	COLOR color;								//!<  color of the line 
	};

/* flags for Histogram widget */

#define GRAPH_HISTOGRAM_NOFILL		0x0001		//!<  don't fill histogram bars or polygon 
#define GRAPH_HISTOGRAM_GRIDLINES	0x0002		//!<  display grid lines 
#define GRAPH_HISTOGRAM_CUMULATIVE	0x0004      //!<  display histogram as cumulative graph 
#define GRAPH_HISTOGRAM_MINMAXONLY	0x0008		//!<  not implemented yet 
#define GRAPH_HISTOGRAM_CREATEMENU	0x0010		//!<  create standart pull-down menu 
#define GRAPH_HISTOGRAM_LOGARITHMIC 0x0020		//!<  logarithmic y - scale 
#define GRAPH_HISTOGRAM_MENUCLOSE	0x0100		//!<  add "close" item to the menu 

#define LABELS_NONE		1
#define LABELS_PLAIN		2
#define LABELS_INBOX		3

#define HISTOGRAM_IGNORE		0
#define HISTOGRAM_BAR			1
#define HISTOGRAM_OUTLINE		2
#define HISTOGRAM_STRIP			3
#define HISTOGRAM_LASTSTYLE	3

/* flags for XYPlot widget */
#define XYPLOT_DRAWSTAIRS			0x0001			//!<  draw plot in the stairs mode 
#define XYPLOT_SOLIDPLOT			0x0002			//!<  fill area below plot 
#define XYPLOT_DRAWGRID				0x0004			//!<  draw grid lines 
#define XYPLOT_HASTOOL				0x0008			//!<  xyplot has a crosshair tool 
#define XYPLOT_DRAWNODES			0x0010			//!<  draw circles arround graph nodes 
#define XYPLOT_SNAPTOVERTEX			0x0020			//!<  snap tool to the closest vertex 
#define XYPLOT_XVALUEISINTEGER		0x0040			//!<  x value is integer value 

typedef fastdelegate::FastDelegate<void(MGD::CONTEXT*, int, int)> MXGRAPH_DELEGATE_DRAW;



#if defined(__cplusplus)
extern "C" {
#endif

//! Add a callback function to the "MxGraph" widget.
int MxGraphAddCallback (
	Widget wgraph, 							//!< "MxGraph" widget returned by MxGraphCreate function
	McbCallbackFunc,							//!< Callback function
	void *data,									//!< Data passed to the callback function
	UINT32 reason								//!< Callback reason , see MxGraph_...Reason masks
	);

//! Redraw contents of the "MxGraph" widget.
int MxGraphDraw (
	Widget wgraph								//!< "MxGraph" widget
	);

//! Enable redraw of "MxGraph" widget.
int MxGraphEnableRedraw (
	Widget wgraph, 
	int flag
	);

//! Remove previously assigned callback function.
void MxGraphRemoveCallback (
	Widget w, 
	McbCallbackFunc, 
	void *data,
	UINT32 reason
	);

//! Create "MxGraph" widget.
//!
//! @return "MxGraph" widget returned or NULL if error.
Widget MxGraphCreate (
	Widget parent,								//!< Parent widget
	int Width,									//!< Initial width of the drawing area 
	int Height,									//!< Initial height of the drawing area
	MXGRAPH_DELEGATE_DRAW DelegateDraw,	//!< Delegate to call for drawing
	Arg *arg,									//!< Attachments for the "MxGraph" widget
	int n,										//!< Number of elements in arg arrray
	void *userdata
	);

//! Save contents of the pixmap to the raster. 
int MxGraphSnapshot (
	Widget wgraph
	);

//!	Get user data.
//! 
//! @return Pointer to the userdata or NULL.
void* MxGraphGetUserData (
	Widget wgraph
	);

//! Get actual dimension of the "MxGraph's" DrawArea widget.
int MxGraphGetDimensions (
	Widget wgraph,								//!< "MxGraph" widget
	int *Width,									//!< Width in pixels returned
	int *Height									//!< Height in pixels returned
	);

//! Get tool handle associated with any MxGraph... widget.
//!
//! @return Tool handle or NULL.
//!
//! NOTE: You can use this function with any MxGraph... widget.
MXTHANDLE MxGraphGetTool (
	Widget wgraph								//!< Any MxGraph... widget (MxGraphXYPlot or MxGraphHistogram)
	);

//! Clear contents of the histogram widget.
#define	MxGraphHistogramClear(wdt)	(MxGraphHistogramSetData(wdt,NULL,0))

//! Compute histogram from raw data and assign.
int MxGraphHistogramComputeFromData (
	Widget whisto,								//!< "histogram" widget passed
	DOUBLE *data,								//!< Raw data to compute histogram from
	int numdata,								//!< Number of items in the data array
	int numintervals							//!< Number of histogram intervals to compute
	);
	
//! Create a histogram widget.
//!
//! @return "Histogram" widget or NULL.
//!
//! Flags:
//! 	GRAPH_HISTOGRAM_NOFILL			don't fill histogram bars or polygon
//!		GRAPH_HISTOGRAM_GRIDLINES		display grid lines
//!		GRAPH_HISTOGRAM_LABELS	 		display labels
//!		GRAPH_HISTOGRAM_CUMULATIVE		display histogram as cumulative graph
//!		GRAPH_HISTOGRAM_MINMAXONLY		not implemented yet
//!		GRAPH_HISTOGRAM_CREATEMENU		create standart pull-down menu
//!		GRAPH_HISTOGRAM_LOGARITHMIC 	logarithmic y - scale
//!		GRAPH_HISTOGRAM_LABELINBOX		draw label boxes
Widget MxGraphHistogramCreate (
	Widget parent,								//!< Parent widget
	int Width,									//!< Initial width of the widget
	int Height,									//!< Initial height of the widget
	int numintervals,							//!< Number of intervals or 0
	UINT32 *data,								//!< Histogram data or NULL
	UINT32 flags,								//!< Flags
	Arg *arg,									//!< Attachments
	int n											//!< Number of elements in arg
	);

//! Enable/disable tooltip in the "Histogram" window.
int MxGraphHistogramEnableTooltip (
	Widget whisto, 							//!< Histogram widget 
	int flag										//!< True or False
	);

//! Get number of counts for specified bin number.
int MxGraphHistogramGetCount (
	Widget whisto,								//!< Histogram widget
	INT32 bin,									//!< Bin number
	INT32 *count								//!< Number of counts returned
	);

//! Get data range for a histogram.
//!
//! NOTE: if histogram data doesn't have start/end value set,
//!	then this function returns:		
//!		start = 0.0		end = numintervals - 1
int MxGraphHistogramGetDataRange (
	Widget whisto,								//!< Histogram widget
	DOUBLE *start,								//!< Starting value returned
	DOUBLE *end									//!< Ending value returned 
	);

//! Get histogram maximum count and it's position (bin number). 
int MxGraphHistogramGetMaxCount (
	Widget whisto,								//!< Histogram widget
	INT32 *bin,									//!< Bin number returned
	INT32 *count								//!< Number of counts returned
	);

//! Get number of intervals in the histogram.
int MxGraphHistogramGetNumIntervals (
	Widget whisto,								//!< Histogram widget
	INT32 *numintervals						//!< Number of intervals returned
	);

//! Convert screen pixel position into (value & count).
int MxGraphHistogramScreenToData (
	Widget whisto,								//!< Histogram widget
	int x,										//!< Screen pixel coordinate
	int y,										//!< Screen pixel coordinate
	INT32 *bin,									//!< Bin number returned
	DOUBLE *value,								//!< Value returned (not a bin number)
	INT32 *count								//!< Count returned
	);

//! Assign data in histogram form to the histogram widget.
int MxGraphHistogramSetData (
	Widget whisto,								//!< Histogram widget
	UINT32 *data,								//!< Array of counts
	int items									//!< Number of entries in data array
	);

//! Assign data in histogram form to the histogram widget, double form.
int MxGraphHistogramSetDataDouble (
	Widget whisto,
	DOUBLE *data,
	int numintervals
	);

//! Set display flags.
//!
//! Flags:
//!		GRAPH_HISTOGRAM_NOFILL			don't fill histogram bars or polygon
//!		GRAPH_HISTOGRAM_GRIDLINES	   display grid lines
//!		GRAPH_HISTOGRAM_LABELS	 	   display labels
//!		GRAPH_HISTOGRAM_CUMULATIVE	   display histogram as cumulative graph
//!		GRAPH_HISTOGRAM_MINMAXONLY	   not implemented yet
//!		GRAPH_HISTOGRAM_CREATEMENU	   create standart pull-down menu
//!		GRAPH_HISTOGRAM_LOGARITHMIC   logarithmic y - scale
//!		GRAPH_HISTOGRAM_LABELINBOX	   draw label boxes
int MxGraphHistogramSetFlags (
	Widget whisto,								//!< Histogram widget
	UINT32 flags								//!< Flags
	);

//! Set appearance parameters of the histogram widget.
//!
//! Draw styles:
//! 	HISTOGRAM_IGNORE  - ignore draw style reassignment
//!		HISTOGRAM_BAR		- display as a set of 3D-bars
//!		HISTOGRAM_OUTLINE	- display as histogram outline
//!		HISTOGRAM_STRIP	- display as "fancy" 3D-strip
int MxGraphHistogramSetStyle (
	Widget whisto,								//!< Histogram widget
	COLOR *fg,									//!< NULL or foreground color (Black by default)
	COLOR *bg,									//!< NULL or background color (White by default)
	COLOR *fill,								//!< NULL or fill color (Green by default)
	COLOR *labelcover,						//!< NULL or color for displaying labels (Black by default)
	DOUBLE levelcount,						//!< Number of counts used as an interval for drawing of the horizontal grid lines and labeling (Pass negative value to ignore).
	int DrawStyle								//!< Type of the histogram representation 
	);

//! Set label font and size.
int MxGraphHistogramSetTextStyle (
	Widget whisto,								//!< Histogram widget
	const char *Fontname,					//!< Font name to use
	float size									//!< Label size 8,10,12,14...
	);

//! Set parameters for ranging histogram values and displaying horizontal tic-marks and labels.
//!
//!	NOTE: This function is important for assignment real world values to the horizontal scale. 
//! If programmer doesn't call this function, tic marks will represent just array numbers.
int MxGraphHistogramSetTicMarks (
	Widget whisto,								//!< Histogram widget
	DOUBLE datastart,							//!< Value that correspond to the first entry of the histogram data array
	DOUBLE dataend,							//!< Value that corresponds to the last entry of the histogram array
	DOUBLE ticstart,							//!< Data value for the first vertical tic-mark or grid-line
	DOUBLE ticspacing							//!< Spacing in data values between vertical tic-marks
	);

//! Assign data to the histogram widget.
int MxGraphHistogramSetFromRaster (
	Widget whisto, 							//!< Histogram widget
	RVC::OBJITEM objitem,
	const REGION2D *region,							
	int ColorComp								//!< Color component for RGB/BGR rasters
	);

//! 
int MxGraphHistogramSmooth (
	Widget whisto								//!< Histogram widget
	);

//! This functions checks and removes 0 value from histogram.
int MxGraphHistogramCheckForNull (
	Widget whisto								//!< Histogram widget
	);

//! Add line to the plot using specific scaling factors. 
int MxGraphXYPlotAddLineScaled (
	Widget wplot, 								//!< XYPlot widget
	const MIUNICODE *name,						//!< Line name (will be used for data tip, click right mouse button)
	DPOLYLINE *Line,							//!< Polyline data
	const COLOR *color,								//!< Color to use
	DOUBLE linewidth,							//!< Linewidth in pixels
	DOUBLE offset,								//!< Offset value to use 
	DOUBLE scale,								//!< Scale value to use
	int Redraw									//!< Redraw flag 
	);

//! Add line to the plot window.
int MxGraphXYPlotAddLine (
	Widget wplot,								//!< XYPlot widget
	const MIUNICODE *name,						//!< Line name (will be used for data tip, click right mouse button)
	DPOLYLINE *Line,							//!< Polyline data 
	const COLOR *color,								//!< Color to use
	DOUBLE linewidth,							//!< Linewidth to use in pixels
	int Redraw									//!< Redraw flag
	);

//! Clear content of the plot window.
int MxGraphXYPlotClear (
	Widget wplot,								//!< XYPlot Widget
	int Redraw									//!< TRUE/FALSE redraw flag
	);

//! Create XYPlot widget.
//!
//!	@return XYPlot widget or NULL.
//!
//! Flags:	
//!		XYPLOT_DRAWSTAIRS - draw plot in the stairs mode 
//!		XYPLOT_SOLIDPLOT  - fill area below plot 	
//!		XYPLOT_DRAWGRID   - draw grid lines 
//!		XYPLOT_HASTOOL	   - xyplot has a crosshair tool 
//!		XYPLOT_DRAWNODES  - draw circles arround graph nodes 
Widget MxGraphXYPlotCreate (
	Widget parent,								//!< Parent widget
	int Width,									//!< Width of the XYPlot widget
	int Height,									//!< Height of the XYPlot widget
	const MIUNICODE *title,					//!< Title of the XYPlot widget
	const MIUNICODE *xtitle,					//!< Title for the horizontal axis
	const MIUNICODE *ytitle,					//!< Title for the vertical axis
	const MIUNICODE *y2title,					//!< Title for the secondary vertical axis (not yet impl.)
	UINT32 flags,								//!< Flags
	Arg *argext,								//!< Argument list
	int next										//!< Number of items in argument list
	);

//! Edit Y-value of the vertex.
//!
//! @return TRUE on success, FALSE if no line found, error code < 0 on error.
int MxGraphXYPlotEditVertex (
	Widget wplot,								//!< XYplot widget
	const MIUNICODE *linename,				//!< Line name for identification
	INT32 vertexnum,							//!< Vertex number 0....NumPoints-1
	DOUBLE y, 									//!< New value for y-coordinate
	int redraw									//!< Redraw flag TRUE or FALSE
	);

//! Get current colors.
int MxGraphXYPlotGetColors (
	Widget wplot,								//!< XYplot widget
	COLOR *A,									//!< Color used for plot area background color
	COLOR *B,									//!< Color used for background color
	COLOR *TextColor							//!< Color used for drawing text
	);

//! Get extents of the plot window in data-space.
int MxGraphXYPlotGetExtents (
	Widget wplot,								//!< XYPlot widget
	DRECT2D *rect								//!< Rectangle returned
	);

//! Get horizontal data range for display.
int MxGraphXYPlotGetHorizontalRange (
	Widget wplot,								//!< XYPlot widget
	DOUBLE *minval,							//!< Min. value for display range (returned)
	DOUBLE *maxval								//!< Max. value for display range (returned)
	);

//! Get a copy of the line.
//!
//! @return TRUE on success, FALSE if no line found, error code < 0 on error.
//!
//! NOTE: Programmer has to free memory for (poly->point) array
//!	when done working with the structure.  This is just a copy of original line.
int MxGraphXYPlotGetLine (
	Widget wplot,								//!< XYplot widget
	const MIUNICODE *linename,				//!< Line name for identification
	DPOLYLINE *poly							//!< Polyline structure returned
	);

//! Get vertical data range for display.
int MxGraphXYPlotGetVerticalRange (
	Widget wplot,								//!< XYPlot Widget
	DOUBLE *minval,							//!< Min. value for display range (returned)
	DOUBLE *maxval								//!< Max. value for display range (returned)
	);

//! Set flags to use.
//!
//! Flags:
//! 	XYPLOT_DRAWSTAIRS - draw plot in the stairs mode 
//!		XYPLOT_SOLIDPLOT  - fill area below plot 	
//!		XYPLOT_DRAWGRID	- draw grid lines 
//!		XYPLOT_HASTOOL	   - xyplot has a crosshair tool (this should be set only when the tool is created!)
//!		XYPLOT_DRAWNODES  - draw circles arround graph nodes
int MxGraphXYPlotSetFlags (
	Widget wplot,								//!< XYplot Widget
	UINT32 flags								//!< Flags
	);

//! Set font to use.
int MxGraphXYPlotSetFontname (
	Widget wplot,								//!< XYplot widget
	const MIUNICODE *fontname					//!< Fontname to use
	);

//! Set size of the font to use.
int MxGraphXYPlotSetFontSize (
	Widget wplot,								//!< XYplot widget
	int Size										//!< Size of the font
	);

//! Set new names for the axises using NON-MIUNICODE strings.
int MxGraphXYPlotSetAxisNames (
	Widget wplot,							  	//!< XYPlot widget
	const char *title,						//!< Title of the XYPlot widget
	const char *xtitle,						//!< Title for the horizontal axis
	const char *ytitle,						//!< Title for the vertical axis
	const char *y2title = 0					//!< Title for the secondary vertical axis (not used)
	);

//! Set new names for the axises using MIUNICODE strings.
int MxGraphXYPlotSetAxisNamesUC (
	Widget wplot,								//!< XYPlot widget
	const MIUNICODE *title,					//!< Title of the XYPlot widget
	const MIUNICODE *xtitle,				//!< Title for the horizontal axis
	const MIUNICODE *ytitle,				//!< Title for the vertical axis
	const MIUNICODE *y2title = 0			//!< Title for the secondary vertical axis (not used)
	);

//! Set horizontal data range for display.
int MxGraphXYPlotSetHorizontalRange (
	Widget wplot,								//!< XYPlot widget
	DOUBLE minval,								//!< Min. value for display range
	DOUBLE maxval,								//!< Max. value for display range
	int Redraw									//!< Redraw flag
	);

//! Set callback for change to tool position.
//! NOTE: Tool should be created with XYPLOT_HASTOOL flag in order to be able to use crosshair 
//! tool and this callback.
int MxGraphXYPlotSetChangePositionCallback (
	Widget wplot,								//!< XYplot widget
	McbCallbackFunc func,					//!< Callback function
	void *userdata								//!< Data to pass to callback
	);

//! Set callback for cursor tracking.
void MxGraphXYPlotSetCursorTrackCallback (
	Widget wplot,								//!< XYplot widget
	McbCallbackFunc func,					//!< Callback function
	void *userdata								//!< Data to pass to callback
	);

//! Set vertical data range for display. 
int MxGraphXYPlotSetVerticalRange (
	Widget wplot,								//!< XYplot widget
	DOUBLE minval,								//!< Min. value for display range
	DOUBLE maxval,								//!< Max. value for display range
	int Redraw									//!< Redraw flag
	);

//! Set colors to use.
int MxGraphXYPlotSetColors (
	Widget wplot,								//!< XYplot widget
	COLOR *A,									//!< Color used for plot area background color
	COLOR *B,									//!< Color used for background color
	COLOR *TextColor							//!< Color used for drawing text
	);

//! Pop-up dialog that allows you to edit.
int MxGraphXYPlotEditLineColors (
	Widget wplot,								//!< XYPlot widget
	int Redraw									//!< Redraw flag (redraw if user click "OK")
	);

//! Clear NDview.
int MxGraphNDViewClearPoints (
	Widget wview
	);

//! Assign N-dimensional points to visualizer.
int MxGraphNDViewAddPoints (
	Widget wview,
	INT32 numdim,
	POINT_ND *points,
	INT32 numpts,
	int Normalize,
	int ToClass
	);

//! Add one N-dimensional point to the NDview.
//!
//! This function accepts ONLY NORMALIZED DATA in range between 0...1.
int MxGraphNDViewAddSinglePoint (
	Widget wview,
	INT32 numdim,								//!< This is necessary for dimension control
	POINT_ND *point
	);

//! Create N-dimensional view widget.
Widget MxGraphNDViewCreate (
	Widget parent,
	int Width,
	int Height,
	UINT32 flags,
	Arg *argext,
	int next
	);

//! Edit color map of the view. 
int MxGraphNDViewEditColors (
	Widget wview,
	int Redraw
	);

//! Get maximum number of classes.
int MxGraphNDViewGetMaxNumClasses (
	Widget wview
	);

//! Get color assigned to the class.
int MxGraphNDViewGetClassColor (
	Widget wview,
	int classnum,
	COLOR *color
	);

//! Rotate using "rolling ball" algorithm.
int MxGraphNDViewRotateRandom (
	Widget wview
	);

//! Rotate using "rolling ball" algorithm. 
int MxGraphNDViewRotate (
	Widget wview
	);

//! Assign N-dimensional points to visualizer.
int MxGraphNDViewSetPoints (
	Widget wview,
	INT32 numdim,
	POINT_ND *points,
	INT32 numpts,
	int Norm
	);

//! Create N-dimensional view widget.
int MxGraphNDViewSetRotationVector (
	Widget wview, 
	DOUBLE dx, 
	DOUBLE dy, 
	DOUBLE radius
	);

//! 
int MxGraphNDViewSetViewAxises (
	Widget wview,
	int haxis,
	int vaxis
	);

//! Do the random spinning of the data usign Rolling Ball.
int MxGraphNDViewSpinRandom (
	Widget wview
	);

//! 
int MxGraphNDViewSpinSetCancel (
	Widget wview
	);

//! 
int MxGraphNDViewSpinGetCancel (
	Widget wview
	);

//! Set the point mapping callback.
int MxGraphNDViewSetPointMappingCallback (
	Widget wview,								 
	int (*MapFunc)(void*,INT32,COLOR*,int),
	void *userdata								//!< User supplied private data structure
	);

//! 
int MxGraphNDViewSetSpeed (
	Widget wview,
	DOUBLE time_speed
	);

//! Update one coordinate value of the specified point.
//!
//! This function accepts ONLY NORMALIZED DATA in range between 0...1.
int MxGraphNDViewUpdatePointCoord (
	Widget wview, 
	INT32 pointnum, 							//!< Point number
	INT32 numdim, 								//!< Total number of dimensions, this is necessary for dimension control
	INT32 dim, 									//!< Dimension to update
	DOUBLE value								//!< Value to update
	);

//! Create a graphical scroll bar.
Widget MxGraphSliderCreate (
	Widget parent,
	int Width,
	int Height,
	int MinVal,
	int MaxVal,
	int InitVal,
	int *Value,
	UINT32 flags,
	Arg *argext,
	int next
	);

//! Get the scroll bar's minimum and maximum values.
int MxGraphSliderGetMinMax (
	Widget wgraph,
	int *MinVal,
	int *MaxVal
	);

//! Get the current value of scroll bar.
int MxGraphSliderGetValue (
	Widget wgraph,
	int *Value
	);

//! Set the background color for the scroll bar.
int MxGraphSliderSetBackgroundColor (
	Widget wgraph,
	COLOR *color
	);

//! Set marker color for scroll bar.
int MxGraphSliderSetMarkerColor (
	Widget wgraph,
	COLOR *fillcolor
	);

//! Set the minimum and maximum values for the scroll bar.
int MxGraphSliderSetMinMax (
	Widget wgraph,
	int NewMin,
	int NewMax, 
	int Redraw
	);

//! Set the current value for the scroll bar.
int MxGraphSliderSetValue (
	Widget wgraph,
	int NewValue,
	int Redraw
	);

//! Set the callback for a change of value for the scroll bar.
int MxGraphSliderSetValueChangeCallback (
	Widget wgraph,
	McbCallbackFunc func,
	void *userdata
	);

//! 
int MxGraphSliderSetSensitive (
	Widget wgraph,
	int Flag
	);

//! Create a graphical selector.
Widget MxGraphSelectorCreate (
	Widget parent,
	int Width,
	int Height,
	int NumSelections,
	UINT8 *Bitmap,
	UINT32 flags,
	Arg *argext,
	int next
	);

#if defined(__cplusplus)
}								
#endif


#endif	//!<  INC_MI32_MXGRAPH_H 
