/**
 * \file region2d.h <mi32/region2d.h>
 * \brief Functions and structure for regions
 *
 * \if NODOC
 * $Id: region2d.h_v 1.84 2012/11/09 18:04:31 mju Exp $
 *
 * $Log: region2d.h_v $
 * Revision 1.84  2012/11/09 18:04:31  mju
 * Remove legacy methods using coordop.
 *
 * Revision 1.83  2012/11/08 17:47:17  mju
 * Support spatref::coordtrans.
 *
 * Revision 1.82  2012/10/09 13:51:43  mju
 * Add round2d.
 *
 * Revision 1.81  2012/05/29 14:45:23  mju
 * Add addLines/Points.
 *
 * Revision 1.80  2012/05/18 20:56:57  mju
 * Add subregion2d::fitRectangleInside and convertToLongFloor with scale/offset.
 *
 * Revision 1.78  2010/11/04 21:05:56  mju
 * Remove generateTriangles back to own class.
 *
 * Revision 1.77  2010/11/03 15:32:04  mju
 * Add subregion::generateTriangles.
 *
 * Revision 1.76  2010/10/12 20:31:41  mju
 * Add subregion::subtractValidated with threshold parm.
 *
 * Revision 1.74  2009/08/04 16:48:01  mju
 * Add normalizeToCRS.
 *
 * Revision 1.73  2008/06/11 21:27:21  bcolombe
 * Added assignment from DRECT3D
 *
 * Revision 1.72  2008/02/20 01:01:07  vdronov
 * added several buffer methods
 *
 * Revision 1.70  2007/12/20 15:34:33  mju
 * Add computePerimeter using spatref crs measurement method.
 *
 * Revision 1.69  2007/12/19 21:00:42  mju
 * Add computeCentroid for subregion and region.
 *
 * Revision 1.68  2007/10/03 21:41:34  scowan
 * Core level support for lines and points.
 *
 * Revision 1.65  2007/09/21 21:32:40  mju
 * Add point and line components to region2d.
 *
 * Revision 1.60  2007/05/10 20:05:21  scowan
 * Added generateRandomPointInside method.
 *
 * Revision 1.59  2007/04/13 15:12:43  scowan
 * Added setDimension method.
 *
 * Revision 1.55  2005/11/11 16:27:49  dfriberg
 * GetNumIslands returns 0 if no polylines in subregion2d
 *
 * Revision 1.54  2005/09/29 20:17:02  scowan
 * Added method to generate a point inside a subregion.
 *
 * Revision 1.53  2005/06/06 17:40:08  scowan
 * Added testRegion method that takes a region2d.
 *
 * Revision 1.49  2005/03/03 22:16:03  scowan
 * Added remove islands method.
 *
 * Revision 1.48  2004/12/29 17:02:17  scowan
 * Added method to append subregion to region for cad drawing.
 *
 * Revision 1.44  2004/08/17 21:30:43  mju
 * Add updateExtentsRect for subregion and region.
 *
 * Revision 1.43  2004/08/09 23:54:01  scowan
 * Added set orientation methods.
 *
 * Revision 1.1  2002/10/09 22:32:03  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_REGION2D_H
#define	INC_MI32_REGION2D_H

#ifndef	INC_MI32_POLYLINE_H
	#include <mi32/polyline.h>
#endif

#ifndef	INC_MI32_SPATREF_H
	#include <mi32/spatref.h>
#endif


#ifdef GEOMDLL
	#define GEOMLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define GEOMLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
class TRANS2D_AFFINE;
class TRANS2D_MAPGEN;
class REGION2D;
class SERIALIZER;
class BINARYTOREGION2D;
namespace RVC {
	class CAD;
	class VECTOR;
	class REGION;
	class REGIONHANDLE;
	}
#endif


//! Generic 2D sub-region object in memory.
//! The sub-region can be represented by either a single rectangle or as a polygon and a list of zero
//! or more islands.  All POLYLINE's stored in the SUBREGION are closed.
class GEOMLIBCLASSEXPORT SUBREGION2D {
	public:
	
		typedef POLYLINELIST::CONST_ITERATOR CONST_ITERATOR;

		//! Filter for copying subregion.
		class GEOMLIBCLASSEXPORT COPYFILTER {
			public:
				//! Constructor.
				COPYFILTER ();
				//! Destructor.
				virtual ~COPYFILTER () = 0;
				//! Determine if island should be included.
				bool IsIslandIncluded (
					const POLYLINE& island
					) { return (v_IsIslandIncluded(island)); }
			private:
				//! Override to determine if island should be included.
				//! Default implementation always returns true.
				virtual bool v_IsIslandIncluded (
					const POLYLINE& island
					);
			};
	
		//! Default Constructor
		SUBREGION2D ();
			
		//! Copy Constructor
		SUBREGION2D (
			const SUBREGION2D& rhs
			);
			
		//! Constructor from DRECT2D
		SUBREGION2D (
			const DRECT2D& rect
			);
			
		//! Destructor
		~SUBREGION2D ();
			
		//! Assignment operator
		SUBREGION2D& operator= (
			const SUBREGION2D& rhs
			);

		//! Assignment operator from rectangle
		SUBREGION2D& operator= (
			const DRECT2D& rhs
			);

		//! Assignment operator from 3d rectangle
		SUBREGION2D& operator= (
			const DRECT3D& rhs
			);

		//! Assigns the sub-region to the passed in polygon
		//! This will clear any existing information in 'this'
		ERRVALUE Assign (
			const POLYLINE& Polygon
			);

		//! Assign from XMLNODE.
		ERRVALUE AssignFromXML (
			const XMLNODE *topnode
			);

		//! Assigns the sub-region to the passed in polygon.
		//! A validated polygon is a polygon that does not intersect itself.
		//! This will clear any existing information in 'this'
		void AssignValidated (
			const POLYLINE& Polygon
			);
			
		//! Assigns the sub-region to the passed in polygon list.  
		//! Assumes that the first polygon is the shell and the rest are islands.
		//! A validated polygon is a polygon that does not intersect itself.
		//! This will clear any existing information in 'this'
		void AssignValidated (
			const POLYLINELIST& PolygonList
			);
			
		//! Get the beginning of the list of POLYLINE's.
		//! The first item is the shell polygon, the rest following are islands.
		//! @return Iterator to the beginning of the POLYLINE list
		CONST_ITERATOR Begin (
			) const { return (m_ItemList.Begin()); }

		//! Clear the sub-region
		void Clear ();
			
		//! Clip a polyline to a sub-region, call the ClipTarget for each segment
		//! @return TRUE if part of the line is clipped, FALSE if not, < 0 error
		int ClipLine (
			POLYLINE& PolyLine,
			POLYLINE::CLIP& ClipTarget,
			CLIPMODE ClipMode = CLIPMODE_Inside
			) const;
			
		//! Compute the sub-region area in sub-region units
		double ComputeArea () const;
	
		//! Compute the sub-region area in sub-region units but do not exclude islands in the calculation
		double ComputeAreaNoIslands () const;

		//! Compute centroid of subregion.
		//! @return Subregion area (useful for weighting).
		double ComputeCentroid (
			DPOINT3D& centroid					//!< Centroid returned
			) const;
	
		//! Compute the sub-region perimeter in sub-region units
		double ComputePerimeter () const;
	
		//! Compute the sub-region perimeter in sub-region units but do not include islands in the calculation
		double ComputePerimeterNoIslands () const;
	
		//! Translate the sub-region using an affine transformation
		void ConvertForward (
			const TRANS2D_AFFINE& taf
			);
	
		//! Translate the sub-region using a mapgen transformation
		ERRVALUE ConvertForward (
			const TRANS2D_MAPGEN& tmg
			);
	
		//! Translate using SPATREF::COORDTRANS.
		ERRVALUE ConvertForward (
			const SPATREF::COORDTRANS& trans					//!< Transformation
			);
	
		//! Translate using TRANS2D_MAPGEN with densification.
		ERRVALUE ConvertForwardDense (
			const TRANS2D_MAPGEN& tmg
			);
	
		//! Translate using SPATREF::COORDTRANS with densification.
		ERRVALUE ConvertForwardDense (
			const SPATREF::COORDTRANS& trans,				//!< Transformation
			double tolerance										//!< Densification tolerance in output CRS unit
			);

		//! Translate the sub-region using an affine transformation
		void ConvertInverse (
			const TRANS2D_AFFINE& taf
			);
	
		//! Translate the sub-region using a mapgen transformation
		ERRVALUE ConvertInverse (
			const TRANS2D_MAPGEN& tmg
			);
	
		//! Translate the sub-region using a mapgen transformation with densification
		//! If the sub-region is the extents only, this will convert it to a polygon sub-region
		//! before doing the transformation
		ERRVALUE ConvertInverseDense (
			const TRANS2D_MAPGEN& tmg
			);
			
		//! Translate using SPATREF::COORDTRANS.
		ERRVALUE ConvertReverse (
			const SPATREF::COORDTRANS& trans					//!< Transformation
			);
	
		//! Translate using SPATREF::COORDTRANS with densification.
		ERRVALUE ConvertReverseDense (
			const SPATREF::COORDTRANS& trans,				//!< Transformation
			double tolerance										//!< Densification tolerance in output CRS unit
			);

		//! Convert subregion from double's to INT32's using floor method.
		ERRVALUE ConvertToLongFloor (
			SIMPLE_ARRAY<LPOLYGON>& PolygonArray,		//!< Array of polygons returned
			SIMPLE_ARRAY<LPOINT2D>& PointArray			//!< Array of points (referenced by polygons) returned
			) const;

		//! Convert subregion from double's to INT32's using floor method with scale and offset.
		ERRVALUE ConvertToLongFloor (
			SIMPLE_ARRAY<LPOLYGON>& PolygonArray,		//!< Array of polygons returned
			SIMPLE_ARRAY<LPOINT2D>& PointArray,			//!< Array of points (referenced by polygons) returned
			const DPOINT2D& scale,							//!< Scaling factors
			const DPOINT2D& offset							//!< Offset values added after scaling
			) const;

		//! Convert subregion from double's to INT32's, using floor with a near integer offset.
		ERRVALUE ConvertToLongNearInt (
			SIMPLE_ARRAY<LPOLYGON>& PolygonArray,
			SIMPLE_ARRAY<LPOINT2D>& PointArray
			) const;

		//! Convert subregion from double's to INT32's with rounding, removes duplicate points.
		ERRVALUE ConvertToLongRound (
			SIMPLE_ARRAY<LPOLYGON>& PolygonArray,
			SIMPLE_ARRAY<LPOINT2D>& PointArray
			) const;
	
		//! Copy subregion to specified target with filter.
		ERRVALUE CopyTo (
			SUBREGION2D& target,								//!< Target subregion
			COPYFILTER& filter								//!< Filter to determine which islands to copy
			) const;
	
		//! Dump coordinates to string.
		//! Though intended for debugging, not for serialization, the format is compatible
		//! with AssignFromXML when maximum precision is used.
		void DumpCoords (
			MISTRING& result,							//!< String to append to
			int indent,									//!< Amount to indent each coordinate
			bool maxprec = false						//!< Use maximum precision
			) const;

		//! Get the end of the list of POLYLINE's
		//! @return End of POLYLINE list, see Begin() for details
		CONST_ITERATOR End () const
			{ return (m_ItemList.End()); }

		//! Attempt to fit rectangle of specified size inside subregion.
		//! @return TRUE if rectangle fits, FALSE if not, < 0 if error.
		//! This function is typically used to generate a "label point" inside the subregion with the
		//! specified rectangle size determined from the desired label size.
		//! The QuantFactor controls the speed vs accuracy of this operation.  The smaller dimension of the
		//! rectangle to be fit will used with the quantization factor to scale coordinates.  Larger values
		//! will be more accurate, but the computation time increases approximately by the square of the
		//! quantization factor.  Larger factors will also require more memory.
		int FitRectangleInside (
			const DPOINT2D& size,					//!< Size of rectangle
			DPOINT2D& ptCenterRet,					//!< Point at center of rectangle returned rectangle fits, unchanged if not
			UINT32 QuantFactor = 10,				//!< Quantization factor, must be positive
			bool bUseFirstFound = false			//!< Use first found location instead of one farthest from edge
			) const;

		//! Generate a point inside of the subregion
		//! This does deal with the islands of the subregion, the point will not be in an island.
		ERRVALUE GeneratePointInside (
			DPOINT2D& PointInside
			) const;
			
		//! Generate a point inside of the subregion
		//! This does deal with the islands of the subregion, the point will not be in an island.
		//! If called multiple times for the same subregion, the points generated will not be the same.
		ERRVALUE GenerateRandomPointInside (
			DPOINT2D& PointInside
			) const;

		//! Get sub-region extents
		//! @return Extents of the sub-region
		const DRECT3D& GetExtents () const
			{ return (m_Extents); }
	
		//! Get sub-region extents
		//! @return Extents of the sub-region
		const DRECT2D& GetExtents2D () const
			{ return (m_Extents); }
	
		//! Get number of islands in the sub-region
		//! @return Number of islands in the sub-region
		INT32 GetNumIslands () const
			{ return (IsEmpty() ? 0 : m_ItemList.GetNumItems()-1); }
	
		//! Get XML representation.
		XMLNODE* GetXML (
			XMLNODE *parentnode,						//!< Parent node
			const char *nodename = "SubRegion"	//!< Node name
			) const;
			
		//! Intersect an specified rectangle area with the sub-region
		//! If the rectangle does not intersect with the sub-region area, 'this' will be set to empty
		ERRVALUE Intersect (
			const DRECT2D& Rect,
			MILIST<SUBREGION2D>& SubRegionList			//!< Filled in if operation creates more than one sub-region
			);

		//! Intersect an specified area against the sub-region
		//! If the polygon does not intersect the sub-region area, 'this' will be set to empty
		//! If the "PolyLine" passed in is invalid, this method will only use one of the 
		//! valid polygons it finds, otherwise the results will be in error.
		ERRVALUE Intersect (
			const POLYLINE& PolyLine,
			MILIST<SUBREGION2D>& SubRegionList			//!< Filled in if operation creates more than one sub-region
			);

		//! Intersect an specified area against the sub-region
		//! If the "SubRegion" does not intersect with the sub-region area, 'this' will be set to empty
		ERRVALUE Intersect (
			const SUBREGION2D& SubRegion,
			MILIST<SUBREGION2D>& SubRegionList			//!< Filled in if operation creates more than one sub-region
			);

		//! Intersect an specified area from the sub-region known to be valid
		//! A validated polygon is a polygon that does not intersect itself.
		//! If the polygon does not intersect the sub-region area, 'this' will be set to empty
		ERRVALUE IntersectValidated (
			const POLYLINE& PolyLine,
			MILIST<SUBREGION2D>& SubRegionList			//!< Filled in if operation creates more than one sub-region
			);
			
		//! Determine if the two subregions are equivalent 
		//! This method uses "Fuzzy" vertex comparisons and the direction and order of the elements are not relevant
		//! Specify a different threshold to allow greater distances
		//! @return 'True' if they are, 'False' if not
		bool IsEquivalentTo (
			const SUBREGION2D& rhs,
			double threshold = 0.0			//!< Default threshold is max absolute value of rectangle coordinate * 1.0E-13
			) const;
	
		//! Determine if the point falls inside the sub-region
		//! @return 'True' if point is inside the sub-region, 'false' if not
		bool IsPointInside (
			const DPOINT2D& point
			) const;
	
		//! Is the sub-region an empty sub-region
		bool IsEmpty (
			) const { return (!m_Extents.IsValid()); }

		//! Is the sub-region a simple rectangle region?			
		bool IsRectangle (
			) const { return (m_IsRectangle); }

		//! Remove all of the islands from the subregion
		void RemoveIslands ();
			
		//! Reverse the orientation of the polygons and islands in the subregion
		void Reverse ();

		//! Round X/Y values to specified interval.
		//! Note that rounding may result in self-intersections.
		void Round2D (
			double interval,						//!< Rounding interval
			ROUNDMODE mode							//!< ROUNDMODE_Nearest, ROUNDMODE_Up, ROUNDMODE_Down
			);

		//! Set the dimension of the points stored in 'this' as 2D or 3D vertices.
		void SetDimension (
			DIMENSION dim
			);

		//! Set subregion orientation
		//! Outer shell polygon is set to the orientation specified in respect to itself. 
		//! Islands are set in the opposite orientation in respect to themselves
		void SetOrientation (
			ORIENTATION Orientation
			);
			
		//! Remove an specified area from the sub-region
		//! If the island removes the entire sub-region area, 'this' will be set to empty
		//! If the "Island" passed in is invalid, this method will only use one of the 
		//! valid polygons it finds, otherwise the results will be in error.
		ERRVALUE Subtract (
			const POLYLINE& Island,
			MILIST<SUBREGION2D>& SubRegionList			//!< Filled in if operation creates more than one sub-region
			);

		//! Remove an specified area from the sub-region
		//! If the "SubRegion" removes the entire sub-region area, 'this' will be set to empty
		ERRVALUE Subtract (
			const SUBREGION2D& SubRegion,
			MILIST<SUBREGION2D>& SubRegionList			//!< Filled in if operation creates more than one sub-region
			);

		//! Remove an specified rectangle area from the sub-region
		//! If the island removes the entire sub-region area, 'this' will be set to empty
		ERRVALUE Subtract (
			const DRECT2D& IslandRect,
			MILIST<SUBREGION2D>& SubRegionList			//!< Filled in if operation creates more than one sub-region
			);

		//! Remove an specified area from the sub-region known to be valid
		//! A validated polygon is a polygon that does not intersect itself.
		//! If the island removes the entire sub-region area, 'this' will be set to empty
		ERRVALUE SubtractValidated (
			const POLYLINE& Island,
			MILIST<SUBREGION2D>& SubRegionList			//!< Filled in if operation creates more than one sub-region
			);
			
		//! Remove an specified area from the sub-region known to be valid
		//! A validated polygon is a polygon that does not intersect itself.
		//! If the island removes the entire sub-region area, 'this' will be set to empty
		ERRVALUE SubtractValidated (
			const POLYLINE& Island,
			MILIST<SUBREGION2D>& SubRegionList,			//!< Filled in if operation creates more than one sub-region
			double ConflateThreshold
			);
			
		//! Test the line given the test criteria against the sub-region
		//! @return "True" if the line meets the criteria, "false" if not
		bool TestLine (
			const POLYLINE& PolyLine,
			TESTCRITERIA TestCriteria
			) const;

		//! Test the line against the sub-region
		//! @return POLYLINE::COMPRESULT
		POLYLINE::COMPRESULT TestLine (
			const POLYLINE& PolyLine
			) const;

		//! Test the point given the test criteria against the sub-region
		//! @return "True" if the point meets the criteria, "false" if not
		bool TestPoint (
			const DPOINT2D& Point,
			TESTCRITERIA TestCriteria
			) const;

		//! Test the polygon given the test criteria against the sub-region
		//! @return "True" if the polygon meets the criteria, "false" if not
		bool TestPolygon (
			const POLYLINE& PolyLine,
			TESTCRITERIA TestCriteria
			) const;

		//! Test the polygon against the sub-region
		//! @return POLYLINE::COMPRESULT
		POLYLINE::COMPRESULT TestPolygon (
			const POLYLINE& PolyLine
			) const;

		//! Test the subregion given the test criteria against 'this'
		//! @return "True" if the subregion meets the criteria, "false" if not
		bool TestSubRegion (
			const SUBREGION2D& SubRegion,
			TESTCRITERIA TestCriteria
			) const;

		//! Test the subregion against 'this'
		//! @return POLYLINE::COMPRESULT
		POLYLINE::COMPRESULT TestSubRegion (
			const SUBREGION2D& SubRegion
			) const;

		//! Union (ADD) a rectangle to the sub-region
		ERRVALUE Union (
			const DRECT2D& Rect,
			MILIST<SUBREGION2D>& SubRegionList,			//!< Filled in if operation creates more than one sub-region
			POLYLINE::COMBINERESULT& CombineResult
			);

		//! Union (ADD) a Polygon to the sub-region
		ERRVALUE Union (
			const POLYLINE& PolyLine,
			MILIST<SUBREGION2D>& SubRegionList			//!< Filled in if operation creates more than one sub-region
			);

		//! Union (ADD) a sub-region to the sub-region
		ERRVALUE Union (
			const SUBREGION2D& SubRegion,
			MILIST<SUBREGION2D>& SubRegionList,			//!< Filled in if operation creates more than one sub-region
			POLYLINE::COMBINERESULT& CombineResult
			);

		//! Union (ADD) a polygon to the sub-region
		ERRVALUE UnionValidated (
			const POLYLINE& PolyLine,
			MILIST<SUBREGION2D>& SubRegionList,			//!< Filled in if operation creates more than one sub-region
			POLYLINE::COMBINERESULT& CombineResult
			);
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
	
		POLYLINELIST m_ItemList;
		DRECT3D m_Extents;
		bool m_IsRectangle;
		
		ERRVALUE ComputeBuffer (const double Distance, REGION2D& Buffer) const;
		void UpdateExtentsRect ();
		ERRVALUE UpdateFromShells (POLYLINELIST& PolyLineList, MILIST<SUBREGION2D>& SubRegionList);
	
		friend class RVC::VECTOR;			//!< For RVC::VECTOR::ReadSubRegion() and RVC::VECTOR::ReadRegion()
		friend class RVC::REGION;			//!< For RVC::REGION::GetRegion() and RVC::REGION::SetRegion()
		friend class REGION2D;				//!< For RVC::REGION::Assign()
		friend class RVC::REGIONHANDLE;
		friend class BINARYTOREGION2D;
		friend class SERIALIZER;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

typedef MILIST<SUBREGION2D> SUBREGIONLIST;
			

//! Container for generic 2D region in memory.
//! A region has three basic component types, point, line and subregion, and can have zero or more
//! of each type.  A region may also be represented as a simple rectangle.
//! The coordinate reference system SPATREF::COORDREFSYS for the component coordinates is contained also.
class GEOMLIBCLASSEXPORT REGION2D {
	public:

		typedef SUBREGIONLIST::CONST_ITERATOR CONST_ITERATOR;
		
		//! Filter for copying region.
		class GEOMLIBCLASSEXPORT COPYFILTER : public SUBREGION2D::COPYFILTER {
			public:
				//! Constructor.
				COPYFILTER ();
				//! Destructor.
				virtual ~COPYFILTER () = 0;
				//! Determine if line should be included.
				bool IsLineIncluded (
					const POLYLINE& line
					) { return (v_IsLineIncluded(line)); }
				//! Determine if point should be included.
				bool IsPointIncluded (
					const DPOINT2D& point
					) { return (v_IsPointIncluded(point)); }
				//! Determine if subregion should be included.
				bool IsSubRegionIncluded (
					const SUBREGION2D& subregion
					) { return (v_IsSubRegionIncluded(subregion)); }
			private:
				//! Override to determine if line should be included.
				//! Default implementation always returns true.
				virtual bool v_IsLineIncluded (
					const POLYLINE& line
					);
				//! Override to determine if point should be included.
				//! Default implementation always returns true.
				virtual bool v_IsPointIncluded (
					const DPOINT2D& point
					);
				//! Override to determine if subregion should be included.
				//! Default implementation always returns true.
				virtual bool v_IsSubRegionIncluded (
					const SUBREGION2D& subregion
					);
			};
	
		//! Default constructor, initializes to an invalid region		
		REGION2D (
			);
	
		//! Copy constructor
		REGION2D (
			const REGION2D& rhs
			);
	
		//! Constructor from DRECT2D
		REGION2D (
			const DRECT2D& rhs
			);

		//! Constructor from SUBREGION2D.
		REGION2D (
			const SUBREGION2D& SubRegion
			);
	
		//! Constructor from a list of polylines
		REGION2D (
			const POLYLINELIST&
			);
	
		//! Destructor
		~REGION2D (
			);
			
		//! Assignment
		REGION2D& operator= (
			const REGION2D& rhs
			);
			
		//! Assignment from DRECT2D.
		REGION2D& operator= (
			const DRECT2D& rhs
			);

		//! Assignment from DRECT3D.
		REGION2D& operator= (
			const DRECT3D& rhs
			);

		//! Assignment from SUBREGION2D.
		REGION2D& operator= (
			const SUBREGION2D& SubRegion
			);
			
		//! Add a line to the region without validating it.
		void AddLine (
			const POLYLINE& PolyLine
			);
	
		//! Add lines to region without validation.
		void AddLines (
			const POLYLINELIST& LineList
			);
	
		//! Add a point to the region without validating it.
		void AddPoint (
			const DPOINT3D& Point
			);

		//! Add points without validation.
		void AddPoints (
			const POLYLINE& points
			);
	
		//! Add a sub-region to the list without validating it.
		void AddSubRegion (
			const SUBREGION2D& SubRegion
			);
	
		//! Assign from a POLYLINE and an index array defining how the polyline is formatted
		ERRVALUE Assign (
			const POLYLINE& PolyLine,
			const SIMPLE_ARRAY<INT32>& Index			//!< Each entry is the number of points >0 is shell <0 is island
			);
			
		//! Assign from XMLNODE.
		ERRVALUE AssignFromXML (
			const XMLNODE *topnode
			);

		//! Get the beginning of the list of SUBREGION's.
		//! @return Iterator to the beginning of the subregion list
		CONST_ITERATOR Begin (
			) const { return (m_SubList.Begin()); }

		//! Clear the region
		void Clear (
			);
			
		//! Clear coordinate reference system.
		void ClearCoordRefSys (
			);
			
		//! Clip a polyline to a region, call the ClipTarget for each segment
		//! @return TRUE if part of the line is clipped, FALSE if not, < 0 error
		int ClipLine (
			POLYLINE& PolyLine,
			POLYLINE::CLIP& ClipTarget,
			CLIPMODE ClipMode = CLIPMODE_Inside
			) const;
			
		//! Compute the region area in region units
		double ComputeArea (
			) const;
	
		//! Compute the region area in region units but do not exclude islands in the calculation
		double ComputeAreaNoIslands (
			) const;
	
		//! Compute centroid of region.
		//! If region has subregions, only they will be used in calculation, with weighting by area.
		//! Otherwise if region has lines, only they will be used, with weighting by length.
		//! Otherwise points will be used with equal weighting.
		void ComputeCentroid (
			DPOINT3D& centroid					//!< Centroid returned
			) const;
	
		//! Compute difference between expansion of current region by Distance and current region if Distance is positive
		//! Compute difference between current region and shrinkage of current region by Distance if Distance is negative
		//! Points and lines would be processed and not copied to result
		ERRVALUE ComputeExpansion (
			const double Distance,					// Expansion (Shrinkage if negative) distance
			REGION2D& Expansion						// Expansion returned
			) const;

		//! Compute the region perimeter in region units
		//! This includes any lines that are part of the region
		double ComputePerimeter (
			) const;
	
		//! Compute the region perimeter using spatial reference measurement method.
		//! This includes any lines that are part of the region
		ERRVALUE ComputePerimeter (
			double& result,
			SPATREF::COORDREFSYS::MEASUREMETHOD method,
			UOM::IDNUM LengthUnitIdNum = UOM::IDNUM_Length_Meter
			) const;
	
		//! Compute the region perimeter in region units but do not include sub-region islands in the calculation
		//! This includes any lines that are part of the region
		double ComputePerimeterNoIslands (
			) const;
	
		//! Translate the region using an affine transformation
		//! If the COORDREFSYS of 'this' is set, it will be changed to "Undefined"
		void ConvertForward (
			const TRANS2D_AFFINE& taf
			);
	
		//! Translate the region using a mapgen transformation
		//! An error will occur if the COORDREFSYS of 'this' is not local and the TRANS2D_MAPGEN's input transformation is not identity or
		//! the COORDREFSYS of 'this' does not match the TRANS2D_MAPGEN's input COORDREFSYS.
		//! If the TRANS2D_MAPGEN's output transformation is identity, 'this' COORDREFSYS will be set to the TRANS2D_MAPGEN's output COORDREFSYS.
		//! otherwise it will be set to "Undefined".
		ERRVALUE ConvertForward (
			const TRANS2D_MAPGEN& tmg
			);
	
		//! Translate using SPATREF::COORDTRANS.
		//! Region coordinate reference system "GetCoordRefSys()' must equate to the trans.GetSourceCRS().
		//! The trans.GetTargetCRS() is assigned as the region's COORDREFSYS after the transformation.
		ERRVALUE ConvertForward (
			const SPATREF::COORDTRANS& trans					//!< Transformation
			);
	
		//! Translate the region using a mapgen transformation with densification
		//! An error will occur if the COORDREFSYS of 'this' is not local and the TRANS2D_MAPGEN's input transformation is not identity or
		//! the COORDREFSYS of 'this' does not match the TRANS2D_MAPGEN's input COORDREFSYS.
		//! If the TRANS2D_MAPGEN's output transformation is identity, 'this' COORDREFSYS will be set to the TRANS2D_MAPGEN's output COORDREFSYS.
		//! otherwise it will be set to "Undefined".
		ERRVALUE ConvertForwardDense (
			const TRANS2D_MAPGEN& tmg
			);
	
		//! Translate using SPATREF::COORDTRANS with densification.
		//! Region coordinate reference system "GetCoordRefSys()' must match the trans.GetSourceCRS().
		//! The trans.GetTargetCRS() is assigned as the region's COORDREFSYS after the transformation.
		ERRVALUE ConvertForwardDense (
			const SPATREF::COORDTRANS& trans,				//!< Transformation
			double tolerance										//!< Densification tolerance in output CRS unit
			);

		//! Translate the region using an affine transformation.
		//! If the COORDREFSYS of 'this' is set, it will be changed to "Undefined"
		void ConvertInverse (
			const TRANS2D_AFFINE& taf
			);
	
		//! Translate the region using a mapgen transformation.
		//! An error will occur if the COORDREFSYS of 'this' is not local and the TRANS2D_MAPGEN's output transformation is not identity or
		//! the COORDREFSYS of 'this' does not match the TRANS2D_MAPGEN's output COORDREFSYS.
		//! If the TRANS2D_MAPGEN's input transformation is identity, 'this' COORDREFSYS will be set to the TRANS2D_MAPGEN's input COORDREFSYS
		//! otherwise it will be set to "Undefined".
		ERRVALUE ConvertInverse (
			const TRANS2D_MAPGEN& tmg
			);
	
		//! Translate the region using a mapgen transformation with densification.
		//! An error will occur if the COORDREFSYS of 'this' is not local and the TRANS2D_MAPGEN's output transformation is not identity or
		//! the COORDREFSYS of 'this' does not match the TRANS2D_MAPGEN's output COORDREFSYS.
		//! If the TRANS2D_MAPGEN's input transformation is identity, 'this' COORDREFSYS will be set to the TRANS2D_MAPGEN's input COORDREFSYS.
		//! otherwise it will be set to "Undefined".
		ERRVALUE ConvertInverseDense (
			const TRANS2D_MAPGEN& tmg
			);
			
		//! Translate using SPATREF::COORDTRANS.
		//! Region coordinate reference system must equate to the trans.GetTargetCRS().
		//! The trans.GetSourceCRS() is assigned as the region's COORDREFSYS after the transformation.
		ERRVALUE ConvertReverse (
			const SPATREF::COORDTRANS& trans					//!< Transformation
			);
	
		//! Translate using SPATREF::COORDTRANS with densification.
		//! Region coordinate reference system must equate to the trans.GetTargetCRS().
		//! The trans.GetSourceCRS() is assigned as the region's COORDREFSYS after the transformation.
		ERRVALUE ConvertReverseDense (
			const SPATREF::COORDTRANS& trans,				//!< Transformation
			double tolerance										//!< Densification tolerance in output CRS unit
			);

		//! Convert	'this' to a different COORDREFSYS.
		ERRVALUE ConvertTo (
			const SPATREF::COORDREFSYS& DestCoordRefSys,
			double tolerance = 0.0
			);
	
		//! Copy region to specified target with filter.
		ERRVALUE CopyTo (
			REGION2D& target,									//!< Target subregion
			COPYFILTER& filter								//!< Filter to determine which subregions and islands to copy
			) const;
	
		//! Dump coordinates to string.
		//! Though intended for debugging, not for serialization, the format is compatible
		//! with AssignFromXML when maximum precision is used.
		void DumpCoords (
			MISTRING& result,							//!< String to append to
			int indent,									//!< Amount to indent each coordinate
			bool maxprec = false						//!< Use maximum precision
			) const;

		//! Get the end of the list of SUBREGION's
		//! @return End of SUBREGION list, see Begin() for details
		CONST_ITERATOR End (
			) const { return (m_SubList.End()); }

		//! Exclusive Union (XOR) RegionB with 'this'
		ERRVALUE ExclusiveUnion (
			const REGION2D& RegionB
			);
			
		//! Exclusive Union (XOR) RegionB with RegionA and place in 'this'
		//! Will convert 'this' to RegionA's SPATREF::COORDREFSYS
		ERRVALUE ExclusiveUnion (
			const REGION2D& RegionA,
			const REGION2D& RegionB
			);
			
		//! Expand current region by Distance if Distance is positive
		//! Shrink current region by Distance if Distance is negative
		//! Points and lines would be processed if expand and not copied to result
		ERRVALUE Expand (
			const double Distance					// Expansion (Shrinkage if negative) distance
			);

		//! Get the region's coordinate reference system
		//! @return SPATREF::COORDREFSYS of the region
		const SPATREF::COORDREFSYS& GetCoordRefSys (
			) const { return (m_CoordRefSys); }
			
		//! Get region extents
		//! @return Extents of the entire region
		const DRECT3D& GetExtents (
			) const { return (m_Extents); }
	
		//! Get region extents
		//! @return Extents of the entire region
		const DRECT2D& GetExtents2D (
			) const { return (m_Extents); }
	
		//! Get the POLYLINELIST of the line component of this region
		//! @return The line component list
		const POLYLINELIST& GetLineList (
			) const { return (m_LineList); }
		
		//! Get number of line components.
		//! @return Number of lines.
		INT32 GetNumLines (
			) const { return (m_LineList.GetNumItems()); }

		//! Get number of point components.
		//! @return Number of points
		INT32 GetNumPoints (
			) const { return (m_PointList.GetNumPoints()); }

		//! Get number of subregions
		//! @return Number of subregions
		INT32 GetNumSubRegions (
			) const { return (m_SubList.GetNumItems()); }

		//! Get the POLYLINE of the point component of this region
		//! @return The point component list
		const POLYLINE& GetPointList (
			) const { return (m_PointList); }
			
		//! Get the SUBREGION2D of the subregion component of this region
		//! @return The subregion component list
		const SUBREGIONLIST& GetSubRegionList (
			) const { return (m_SubList); }
	 
		//! Get XML representation.
		XMLNODE* GetXML (
			XMLNODE *parentnode,						//!< Parent node
			const char *nodename = "Region"		//!< Node name
			) const;
			
		//! Determine if has any point components.
		bool HasPoints (
			) const { return (m_PointList.GetNumPoints() > 0); }
			
		//! Determine if has any line components.
		bool HasLines (
			) const { return (!m_LineList.IsEmpty()); }
			
		//! Determine if has any subregion components.
		bool HasSubRegions (
			) const { return (!m_SubList.IsEmpty()); }
			
		//! Intersect rectangle with 'this'
		ERRVALUE Intersect (
			const DRECT2D& Rect
			);
			
		//! Intersect a polygon with 'this'
		ERRVALUE Intersect (
			const POLYLINE& PolyLine
			);
			
		//! Intersect a sub-region with 'this'
		ERRVALUE Intersect (
			const SUBREGION2D& SubRegion
			);
			
		//! Intersect RegionB with 'this'
		ERRVALUE Intersect (
			const REGION2D& RegionB
			);
			
		//! Intersect RegionB with RegionA and place in 'this'
		//! Will convert 'this' to RegionA's SPATREF::COORDREFSYS
		ERRVALUE Intersect (
			const REGION2D& RegionA,
			const REGION2D& RegionB
			);
			
		//! Intersect a validated polygon with 'this'
		ERRVALUE IntersectValidated (
			const POLYLINE& PolyLine
			);
			
		//! Is the region an empty region
		bool IsEmpty (
			) const { return (!m_Extents.IsValid()); }

		//! Determine if the two regions are equivalent 
		//! This method uses "Fuzzy" vertex comparisons and the direction and order of the elements are not relevant
		//! Specify a different threshold to allow greater distances
		//! @return 'True' if they are, 'False' if not
		bool IsEquivalentTo (
			const REGION2D& rhs,
			double threshold = 0.0			//!< Default threshold is max absolute value of rectangle coordinate * 1.0E-13
			) const;
	
		//! Determine if the point falls inside the region
		//! @return 'True' if point is inside region, 'false' if not
		bool IsPointInside (
			const DPOINT2D& point
			) const;
			
		//! Is the region a simple rectangle region?
		bool IsRectangle (
			) const { return (m_IsRectangle); }

		//! Normalize region to range used for CRS if angular axis.
		//! Will handle wrapping for X (longitude) axis and clipping for Y (latitude) axis.
		//! If CRS does not require normalization region will be unchanged.
		ERRVALUE NormalizeToCRS (
			);
			
		//! Remove all of the islands from the region
		void RemoveIslands ();
			
		//! Reverse the orientation of the subregions in the region
		void Reverse ();

		//! Round X/Y values to specified interval.
		//! Note that rounding may result in self-intersections.
		void Round2D (
			double interval,						//!< Rounding interval
			ROUNDMODE mode							//!< ROUNDMODE_Nearest, ROUNDMODE_Up, ROUNDMODE_Down
			);

		//! Set the region's coordinate reference system
		//! Does not do any transformations of the region, 
		//! assumes that the region is in the system specified
		void SetCoordRefSys (
			const SPATREF::COORDREFSYS& CoordSysRef
			) { m_CoordRefSys = CoordSysRef; }

		//! Set the dimension of the points stored in 'this' as 2D or 3D vertices.
		void SetDimension (
			DIMENSION dim
			);

		//! Set region orientation
		//! Each subregion is set using SUBREGION2D::SetOrientation()
		void SetOrientation (
			ORIENTATION Orientation
			);
			
		//! Set the Z init and Z last fields of the extents box			
		void SetZRange (
			const DOUBLE_RANGE& range
			);
			
		//! Subtract rectangle from 'this'
		ERRVALUE Subtract (
			const DRECT2D& Rect
			);
			
		//! Subtract a polygon from 'this'
		ERRVALUE Subtract (
			const POLYLINE& PolyLine
			);
			
		//! Subtract a sub-region from 'this'
		ERRVALUE Subtract (
			const SUBREGION2D& SubRegion
			);
			
		//! Subtract RegionB from 'this'
		ERRVALUE Subtract (
			const REGION2D& RegionB
			);
			
		//! Subtract RegionB from RegionA and place in 'this'
		//! Will convert 'this' to RegionA's SPATREF::COORDREFSYS
		ERRVALUE Subtract (
			const REGION2D& RegionA,
			const REGION2D& RegionB
			);
			
		//! Subtract a validated polygon from 'this'
		ERRVALUE SubtractValidated (
			const POLYLINE& PolyLine
			);
			
		//! Test the line given the test criteria against the region
		//! @return "True" if the line meets the criteria, "false" if not
		bool TestLine (
			const POLYLINE& PolyLine,
			TESTCRITERIA TestCriteria
			) const;

		//! Test the point given the test criteria against the sub-region
		//! @return "True" if the point meets the criteria, "false" if not
		bool TestPoint (
			const DPOINT2D& Point,
			TESTCRITERIA TestCriteria
			) const;

		//! Test the polygon given the test criteria against the region
		//! @return "True" if the polygon meets the criteria, "false" if not
		bool TestPolygon (
			const POLYLINE& PolyLine,
			TESTCRITERIA TestCriteria
			) const;

		//! Test the sub-region given the test criteria against the region
		//! @return "True" if the sub-region meets the criteria, "false" if not
		bool TestSubRegion (
			const SUBREGION2D& SubRegion,
			TESTCRITERIA TestCriteria
			) const;

		//! Test the 'Region' region given the test criteria against 'this' region
		//! @return "True" if the 'Region' region meets the criteria, "false" if not
		bool TestRegion (
			const REGION2D& Region,
			TESTCRITERIA TestCriteria
			) const;

		//! Union (ADD) rectangle to 'this'
		ERRVALUE Union (
			const DRECT2D& Rect
			);
			
		//! Union (ADD) a polygon to 'this'
		ERRVALUE Union (
			const POLYLINE& PolyLine
			);
			
		//! Union (ADD) a sub-region to the region
		ERRVALUE Union (
			const SUBREGION2D& SubRegion
			);
			
		//! Union (ADD) RegionB to RegionA and place in 'this'
		//! Will convert 'this' to RegionA's SPATREF::COORDREFSYS
		ERRVALUE Union (
			const REGION2D& RegionA,
			const REGION2D& RegionB
			);
			
		//! Union (ADD) RegionB to 'this'
		ERRVALUE Union (
			const REGION2D& RegionB
			);
			
		//! Union (ADD) a validated polygon to 'this'
		ERRVALUE UnionValidated (
			const POLYLINE& PolyLine
			);
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
	
		SUBREGIONLIST m_SubList;
		POLYLINELIST m_LineList;
		POLYLINE m_PointList;
		DRECT3D m_Extents;
		SPATREF::COORDREFSYS m_CoordRefSys;
		bool m_IsRectangle;
		
		void AppendDisjointRegion (const REGION2D& rhs);
		ERRVALUE ComputeBuffer (const double Distance, REGION2D& Buffer) const;
		bool UnionRectangle (const DRECT2D& Rect);
		void UpdateExtentsRect ();
	
		class INTERSECTREGION;
		friend class INTERSECTREGION;
		friend class RVC::VECTOR;				//!< For RVC::VECTOR::ReadRegion()
		friend class RVC::REGION;				//!< For RVC::REGION::GetRegion() and RVC::REGION::SetRegion()
		friend class RVC::CAD;					//!< For RVC::CAD::v_ReadNextRegion()
		friend class RVC::REGIONHANDLE;
		friend class BINARYTOREGION2D;
		friend class SERIALIZER;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#undef GEOMLIBCLASSEXPORT

#endif		// INC_MI32_REGION2D_H

