/**
 * mitls.h - Definition for Thread local storage code
 *
 * \if NODOC
 * $Id: thrlcstr.h_v 1.15 2007/10/17 22:54:22 scowan Exp $
 *
 * $Log: thrlcstr.h_v $
 * Revision 1.15  2007/10/17 22:54:22  scowan
 * Added example code to template class.
 *
 * Revision 1.14  2007/09/06 14:44:58  scowan
 * Added inclusion guard.
 *
 * Revision 1.13  2007/09/05 22:10:22  scowan
 * Undefine the dll spec at end of include file.
 *
 * Revision 1.12  2007/02/02 23:39:10  scowan
 * Moved previous standalone thread classes into mithread namespace
 *
 * Revision 1.11  2006/02/16 16:37:39  dwilliss
 * changed an ifdef to use new define instead of a deprecated one
 *
 * Revision 1.10  2005/10/25 13:50:23  scowan
 * Updated changes from 7.1.
 *
 * Revision 1.9  2005/10/24 23:06:26  scowan
 * Added comments and code.
 *
 * Revision 1.8  2005/09/21 19:36:25  scowan
 * Fixed apple compiler issue.
 *
 * Revision 1.7  2005/04/26 20:55:59  scowan
 * Fixed for unix.
 *
 * Revision 1.6  2005/04/26 20:45:38  scowan
 * FIxed BASe class issues.
 *
 * Revision 1.5  2005/04/26 20:19:11  scowan
 * Modified to work on all platforms.
 *
 * Revision 1.4  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.3  2002/05/17 19:46:05  dwilliss
 * Export the class even on the Mac using their non-standard way
 *
 * Revision 1.2  2001/12/12 16:33:37  scowan
 * Created template class to remove cast.
 *
 * Revision 1.1  2001/12/11 23:04:58  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_THRLCSTR_H
#define	INC_MI32_THRLCSTR_H

#ifndef	INC_MI32_STDDEFNS_H
	#include <mi32/stddefns.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

namespace MITHREAD {

//! The THREADLOCALSTORAGE class manages Thread Local Storage for multiple platforms.  This class is mainly used as a static global 
//! This class is to be used ONLY from a local static variable in a static function or method.
//! Any other use will cause memory leaks and / or process instability.
//! It is expected that the THREADLOCALSTORAGE_T template class be used for most TLS needs
class CLASSLIBEXPORT LOCALSTORAGE {
	public:
		//! Interface class to delete the class when the thread local storage is freed.
		//! See documentation on LOCALSTORAGE_T<> for use of this class.
		class CLASSLIBEXPORT BASE {			
			protected:
				BASE ();
				virtual ~BASE () = 0;
				
			friend class LOCALSTORAGE;
			};
	
	protected:
		// Default ctor.  Obtain index for thread local data
		LOCALSTORAGE (
			);
			
		//! Default dtor
		virtual ~LOCALSTORAGE (
			);
			
		//! Set the BASE* value stored for the current thread
		void SetValue (
			BASE* data
			);
			
		//! Get the BASE* value stored for the current thread
		//! @return Thread Local Storage of BASE*
		BASE* GetValue (
			) const;
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Used by implementation for DLL-safe private members.
		class PRIV;
		PRIV *m_priv;
		
		#if defined(UNIX)
		static void dtorFunc (void* data);
		#else
		static void dtorFunc (void* cb, void* data);
		#endif
		#endif //!< GENERATING_DOXYGEN_OUTPUT
	};
	

//! Template class to assist in handling thread local storage data
//! This class is to be used ONLY from a local static variable in a static function or method.
//! Any other use will cause memory leaks and / or process instability.
//! @code
//!	//! Thread local storage container class for thread error state
//! 	class GETERRSTRDATA : public MITHREAD::LOCALSTORAGE::BASE {
//! 		public:
//! 			GETERRSTRDATA (
//! 				) {}
//! 				
//! 			virtual ~GETERRSTRDATA (
//! 				) {}
//!				
//! 			ERRORSTATE& GetErrorStateRef (
//! 				) { return (m_ErrorState); }
//! 
//! 			static void TLSSystemClose (
//! 				) { s_ErrorSystemDisabled = true; }			// Do not allow any more calls to the error system
//!		
//! 		private:
//!			ERRORSTATE m_ErrorState;
//! 		};
//!
//!
//!	ERRORSTATE& ERRORSTATE::GetThreadErrStateRef (
//!		) {
//!		static MITHREAD::LOCALSTORAGE_T<GETERRSTRDATA> s_ErrTLS;
//!	 
//!		return (s_ErrTLS.GetValue()->GetErrorStateRef());
//!		}
//! @endcode
template <class _CT> class LOCALSTORAGE_T : public LOCALSTORAGE {
	public:
		//! Default virtual dtor
		//! Makes a call to a 'static' method on the template class called TLSSystemClose().
		//! There is no default method for this, nor can it be virtual or non-static.  This tells the
		//! class in question that the dtor for the THREADLOCALSTORAGE class is being called.  The error
		//! reporting system uses this to determine that it cannot handle any more errors.
		virtual ~LOCALSTORAGE_T (
			) {
			_CT::TLSSystemClose();
			}

		//! Get the TLS data from the presumably global variable.
		//! I considered replacing this with operator->(), but that would cause more overhead than simply 
		//! getting the pointer and using it
		//! @return '_T*' defined on the declaration line
		_CT *GetValue (
			) { 
			BASE* data = LOCALSTORAGE::GetValue();
			if (data == 0) {
				data = new _CT;
				SetValue(data);
				}
			return (static_cast<_CT*>(data));
			}
	};
	
}	// End of MITHREAD namespace

#undef CLASSLIBEXPORT
	
#endif		//!<	INC_MI32_THRLCSTR_H
