/**
 * \file rvc/geometrc.h
 * \brief interface for the RVC::GEOMETRIC class.
 *
 * \if NODOC
 * $Id: geometrc.h_v 1.111 2013/05/07 19:47:47 mju Exp $
 *
 * $Log: geometrc.h_v $
 * Revision 1.111  2013/05/07 19:47:47  mju
 * Remove getobjfilter_hasnoimpliedgeoref as unused.
 *
 * Revision 1.110  2013/05/02 21:52:25  mju
 * Add extentsObjItem to elemselectparms.
 *
 * Revision 1.109  2013/01/10 16:22:52  mju
 * Add defaults for findClosestElement.
 *
 * Revision 1.108  2013/01/08 16:04:59  mju
 * Add extractflag_IgnoreLineTables.
 *
 * Revision 1.107  2012/10/09 15:03:11  mju
 * Add "separate" to setupElemTransList.
 *
 * Revision 1.106  2012/07/25 17:30:15  mju
 * Add elemselectparms::addBorder member.
 *
 * Revision 1.105  2012/07/25 17:12:44  mju
 * Add elemselectparms::separation.
 *
 * Revision 1.102  2012/05/14 22:37:21  mju
 * Add labelgenerator.
 * Change label to not have to contain textstyle / frameparms and add elemtype.
 *
 * Revision 1.101  2012/05/11 17:49:53  mju
 * Change label class to have access/set methods, separate baseline and defriend cad/shape/vector.
 *
 * Revision 1.99  2011/12/23 15:36:55  mju
 * Use labelframeparms hdr and remove others that are not needed for this hdr.
 *
 * Revision 1.96  2010/04/22 20:03:50  mju
 * Add findElementSet using unsignedset.
 *
 * Revision 1.95  2010/04/19 16:20:45  mju
 * Add searchfilter_unsignedset.
 * Add searchresult::getElemSet using unsignedset.
 *
 * Revision 1.93  2008/10/01 20:01:14  scowan
 * Added multi table support cap flag.
 *
 * Revision 1.85  2006/11/01 22:58:47  dfriberg
 * Make extents calculation include leader lines as an option for labels
 *
 * Revision 1.77  2005/10/20 17:00:36  scowan
 * Quadtree and geometric interface changes supported.
 * Support for new quadtree system required changes to virtual method tables.
 *
 * Revision 1.1  2000/10/02 17:39:43  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_GEOMETRC_H
#define INC_RVC_GEOMETRC_H

#ifndef INC_RVC_GEOSPATL_H
	#include <rvc/geospatl.h>
#endif

#ifndef INC_RVC_STYLETYPE_H
	#include <rvc/styletype.h>
#endif

#ifndef INC_MI32_LABELFRAMEPARMS_H
	#include <mi32/labelframeparms.h>
#endif

#ifndef INC_RVC_ELEMENT_H
	#include <rvc/element.h>
#endif

#ifndef INC_RVC_QUADTREE_H
	#include <rvc/quadtree.h>
#endif

#ifndef INC_MI32_REGION2D_H
	#include <mi32/region2d.h>
#endif

#ifndef INC_MI32_TRANS2D_AFFINE_H
	#include <mi32/trans2daffine.h>
#endif

#ifndef INC_MI32_BITSET_H
	#include <mi32/bitset.h>
#endif

#ifndef INC_MI32_ELEMTYPESET_H
	#include <mi32/elemtypeset.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward Declarations
class TRANS2D_MAPGEN;
class BITSET_DEQUE;
class UNSIGNEDSET;
namespace MGD {
	class CONTEXT;
	}
namespace SML {
	class CONTEXT;
	}
namespace RVC {
	struct ANYCALLBACKSTRUCT;
	class CAD;
	class ELEMENT;
	class SHAPE;
	class SPATIALTREE;
	class STYLE;
	class VECTOR;
	}
#endif


namespace RVC {

//=====================================================================================================================

//! Base for "geometric" objects.
class GEOMETRIC : public GEOSPATIAL {
	public:

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class ELEMSELECTFILTER;
		class ELEMSELECTPARMS;
		class ELEMTRANSLIST;
		class GETOBJFILTER_HASMINIMUMELEMENTS;
		class GETOBJFILTER_ISDISPLAYABLE;
		class GETOBJFILTER_ISNOTEMPTY;
		class GETOBJFILTER_ISSELECTABLE;
		class MAKEPARMSBASE;
		class LABEL;
		class OBSERVER;
		class SEARCHFILTER;
		class SEARCHFILTER_BITSET;
		class SEARCHFILTER_UNSIGNEDSET;
		class SEARCHFILTER_ELEMTRANSLIST;
		class SEARCHRESULT;
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! General label container.
		class LABEL {
			public:

				enum BASELINEFIT {
					BASELINEFIT_Straight,
					BASELINEFIT_Exact,
					BASELINEFIT_Spline
					};

				//! Default constructor.
				LABEL ();

				//! Copy constructor.
				LABEL (
					const LABEL& rhs
					);

				//! Destructor.
				~LABEL ();

				//! Assignment operator.
				LABEL& operator= (
					const LABEL& rhs
					);

				//! Access leader line list (non-const).
				//! \internal
				POLYLINELIST& AccessLeaderLineList ()
					{ return (m_LeaderLineList); }

				//! Access frame parameters (non-const).
				//! \internal
				LABELFRAMEPARMS& AccessFrameParms ();

				//! Access text style (non-const).
				//! \internal
				TEXTSTYLE& AccessTextStyle ();

				//! Add leader line.
				void AddLeaderLine (
					const POLYLINE& LeaderLine
					) { m_LeaderLineList.PushBack(LeaderLine); }

				//! Attach frame parameters.
				void AttachFrameParms (
					LABELFRAMEPARMS *& pFrameParms,		//!< Pointer to frame parameters, if take ownership will be cleared, otherwise must remain valid
					bool TakeOwnership						//!< Whether to take ownership of pointer
					);

				//! Attach text style.
				void AttachTextStyle (
					TEXTSTYLE *& pTextStyle,				//!< Pointer to text style, if take ownership will be cleared, otherwise must remain valid
					bool TakeOwnership						//!< Whether to take ownership of pointer
					);

				//! Clear label contents.
				void Clear ();

				//! Compute the extents of this label.
				//! If the height specified in the text style is not in SCREENUNIT_Object units, 
				//! then this method will return invalid extents for the label.
				ERRVALUE ComputeExtents (
					DRECT3D& extents,
					bool IncludeLeaders = true
					) const;
					
				//! Get label baseline.
				//! @return Label baseline points
				const POLYLINE& GetBaseline () const
					{ return (m_BaseLine); }

				//! Get label baseline fit
				//! @return Label baseline fit
				BASELINEFIT GetBaselineFit () const
					{ return (static_cast<BASELINEFIT>(m_BaselineFit)); }

				//! Get general element type being labeled.
				ELEMTYPE GetElemType () const
					{ return (static_cast<ELEMTYPE>(m_ElemType)); }

				//! Get the label frame parameters.
				//! @return The label frame parameters
				const LABELFRAMEPARMS& GetFrameParms () const
					{ return (*m_pFrameParms); }

				//! Get number of leader lines.
				INT32 GetNumLeaderLines () const
					{ return (m_LeaderLineList.GetNumItems()); }

				//! Get label text style.
				//! \deprecated
				DEPRECATED_MSG("Use GetTextStyle instead") const TEXTSTYLE& GetStyle () const
					{ return (*m_pTextStyle); }

				//! Get label text string.
				//! @return Label string
				const MISTRING& GetText () const
					{ return (m_Text); }

				//! Get text height.
				double GetTextHeight () const
					{ return (m_TextHeight); }

				//! Get label text style.
				const TEXTSTYLE& GetTextStyle () const
					{ return (*m_pTextStyle); }

				//! Get (const) iterator at beginning of leader list.
				POLYLINELIST::CONST_ITERATOR LeaderLineBegin () const
					{ return (m_LeaderLineList.Begin()); }
			
				//! Get (const) iterator at end of leader list.
				POLYLINELIST::CONST_ITERATOR LeaderLineEnd () const
					{ return (m_LeaderLineList.End()); }

				//! Set baseline points.
				void SetBaseLine (
					const POLYLINE& BaseLine
					) { m_BaseLine = BaseLine; }

				//! Set baseline fit.
				void SetBaselineFit (
					BASELINEFIT BaselineFit
					) { m_BaselineFit = static_cast<UINT8>(BaselineFit); }

				//! Set general element type being labeled.
				void SetElemType (
					ELEMTYPE ElemType					//!< Element type, typically Point, Line or Polygon
					) { m_ElemType = static_cast<UINT8>(ElemType); }

				//! Set frame parameters.
				void SetFrameParms (
					const LABELFRAMEPARMS& FrameParms
					) { AccessFrameParms() = FrameParms; }

				//! Set text string.
				void SetText (
					const MISTRING& Text
					) { m_Text = Text; }

				//! Set text height.
				void SetTextHeight (
					double TextHeight
					) { m_TextHeight = TextHeight; }

				//! Set text style.
				void SetTextStyle (
					const TEXTSTYLE& TextStyle
					) { AccessTextStyle() = TextStyle; }
			
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				MISTRING m_Text;
				POLYLINE m_BaseLine;
				POLYLINELIST m_LeaderLineList;
				TEXTSTYLE *m_pTextStyle;
				LABELFRAMEPARMS *m_pFrameParms;
				double m_TextHeight;
				UINT8 m_BaselineFit;
				UINT8 m_ElemType;
				bool m_OwnTextStyle;
				bool m_OwnFrameParms;

				static LABELFRAMEPARMS* GetSharedFrameParms ();
				static TEXTSTYLE* GetSharedTextStyle ();

				void Release ();
				#endif
			};	// End LABEL
		typedef MILIST<LABEL> LABELLIST;

		//! This is the base class for the LINE, POINT, POLYGON, and LABEL iterators		
		class ITERATOR {
			public:
				//! Default constructor
				ITERATOR (
					) : m_GeoObj(0) {}
					
				//! Internal constructor
				ITERATOR (
					RVC::GEOMETRIC* GeoObj
					) : m_GeoObj(GeoObj) {}
					
				//! Copy constructor
				ITERATOR (				
					const ITERATOR& rhs
					) : m_GeoObj(rhs.m_GeoObj), m_Element(rhs.m_Element) {}

				//! Dereference operator, returns reference to templated item					
				const RVC::ELEMENT& operator* (
					) const { return (m_Element); }

				//! Dereference operator, returns pointer to templated item
				const RVC::ELEMENT* operator-> (
					) const { return (&**this); }

				bool operator== (				//! Equality operator
					const ITERATOR& rhs
					) const { return (m_GeoObj == rhs.m_GeoObj && m_Element == rhs.m_Element); }

				bool operator!= (				//! Inequality operator
					const ITERATOR& rhs
					) const { return (!(*this == rhs)); }

			protected:
				GEOMETRIC* m_GeoObj;
				RVC::ELEMENT m_Element;
			};

		class LABELITERATOR : public ITERATOR {
			public:
				//! Default constructor
				LABELITERATOR (
					) : ITERATOR() {}

				//! Internal constructor
				LABELITERATOR (
					RVC::GEOMETRIC* GeoObj,
					DIMENSION dim,
					STYLE* StyleObj
					) : ITERATOR(GeoObj), m_dim(dim), m_StyleObj(StyleObj) {
					m_GeoObj->ReadNextLabel(m_Element, m_LabelList, m_dim, m_StyleObj);
					}
					
				//! Internal constructor
				LABELITERATOR (
					RVC::GEOMETRIC* GeoObj
					) : ITERATOR(GeoObj) {}
					
				//! Copy constructor
				LABELITERATOR (				
					const LABELITERATOR& rhs
					) : 
					ITERATOR(rhs), 
					m_LabelList(rhs.m_LabelList), 
					m_dim(rhs.m_dim),
					m_StyleObj(rhs.m_StyleObj)
					{}

				//! Pre-increment operator.
				LABELITERATOR& operator++ (
					) {
					m_GeoObj->ReadNextLabel(m_Element, m_LabelList, m_dim, m_StyleObj);
					return (*this);
					}
					
				const LABELLIST& GetLabelList (
					) const { return (m_LabelList); }
					
			private:
				LABELLIST m_LabelList;
				DIMENSION m_dim;
				STYLE* m_StyleObj;
			};
			
		class LINEITERATOR : public ITERATOR {
			public:
				//! Default constructor
				LINEITERATOR (
					) : ITERATOR() {}

				//! Internal constructor
				LINEITERATOR (
					RVC::GEOMETRIC* GeoObj,
					DIMENSION dim
					) : ITERATOR(GeoObj), m_dim(dim) {
					m_GeoObj->ReadNextLine(m_Element, m_LineList, m_dim);
					}
					
				//! Internal constructor
				LINEITERATOR (
					RVC::GEOMETRIC* GeoObj
					) : ITERATOR(GeoObj) {}
					
				//! Copy constructor
				LINEITERATOR (				
					const LINEITERATOR& rhs
					) : 
					ITERATOR(rhs), 
					m_LineList(rhs.m_LineList), 
					m_dim(rhs.m_dim)
					{}

				//! Pre-increment operator.
				LINEITERATOR& operator++ (
					) {
					m_GeoObj->ReadNextLine(m_Element, m_LineList, m_dim);
					return (*this);
					}
					
				const POLYLINELIST& GetLineList (
					) const { return (m_LineList); }
					
			private:
				POLYLINELIST m_LineList;
				DIMENSION m_dim;
			};
			
		class NODEITERATOR : public ITERATOR {
			public:
				//! Default constructor
				NODEITERATOR (
					) : ITERATOR() {}

				//! Internal constructor
				NODEITERATOR (
					RVC::GEOMETRIC* GeoObj,
					DIMENSION		// dim
					) : ITERATOR(GeoObj) {
					m_GeoObj->ReadNextNode(m_Element, m_Node);
					}
					
				//! Internal constructor
				NODEITERATOR (
					RVC::GEOMETRIC* GeoObj
					) : ITERATOR(GeoObj) {}
					
				//! Copy constructor
				NODEITERATOR (				
					const NODEITERATOR& rhs
					) : 
					ITERATOR(rhs), 
					m_Node(rhs.m_Node) 
					{}

				//! Pre-increment operator.
				NODEITERATOR& operator++ (
					) {
					m_GeoObj->ReadNextNode(m_Element, m_Node);
					return (*this);
					}
					
				const DPOINT3D& GetNode (
					) const { return (m_Node); }
					
			private:
				DPOINT3D m_Node;
			};
			
		class POINTITERATOR : public ITERATOR {
			public:
				//! Default constructor
				POINTITERATOR (
					) : ITERATOR() {}

				//! Internal constructor
				POINTITERATOR (
					RVC::GEOMETRIC* GeoObj,
					DIMENSION dim
					) : ITERATOR(GeoObj), m_dim(dim) {
					m_GeoObj->ReadNextPoint(m_Element, m_PointList, m_dim);
					}
					
				//! Internal constructor
				POINTITERATOR (
					RVC::GEOMETRIC* GeoObj
					) : ITERATOR(GeoObj) {}
					
				//! Copy constructor
				POINTITERATOR (				
					const POINTITERATOR& rhs
					) : 
					ITERATOR(rhs), 
					m_PointList(rhs.m_PointList), 
					m_dim(rhs.m_dim)
					{}

				//! Pre-increment operator.
				POINTITERATOR& operator++ (
					) {
					m_GeoObj->ReadNextPoint(m_Element, m_PointList, m_dim);
					return (*this);
					}
					
				const POLYLINE& GetPointList (
					) const { return (m_PointList); }
					
			private:
				POLYLINE m_PointList;
				DIMENSION m_dim;
			};
			
		class POLYGONITERATOR : public ITERATOR {
			public:
				//! Default constructor
				POLYGONITERATOR (
					) : ITERATOR() {}

				//! Internal constructor
				POLYGONITERATOR (
					RVC::GEOMETRIC* GeoObj,
					DIMENSION dim
					) : ITERATOR(GeoObj), m_dim(dim) {
					m_GeoObj->ReadNextRegion(m_Element, m_Region, m_dim);
					}
					
				//! Internal constructor
				POLYGONITERATOR (
					RVC::GEOMETRIC* GeoObj
					) : ITERATOR(GeoObj) {}
					
				//! Copy constructor
				POLYGONITERATOR (				
					const POLYGONITERATOR& rhs
					) : 
					ITERATOR(rhs), 
					m_Region(rhs.m_Region), 
					m_dim(rhs.m_dim)
					{}

				//! Pre-increment operator.
				POLYGONITERATOR& operator++ (
					) {
					m_GeoObj->ReadNextRegion(m_Element, m_Region, m_dim);
					return (*this);
					}
					
				const REGION2D& GetRegion (
					) const { return (m_Region); }
					
			private:
				REGION2D m_Region;
				DIMENSION m_dim;
			};
	
		enum CAPABILITY {
			CAPABILITY_None = 				0,
			CAPABILITY_OwnSearchMethods = 512,	//!< Object has its own search methods (Oracle, MySQL, ...)
			CAPABILITY_CanAddToEnd,					//!< Object can be modified by adding elements to the end of the object
			CAPABILITY_CanChangeElements,			//!< Existing object elements can be modified
			CAPABILITY_HasStyleByElement,			//!< Each element can have its own style settings
			CAPABILITY_ElementHasExtents,			//!< Each element has extents stored with it, except labels
			CAPABILITY_HasLabels,		  			//!< Object has labels
			CAPABILITY_LabelHasExtents,  			//!< Each label has extents stored with it
			CAPABILITY_CanDeleteLastElement,		//!< Can delete the last element in the set
			CAPABILITY_CanDeleteAnyElement,		//!< Can delete any element in the set
			CAPABILITY_MultipleTableSupport,		//!< Format can support any number of tables
			CAPABILITY_HasSpatialTrees = 766,	//!< Object can support spatial tree indexes
			CAPABILITY_HasQuadTrees = 767			//!< Object can support quad tree indexes
			};

		enum COMPONENT {
			COMPONENT_Point = 		0x01,
			COMPONENT_Line = 			0x02,
			COMPONENT_Polygon = 		0x04,
			COMPONENT_Label = 		0x08,
			COMPONENT_Node =	 		0x10,
			COMPONENT_Geometric = 	(COMPONENT_Point + COMPONENT_Line + COMPONENT_Polygon),
			COMPONENT_All = 			(COMPONENT_Geometric + COMPONENT_Label + COMPONENT_Node)
			};

		enum EXTRACTFLAGS {
			EXTRACTFLAG_None =						0,
			EXTRACTFLAG_RegionMethod =				0x00000001,		//!< SelectMethod is a REGION2D
			EXTRACTFLAG_RemExRecords =				0x00000002,		//!< Remove Excess (Unattached) database records
			EXTRACTFLAG_RemStdAttTables =			0x00000004,		//!< Remove Standard Attribute database tables
			EXTRACTFLAG_IgnoreMissingLines =		0x00000008,		//!< Ignore missing lines from extraction operation, polygon and node line lists
			EXTRACTFLAG_TableJoinNameStruct =	0x00000010,		//!< Join tables in Append functions if names alike and compatable structures
			EXTRACTFLAG_TableJoinStruct =			0x00000020,		//!< Join tables in Append functions if compatable structures
			EXTRACTFLAG_TableRemoveDupRec =		0x00000040,		//!< Do not copy over records if exists in dest table
			EXTRACTFLAG_IgnorePolygonTables =	0x00000080,		//!< Do not copy over polygon databases
			EXTRACTFLAG_RemoveTables =				0x00000100,		//!< Remove destination tables
			EXTRACTFLAG_CopyHyperIndexSub =		0x00000200,		//!< Copy Hyper index subobjects
			EXTRACTFLAG_DensifyPoints =			0x00000400,		//!< Densify line points using transformation parameters
			EXTRACTFLAG_CreateNewTables =			0x00000800,		//!< Copy records and tables instead of maintaining links.
			EXTRACTFLAG_IgnoreLineTables =		0x00001000,		//!< Don't copy line databases
			};

		enum POINTTYPE {
			POINTTYPE_2DXY = 1,				//!< 2-D coordinates, X and Y given (default)
			POINTTYPE_3DXY = 4,				//!< 3-D coordinates, X, Y given, Z constant
			POINTTYPE_3DXYZ = 7				//!< 3-D coordinates, X, Y and Z given
			};

		//! Default constructor
		GEOMETRIC (
			);

		//! Copy constructor
		GEOMETRIC (
			const RVC::GEOMETRIC& rhs
			);

		//! Destructor
		virtual ~GEOMETRIC (
			) = 0;

		//! Assignment
		RVC::GEOMETRIC& operator= (
			const RVC::GEOMETRIC& rhs
			);

		//! Add an element to the search tree
		//! Need to call OpenSearchTree() before using this method
		ERRVALUE AddSearchTreeElement (
			INT32 elemnum,
			ELEMTYPE ElemType,
			const DRECT2D& Extents
			);

		//! Add an element to the search tree
		//! Need to call OpenSearchTree() before using this method
		ERRVALUE AddSearchTreeElement (
			INT32 elemnum,
			ELEMTYPE ElemType,
			const DPOINT2D& Point
			);

		//! Populate a search tree given existing elements using the extents and number of elements.
		//! currently in the object.
		//! This will replace any existing search tree for this type with a new search tree.
		ERRVALUE BuildSearchTree (
			ELEMTYPE ElemType
			) const;

		//! Populate a search tree given existing elements using the extents and number of elements
		//! given by the QSEARCHTREE::MAKEPARMS.
		//! This will replace any existing search tree for this type with a new search tree.
		ERRVALUE BuildSearchTree (
			const QSEARCHTREE::MAKEPARMS& MakeParms
			) const;

		//! Clear a search tree using the extents and number of initial elements given by the QSEARCHTREE::MAKEPARMS.
		//! Does not add known elements to the search tree.
		ERRVALUE ClearSearchTree (
			const QSEARCHTREE::MAKEPARMS& MakeParms
			) const;

		//! Combine styles from source object 'this' to the destination object 'DestObj'.
		//! this method deals with style links from source tables
		ERRVALUE CombineStylesTo (
			RVC::GEOMETRIC& DestObj
			) const;

		//! Combine styles from source object 'this' to the destination object 'DestObj'.
		//! this method deals with style links from source tables
		//! Used for objects with direct style attachments (a.k.a CAD)
		ERRVALUE CombineStylesTo (
			RVC::GEOMETRIC& DestObj,
			SIMPLE_ARRAY<INT32>& FontTrans,
			SIMPLE_ARRAY<INT32> PattTrans[PATTERNTYPE_NumTypes]
			) const;

		//! Copy an element from one part of the search tree to another (atomic method)
		//! Need to call OpenSearchTree() before using this method
		ERRVALUE CopySearchTreeElement (
			INT32 SourceElement,
			INT32 DestElement,
			ELEMTYPE ElemType,
			const DRECT2D& Extents
			);

		//! Copy an element from one part of the search tree to another (atomic method)
		//! Need to call OpenSearchTree() before using this method
		ERRVALUE CopySearchTreeElement (
			INT32 SourceElement,
			INT32 DestElement,
			ELEMTYPE ElemType,
			const DPOINT2D& Point
			);

		//! Create an SML context for the given derived class instance
		ERRVALUE CreateSMLContext (
			SML::CONTEXT **pcontext					//!< SML context allocated and RETURNED
			) const;

		//! Delete the search tree
		ERRVALUE DeleteSearchTree (
			ELEMTYPE ElemType
			);

		//! Delete an element from the search tree
		//! Need to call OpenSearchTree() before using this method
		ERRVALUE DeleteSearchTreeElement (
			INT32 elemnum,
			ELEMTYPE ElemType,
			const DRECT2D& Extents
			);

		//! Delete an element from the search tree
		//! Need to call OpenSearchTree() before using this method
		ERRVALUE DeleteSearchTreeElement (
			INT32 elemnum,
			ELEMTYPE ElemType,
			const DPOINT2D& Point
			);

		//! Extract elements from source 'this' object to existing destination object
		//! Deals with database attachments and element transfers
		//! Not implemented TIN yet
		ERRVALUE ExtractTo (
			RVC::GEOMETRIC& DestObj,
			RVC::GEOMETRIC::ELEMTRANSLIST& TransList,
			const TRANS2D_MAPGEN& tmg,
			RVC::GEOMETRIC::EXTRACTFLAGS ExtractFlags = EXTRACTFLAG_None
			) const { return (v_ExtractTo(DestObj, TransList, tmg, ExtractFlags)); }

		//! Determine closest element based on point and search distance
		//! @return Element found, -1 if no elements are close enough, < -1 if error
		INT32 FindClosestElement (
			ELEMTYPE ElemType,						//!< Element type to search for
			const DPOINT2D& SearchPt,				//!< Search point in object coordinates
			double SearchDist = 0,					//!< Search distance in object coordinates
			double *pDistFromElem = 0				//!< Returned distance to element found
			) const { return (v_FindClosestElementLow(ElemType, SearchPt, 0, SearchDist, pDistFromElem)); }

		//! Determine closest element based on point and search distance with filter
		//! @return Element found, -1 if no elements are close enough, < -1 if error
		INT32 FindClosestElement (
			ELEMTYPE ElemType,						//!< Element type to search for
			const DPOINT2D& SearchPt,				//!< Search point in object coordinates
			SEARCHFILTER& SearchFilter,			//!< Search filter
			double SearchDist = 0,					//!< Search distance in object coordinates
			double *pDistFromElem = 0				//!< Returned distance to element found
			) const { return (v_FindClosestElementLow(ElemType, SearchPt, &SearchFilter, SearchDist, pDistFromElem)); }

		//! Determine set of elements that are within the 'SearchDist' of the 'SearchPt'
		ERRVALUE FindClosestElements (
			const DPOINT2D& SearchPt,
			double SearchDist,
			SIMPLE_ARRAY<RVC::ELEMENT>& ElementList,
			COMPONENT Component = COMPONENT_All
			) const { return (v_FindClosestElementsLow(SearchPt, SearchDist, 0, ElementList, Component)); }

		//! Determine set of elements that are within the 'SearchDist' of the 'SearchPt' with filter
		ERRVALUE FindClosestElements (
			const DPOINT2D& SearchPt,
			double SearchDist,
			SEARCHFILTER& SearchFilter,
			SIMPLE_ARRAY<RVC::ELEMENT>& ElementList,
			COMPONENT Component = COMPONENT_All
			) const { return (v_FindClosestElementsLow(SearchPt, SearchDist, &SearchFilter, ElementList, Component)); }

		//! Obtain set of elements that fall within the rectangle, return list in 'SearchResult'
		//! This method will NOT build search trees to determine the elements that fall within the rectangle.
		//! If a search or spatial tree is not available to limit the search, all elements will be returned
		ERRVALUE FindElementDrawSet (
			ELEMTYPE ElemType,
			const DRECT2D& SearchExtents,
			SEARCHRESULT& SearchResult
			) const;

		//! Obtain set of elements that fall within the rectangle, return list in 'SearchResult'
		//! This method will build search trees, if needed, to determine the elements that fall within the rectangle.
		ERRVALUE FindElementSearchSet (
			ELEMTYPE ElemType,
			const DRECT2D& SearchExtents,
			SEARCHRESULT& SearchResult
			) const;

		//! Obtain set of elements that fall within specified extent, optionally building index/quadtree if needed.
		ERRVALUE FindElementSet (
			ELEMTYPE ElemType,						//!< Element type
			const DRECT2D& Extents,					//!< Extents
			UNSIGNEDSET& ElemSet,					//!< Element set returned
			bool BuildIndex = false					//!< Build search index (quadtree) if does not exist or is outdated
			) const;

		//! Obtain the DBASE_ELEMENT Attachement index from the element specified
		//! @return '-1' if element does not have a database attachment, >= 0 if it does.
		INT32 GetAttachmentIndex (
			const RVC::ELEMENT& Element
			) const { return (v_GetAttachmentIndex(Element)); }

		//! Obtain DIMENSION type
		//! @return DIMENSION type
		DIMENSION GetDimension (
			) const { return ((GetPointType() == POINTTYPE_3DXYZ) ? DIMENSION_3D : DIMENSION_2D); }

		//! Obtain element extents using element type and element number
		ERRVALUE GetElementExtents (
			ELEMTYPE ElemType,
			INT32 ElemNum,
			DRECT3D& Extents					//!< Element extents RETURNED, set to invalid if error or element is deleted
			) const { return (v_GetElementExtents(ElemType, ElemNum, Extents)); }

		//! Get geometric object referred to by 'object'
		//! @return Allocated instance of RVC::GEOMETRIC object (must be 'delete'd) or 0 if error
		static RVC::GEOMETRIC* GetGeometric (
			const RVC::OBJECT& object
			);

		//! Get geometric object referred to by 'ObjItem'
		//! @return Allocated instance of RVC::GEOMETRIC object (must be 'delete'd) or 0 if error
		static RVC::GEOMETRIC* GetGeometric (
			const RVC::OBJITEM& ObjItem
			);

		//! Obtain next element ID depending on element type
		//! @return Element ID to be used for the next element
		INT32 GetNextElementID (
			ELEMTYPE ElemType
			) const { return (v_GetNextElementID(ElemType)); }

		//! Obtain number of elements based on element type
		//! Specifying ELEMTYPE_All will return the total combination of elements in the object
		//! @return Number of elements if valid ELEMTYPE for object, -1 if not
		INT32 GetNumElements (
			ELEMTYPE ElemType
			) const { return (v_GetNumElements(ElemType)); }

		//! Get correct OBJTYPESET for all geometric OBJTYPE's
		//! @return OBJTYPESET for all geometric OBJTYPE's
		static RVC::OBJTYPESET GetObjTypeSet (
			);

		//! Obtain number of point coordinate dimensions
		//! @return Number of dimensions
		INT32 GetPointDimension (
			) const { return (GetPointSize() / sizeof(double)); }

		//! Obtain point coordinate type
		//! @return POINTTYPE coordinate type
		POINTTYPE GetPointType (
			) const { return (v_GetPointType()); }

		//! Obtain size of point coordinate
		//! @return Size of point in bytes
		INT32 GetPointSize (
			) const { return (v_GetPointSize()); }

		//! Obtain the total line length of all of the lines in the geometric object combined
		//! @return Total length of all of the lines combined or zero if not computed
		double GetTotalLineLength (
			) const { return (v_GetTotalLineLength()); }

		//! Determine if the element in this object is deleted.
		//! @return 'true' if the element is deleted, 'false' if not.
		bool IsElementDeleted (
			const RVC::ELEMENT& Element
			) const { return (v_IsElementDeleted(Element)); }

		//! Determine if the object has any elements.
		//! @return 'true' if the object is empty, 'false' if not.
		bool IsEmpty (
			) const { return (v_IsEmpty()); }

		//! Determine if the object type is a geometric object
		static bool IsGeometric (
			RVC::OBJTYPE ObjType
			);

		//! Obtain a start iterator for geometric labels
		//! @return a LABELITERATOR instance that contains the first label or LabelEnd()			
		LABELITERATOR LabelBegin (
			DIMENSION dim = DIMENSION_2D,
			STYLE* StyleObj = 0
			) { return (LABELITERATOR(this, dim, StyleObj)); }

		//! Obtain the end iterator for geometric labels
		LABELITERATOR LabelEnd (
			) { return (LABELITERATOR(this)); }

		//! Obtain a start iterator for geometric lines
		//! @return a LINEITERATOR instance that contains the first line or LineEnd()			
		LINEITERATOR LineBegin (
			DIMENSION dim = DIMENSION_2D
			) { return (LINEITERATOR(this, dim)); }

		//! Obtain the end iterator for geometric lines
		LINEITERATOR LineEnd (
			) { return (LINEITERATOR(this)); }

		//! Obtain a start iterator for geometric nodes
		//! @return a NODEITERATOR instance that contains the first node or NodeEnd()			
		NODEITERATOR NodeBegin (
			DIMENSION dim = DIMENSION_3D
			) { return (NODEITERATOR(this, dim)); }

		//! Obtain the end iterator for geometric nodes
		NODEITERATOR NodeEnd (
			) { return (NODEITERATOR(this)); }

		//! Obtain a start iterator for geometric points
		//! @return a POINTITERATOR instance that contains the first point set or PointEnd()
		POINTITERATOR PointBegin (
			DIMENSION dim = DIMENSION_2D
			) { return (POINTITERATOR(this, dim)); }

		//! Obtain the end iterator for geometric points
		POINTITERATOR PointEnd (
			) { return (POINTITERATOR(this)); }

		//! Obtain a start iterator for geometric polygons
		//! @return a POLYGONITERATOR instance that contains the first polygon set or PolygonEnd()
		POLYGONITERATOR PolygonBegin (
			DIMENSION dim = DIMENSION_2D
			) { return (POLYGONITERATOR(this, dim)); }

		//! Obtain the end iterator for geometric polygons
		POLYGONITERATOR PolygonEnd (
			) { return (POLYGONITERATOR(this)); }

		//! Read an element into 'LineList'.  LineList will be empty if the element is not a 'line' type.
		ERRVALUE ReadElement (
			INT32 ElemNum, 
			POLYLINELIST& LineList, 
			DIMENSION dim
			) const { return (v_ReadElement(ElemNum, LineList, dim)); }
			
		//! Read an element into 'PointList'.  PointList will be empty if the element is not a 'point' type.
		ERRVALUE ReadElement (
			INT32 ElemNum, 
			POLYLINE& PointList, 
			DIMENSION dim
			) const { return (v_ReadElement(ElemNum, PointList, dim)); }
			
		//! Read an element into 'Region'.  Region will be empty if the element is not a 'polygon' type.
		ERRVALUE ReadElement (
			INT32 ElemNum, 
			REGION2D& Region, 
			DIMENSION dim
			) const { return (v_ReadElement(ElemNum, Region, dim)); }
		
		//! Read the next label in order based on the previous label given
		//! The 'Element' field determines which element was previously read and will be set to the next element
		//! that was read on this method's return.  If the ELEMENT::GetNumber() returns -1, the end of the 
		//! element list has been reached.
		ERRVALUE ReadNextLabel (
			ELEMENT& Element,
			MILIST<LABEL>& Label,
			DIMENSION dim = DIMENSION_2D,
			STYLE* StyleObj = 0
			) const { return (v_ReadNextLabel(Element, Label, dim, StyleObj)); }

		//! Read the next point in order based on the previous point given
		//! The 'Element' field determines which element was previously read and will be set to the next element
		//! that was read on this method's return.  If the ELEMENT::GetNumber() returns -1, the end of the 
		//! element list has been reached.
		ERRVALUE ReadNextLine (
			ELEMENT& Element,
			POLYLINELIST& LineList,
			DIMENSION dim = DIMENSION_2D
			) const { return (v_ReadNextLine(Element, LineList, dim)); }

		//! Read the next point in order based on the previous point given
		//! The 'Element' field determines which element was previously read and will be set to the next element
		//! that was read on this method's return.  If the ELEMENT::GetNumber() returns -1, the end of the 
		//! element list has been reached.
		ERRVALUE ReadNextNode (
			ELEMENT& Element,
			DPOINT3D& Node
			) const { return (v_ReadNextNode(Element, Node)); }

		//! Read the next point in order based on the previous point given
		//! The 'Element' field determines which element was previously read and will be set to the next element
		//! that was read on this method's return.  If the ELEMENT::GetNumber() returns -1, the end of the 
		//! element list has been reached.
		ERRVALUE ReadNextPoint (
			ELEMENT& Element,
			POLYLINE& PointList,
			DIMENSION dim = DIMENSION_2D
			) const { return (v_ReadNextPoint(Element, PointList, dim)); }

		//! Read the next point in order based on the previous point given
		//! The 'Element' field determines which element was previously read and will be set to the next element
		//! that was read on this method's return.  If the ELEMENT::GetNumber() returns -1, the end of the 
		//! element list has been reached.
		ERRVALUE ReadNextRegion (
			ELEMENT& Element,
			REGION2D& Region,
			DIMENSION dim = DIMENSION_2D
			) const { return (v_ReadNextRegion(Element, Region, dim)); }

		//! Generate a region given a bitset of polygonal elements to use.
		//! The 'Region' will be empty if the virtual method is not supported.
		ERRVALUE ReadRegion (
			const BITSET_UNOWNED& ElemSet,	//!< Set of polygonal elements to generate region from
			REGION2D& Region,						//!< Region to set
			DIMENSION dim,							//!< Set to dimension needed
			bool NoIslands = false				//!< 'True' of no islands wanted
			) const { return (v_ReadRegion(ElemSet, Region, dim, NoIslands)); }
			
		//! Set extents of an object
		//! Only use if you have a write lock on the entire object
		ERRVALUE SetExtents (
			const DRECT3D &extents
			) { return (v_SetExtents(extents)); }

		//! Set next element ID depending on element type
		ERRVALUE SetNextElementID (
			ELEMTYPE ElemType,
			INT32 NextElementID
			) { return (v_SetNextElementID(ElemType, NextElementID)); }

		//! Set total line length of all of the lines in the geometric object combined
		ERRVALUE SetTotalLineLength (
			double length
			) { return (v_SetTotalLineLength(length)); }

		//! Generate element translation lists dependent on the selection parameters.
		//! @return > 0 if elements are found to meet the selection parameters, '0' if no elements found, < 0 error
		int SetupElemTransList (
			const RVC::GEOMETRIC::ELEMSELECTPARMS& SelectParms,
			RVC::GEOMETRIC::ELEMTRANSLIST& TransList,
			bool separate = false								//!< Separate components if used in more than one element
			) const { return (SetupElemTransList(SelectParms, TransList, 0, separate)); }

		//! Generate element translation lists dependent on the selection parameters.
		//! @return > 0 if elements are found to meet the selection parameters, '0' if no elements found, < 0 error
		int SetupElemTransList (
			const RVC::GEOMETRIC::ELEMSELECTPARMS& SelectParms,
			RVC::GEOMETRIC::ELEMTRANSLIST& TransList,
			RVC::GEOMETRIC::ELEMSELECTFILTER& SelectFilter,
			bool separate = false								//!< Separate components if used in more than one element
			) const { return (SetupElemTransList(SelectParms, TransList, &SelectFilter, separate)); }

		//! Test element against region
		//! The REGION2D needs to be in 'object' coordinates, no conversion is done internally
		//! @return True if test against region was sucessful
		bool TestElementAgainstRegion (
			const RVC::ELEMENT& Element,
			const REGION2D& region,
			TESTCRITERIA TestMode
			) const { return (v_TestElementAgainstRegion(Element, region, TestMode)); }

		//! Update the extents of an object, this is a thread safe method
		ERRVALUE UpdateExtents (
			const DRECT3D &extents
			) { return (v_UpdateExtents(extents)); }

	protected:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		static void GeometricCallbackFunc (ANYCALLBACKSTRUCT* cb, void *data);
		#endif // GENERATING_DOXYGEN_OUTPUT

	private:
		// Pure virtual methods
		virtual ERRVALUE v_ExtractTo (RVC::GEOMETRIC& DestObj, RVC::GEOMETRIC::ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg, RVC::GEOMETRIC::EXTRACTFLAGS ExtractFlags) const = 0;
		virtual INT32 v_FindClosestElementLow (ELEMTYPE ElemType, const DPOINT2D& SearchPt, SEARCHFILTER* SearchFilter, double SearchDist, double* DistFromElem) const = 0;
		virtual ERRVALUE v_GetElementExtents (ELEMTYPE ElemType, INT32 ElemNum, DRECT3D& Extents) const = 0;
		virtual INT32 v_GetNextElementID (ELEMTYPE ElemType) const = 0;
		virtual INT32 v_GetNumElements (ELEMTYPE ElemType) const = 0;
		virtual POINTTYPE v_GetPointType () const = 0;
		virtual INT32 v_GetPointSize () const = 0;
		virtual QSEARCHTREE* v_GetSearchTree (ELEMTYPE ElemType) const = 0;
		virtual double v_GetTotalLineLength () const = 0;
		virtual bool v_IsElementDeleted (const RVC::ELEMENT& Element) const = 0;
		virtual bool v_IsEmpty () const = 0;
		virtual ERRVALUE v_SetExtents (const DRECT3D &extents) = 0;
		virtual ERRVALUE v_SetNextElementID (ELEMTYPE ElemType, INT32 NextElementID) = 0;
		virtual void v_SetSearchTree (ELEMTYPE ElemType, QSEARCHTREE* SearchTree) const = 0;
		virtual ERRVALUE v_SetTotalLineLength (double length) = 0;
		virtual int v_SetupElemTransList (const RVC::GEOMETRIC::ELEMSELECTPARMS& SelectParms, RVC::GEOMETRIC::ELEMTRANSLIST& TransList, RVC::GEOMETRIC::ELEMSELECTFILTER *SelectFilter, bool separate) const = 0;
		virtual bool v_TestElementAgainstRegion (const RVC::ELEMENT& Element, const REGION2D& region, TESTCRITERIA TestMode) const = 0;
		virtual ERRVALUE v_UpdateExtents (const DRECT3D &extents) = 0;

		// Virtual methods
		virtual ERRVALUE v_FindClosestElementsLow (const DPOINT2D& SearchPt, double SearchDist, SEARCHFILTER* SearchFilter, SIMPLE_ARRAY<RVC::ELEMENT>& ElementList, COMPONENT Component) const;
		virtual ERRVALUE v_FindElementDrawSet (ELEMTYPE ElemType, const DRECT2D& SearchExtents, SEARCHRESULT& SearchResult) const;
		virtual ERRVALUE v_FindElementSearchSet (ELEMTYPE ElemType, const DRECT2D& SearchExtents, SEARCHRESULT& SearchResult) const;
		virtual ERRVALUE v_FindElementSet (ELEMTYPE ElemType, const DRECT2D& SearchExtents, UNSIGNEDSET& ElemSet, bool BuildIndex) const;
		virtual INT32 v_GetAttachmentIndex (const RVC::ELEMENT& Element) const;
		virtual SPATIALTREE* v_GetSpatialTree () const;
		virtual ERRVALUE v_ReadElement (INT32 ElemNum, POLYLINELIST& LineList, DIMENSION dim) const;
		virtual ERRVALUE v_ReadElement (INT32 ElemNum, POLYLINE& PointList, DIMENSION dim) const;
		virtual ERRVALUE v_ReadElement (INT32 ElemNum, REGION2D& Region, DIMENSION dim) const;
		virtual ERRVALUE v_ReadNextLabel (ELEMENT& Element, MILIST<LABEL>& Label, DIMENSION dim, STYLE* StyleObj) const;
		virtual ERRVALUE v_ReadNextLine (ELEMENT& Element, POLYLINELIST& LineList, DIMENSION dim) const;
		virtual ERRVALUE v_ReadNextNode (ELEMENT& Element, DPOINT3D& Node) const;
		virtual ERRVALUE v_ReadNextPoint (ELEMENT& Element, POLYLINE& PointList, DIMENSION dim) const;
		virtual ERRVALUE v_ReadNextRegion (ELEMENT& Element, REGION2D& Region, DIMENSION dim) const;
		virtual ERRVALUE v_ReadRegion (const BITSET_UNOWNED& ElemSet, REGION2D& Region, DIMENSION dim, bool NoIslands) const;
		virtual void v_SetSpatialTree (SPATIALTREE*) const;

		#ifndef GENERATING_DOXYGEN_OUTPUT
		ERRVALUE DeleteSearchTrees (OBJTYPE ObjTypeSearchTree, ELEMTYPE ElemType, bool ErrorIfCantDelete = false) const;
		ERRVALUE FillInSearchTree (ELEMTYPE ElemType) const;
		ERRVALUE OpenSearchTree (ELEMTYPE ElemType, OPENMODE mode, bool DoNotCreate = false) const;
		ERRVALUE OpenFoundSearchTree (ELEMTYPE ElemType, OPENMODE mode, bool& NeedTempTree) const;
		int SetupElemTransList (const RVC::GEOMETRIC::ELEMSELECTPARMS& SelectParms, RVC::GEOMETRIC::ELEMTRANSLIST& TransList, RVC::GEOMETRIC::ELEMSELECTFILTER* SelectFilter, bool separate) const;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(GEOMETRIC::EXTRACTFLAGS)
DEFINE_ENUM_OP_BITWISE(GEOMETRIC::COMPONENT)
#endif

//---------------------------------------------------------------------------------------------------------------------

class GEOMETRIC::ELEMSELECTFILTER {
	public:
		ELEMSELECTFILTER () {}

		//! Filter elements based on some external criteria
		//! @return '1' for element acceptance, '0' to reject element, < 0 for an error
		int FilterElement (
			RVC::ELEMENT& Element
			) { return (v_FilterElement(Element)); }

	private:
		virtual int v_FilterElement (RVC::ELEMENT& element) = 0;
	};

//---------------------------------------------------------------------------------------------------------------------

//! Parameters for element selection/extraction.
class GEOMETRIC::ELEMSELECTPARMS {
	public:

		//! Extraction operation by extents.
		enum EXTRACT {
			EXTRACT_All,
			EXTRACT_PartInside,
			EXTRACT_FullInside,
			EXTRACT_ClipInside,
			EXTRACT_PartOutside,
			EXTRACT_FullOutside,
			EXTRACT_ClipOutside,
			EXTRACT_ClipByPolygons
			};

		//! Item defining how component elements are selected.
		struct ITEM {

			enum SELECT {
				SELECT_None = 0,
				SELECT_All,
				SELECT_ByRecord,
				SELECT_ByScript,
				SELECT_ByElement,
				SELECT_Last
				};

			//! Default constructor.
			ITEM ();

			//! Get set of selected elements, indexed by element number.
			const BITSET& GetElementSet () const
				{ return (m_ElemList); }

			//! Get query string.
			const MISTRING& GetQueryStr () const
				{ return (m_QueryStr); }

			//! Get set of selected records, indexed by record number.
			const BITSET& GetRecordSet () const
				{ return (m_RecordList); }

			SELECT GetSelection () const
				{ return (m_SelectType); }

			//! Get table name corresponding to record set.
			const RVC::OBJECTNAME& GetTableName () const
				{ return (m_TableName); }

			SELECT m_SelectType;
			MISTRING m_QueryStr;
			RVC::OBJECTNAME m_TableName;		//!< Table that 'RecordList' is selected from
			BITSET m_RecordList;					//!< List of records in 'TableNum' that the user has selected (bit array)
			BITSET m_ElemList;					//!< List of elements (bit array)
			};

		//! Parameters for output to separate object/files by input.
		class SEPARATION {
			public:

				//! Default constructor.
				SEPARATION ();

				//! Destructor.
				~SEPARATION ();

				//! Clear separation.
				void Clear ();

				//! Get element type to base separation on, ELEMTYPE_None if not separating.
				ELEMTYPE GetElemType () const
					{ return (m_ElemType); }

				//! Get field name.
				const MISTRING& GetFieldName () const
					{ return (m_FieldName); }

				//! Get file/object name prefix.
				const MISTRING& GetPrefix () const
					{ return (m_Prefix); }

				//! Get file/object name suffix.
				const MISTRING& GetSuffix () const
					{ return (m_Suffix); }

				//! Get table name.
				const MISTRING& GetTableName () const
					{ return (m_TableName); }

				//! Determine whether each output is to own file or all in same file.
				bool IsEachToOwnFile () const
					{ return (m_IsEachToOwnFile); }

				//! Set element type and table/field.
				void Set (
					ELEMTYPE ElemType,					//!< Element type to base separation on
					const MISTRING& TableName,			//!< Table name to separate by
					const MISTRING& FieldName			//!< Field name to separate by
					) { m_ElemType = ElemType; m_TableName = TableName; m_FieldName = FieldName; }

				//! Set whether each output is to own file or all in same file.
				void SetEachToOwnFile (
					bool IsEachToOwnFile
					) { m_IsEachToOwnFile = IsEachToOwnFile; }

				//! Set file/object name prefix.
				void SetPrefix (
					const MISTRING& Prefix
					) { m_Prefix = Prefix; }

				//! Set file/object name suffix.
				void SetSuffix (
					const MISTRING& Suffix
					) { m_Suffix = Suffix; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				ELEMTYPE m_ElemType;
				MISTRING m_TableName;
				MISTRING m_FieldName;
				MISTRING m_Prefix;
				MISTRING m_Suffix;
				bool m_IsEachToOwnFile;
				#endif
			};

		//! Default constructor.
		ELEMSELECTPARMS ();

		//! Access (non-const) selection parameter item by element type.
		ITEM& AccessSelectParmItem (
			ELEMTYPE ElemType
			);

		//! Access (non-const) separation parameters.
		SEPARATION& AccessSeparation ()
			{ return (m_Separation); }

		//! Clear all selection values.
		void Clear ();

		//! Get whether to add border when extracting by region.
		bool GetAddBorder () const
			{ return (m_bAddBorder); }

		//! Get OBJITEM for extents.
		const RVC::OBJITEM& GetExtentsObjItem () const
			{ return (m_ExtentsObjItem); }

		//! Get extraction region.
		const REGION2D& GetRegion2D () const
			{ return (m_Region); }

		//! Get extraction mode for use with region.
		EXTRACT GetRegionExtractMode () const
			{ return (m_ExtractMode); }

		//! Get region test criteria.
		TESTCRITERIA GetRegionTestCriteria () const
			{
			TESTCRITERIA TestCriteria = TESTCRITERIA_PartInside;
			switch (m_ExtractMode) {
				case EXTRACT_FullOutside:	TestCriteria = TESTCRITERIA_FullOutside; break;
				case EXTRACT_PartOutside:	TestCriteria = TESTCRITERIA_PartOutside; break;
				case EXTRACT_ClipOutside:	TestCriteria = TESTCRITERIA_PartOutside; break;
				case EXTRACT_FullInside:	TestCriteria = TESTCRITERIA_FullInside; break;
				case EXTRACT_PartInside:	TestCriteria = TESTCRITERIA_PartInside; break;
				case EXTRACT_ClipInside:	TestCriteria = TESTCRITERIA_PartInside; break;
				}
			return (TestCriteria);
			}

		//! Get selection parameter item by element type.
		const ITEM& GetSelectParmItem (
			ELEMTYPE ElemType
			) const;

		//! Access (non-const) selection parameter item by element type.
		ITEM& GetSelectParmItem (
			ELEMTYPE ElemType
			);

		//! Get separation parameters.
		const SEPARATION& GetSeparation () const
			{ return (m_Separation); }

		//! Determine if has non-empty region for possible extraction by extent.
		bool HasRegion () const
			{ return (!m_Region.IsEmpty()); }

		//! Set whether to add border when extracting by region.
		void SetAddBorder (
			bool bAddBorder
			) { m_bAddBorder = bAddBorder; }

		//! Set OBJITEM for extents.
		void SetExtentsObjItem (
			const RVC::OBJITEM& objitem
			) { m_ExtentsObjItem = objitem; }

		//! Set extent region.
		void SetRegion (
			const REGION2D& Region
			) { m_Region = Region; }

		//! Set region extraction mode.
		void SetRegionExtractMode (
			EXTRACT ExtractMode
			) { m_ExtractMode = ExtractMode; }

		//! Set to select all elements.
		void SetSelectAll ();

		//! Set to select no elements.
		void SetSelectNone ();

		//! Set selection parameter item by element type.
		void SetSelectParmItem (
			ELEMTYPE ElemType,
			const ITEM& item
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		ITEM m_SelectParm[6];
		SEPARATION m_Separation;
		REGION2D m_Region;
		EXTRACT m_ExtractMode;
		OBJITEM m_ExtentsObjItem;
		bool m_bAddBorder;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//---------------------------------------------------------------------------------------------------------------------

//! Class to manage element translation lists from source to destination elements
//! Eg: 'DestElemNum = TransList[SourceElemNum]' where 'TransList' is an array of INT32's
//! The value '-1' states that the source element does not translate to a destination number.
class GEOMETRIC::ELEMTRANSLIST {
	public:

		//! Constructor
		ELEMTRANSLIST (
			) {}

		//! Copy Constructor
		ELEMTRANSLIST (
			const ELEMTRANSLIST& rhs
			);

		//! Destructor
		~ELEMTRANSLIST (
			) {}

		//! Assignment Operator
		ELEMTRANSLIST& operator= (
			const ELEMTRANSLIST& rhs
			);

		//! Get base table name to use for extraction.  Currently used the extract to shape objects only.
		//! @return The name of the base table for extraction.
		const DESCRIPTOR& GetBaseTableName (
			) const { return (m_BaseTableName); }

		//! Obtain the element bitset for a particular element type (ELEMTYPE)
		ERRVALUE GetElementBitset (
			ELEMTYPE ElemType,
			BITSET& Bitset
			) const;

		//! Obtain the element bitset deque for a particular element type (ELEMTYPE)
		ERRVALUE GetElementBitsetDeque (
			ELEMTYPE ElemType,
			BITSET_DEQUE& BitsetDeque
			) const;

		//! Obtain the element translation array for a particular element type (ELEMTYPE)
		//! @return Element Translation List
		const SIMPLE_ARRAY<INT32>& GetElementTransList (
			ELEMTYPE ElemType
			) const;

		//! Obtain the element translation array for a particular element type (ELEMTYPE)
		//! @return Element Translation List
		SIMPLE_ARRAY<INT32>& GetElementTransList (
			ELEMTYPE ElemType
			);

		ERRVALUE GetReverseTransList (
			ELEMTYPE ElemType,
			SIMPLE_ARRAY<INT32>& ReverseTrans,
			INT32 NumDestElems
			) const;
			
		//! Set base table name to use for extraction.  Currently used the extract to shape objects only.
		void SetBaseTableName (
			const DESCRIPTOR& BaseTableName
			) { m_BaseTableName = BaseTableName; }

		//! Set an element translation list entry to a pre-built list
		void SetElementTransList (
			const SIMPLE_ARRAY<INT32>& TransList,
			ELEMTYPE ElemType
			);

		//! Set an element translation list entry to a pre-built list
		ERRVALUE SetElementTransList (
			INT32* TransList,
			INT32 NumItems,
			ELEMTYPE ElemType
			);

		//! Set translation list to a one-to-one (Identity) mapping
		ERRVALUE SetIdentity (
			const RVC::GEOMETRIC& GeoObj			//!< Geometric object used to set identity arrays using the object type and number of elements per element type
			);

		//! Set an element translation list entry to a one-to-one (Identity) mapping
		ERRVALUE SetIdentity (
			const RVC::GEOMETRIC& GeoObj,			//!< Geometric object used to set identity arrays using the object type
			ELEMTYPE ElemType							//!< Element Type used to set the identity array entry using the number of elements for this type
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		SIMPLE_ARRAY<INT32> m_TransList[6];
		DESCRIPTOR m_BaseTableName;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//---------------------------------------------------------------------------------------------------------------------

//! GETOBJFILTER derived class to determine if the geometric object has the minimum number of elements of the specified types
class GEOMETRIC::GETOBJFILTER_HASMINIMUMELEMENTS : public GETOBJFILTER {
	public:
		GETOBJFILTER_HASMINIMUMELEMENTS (
			const ELEMTYPESET& ElemTypeSet,
			INT32 MinElements = 1
			) : m_ElemTypeSet(ElemTypeSet), m_MinElements(MinElements) {}

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		ELEMTYPESET m_ElemTypeSet;
		INT32 m_MinElements;

		virtual bool v_IsSelectable (const RVC::OBJITEM& ObjItem, int position);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//---------------------------------------------------------------------------------------------------------------------

//! Geometric object selection filter to disallow selection of empty and invalid extent geometric objects
class GEOMETRIC::GETOBJFILTER_ISDISPLAYABLE : public GETOBJFILTER {
	public:
		GETOBJFILTER_ISDISPLAYABLE () {}
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool v_IsSelectable (const RVC::OBJITEM& ObjItem, int position);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//---------------------------------------------------------------------------------------------------------------------

//! Geometric object selection filter to disallow selection of empty geometric objects
class GEOMETRIC::GETOBJFILTER_ISNOTEMPTY : public GETOBJFILTER {
	public:
		GETOBJFILTER_ISNOTEMPTY () {}
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool v_IsSelectable (const RVC::OBJITEM& ObjItem, int position);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//---------------------------------------------------------------------------------------------------------------------

//! Geometric object selection filter to disallow selection of non geometric objects
class GEOMETRIC::GETOBJFILTER_ISSELECTABLE : public GETOBJFILTER {
	public:
		GETOBJFILTER_ISSELECTABLE () {}
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool v_IsSelectable (const RVC::OBJITEM& ObjItem, int position);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//---------------------------------------------------------------------------------------------------------------------

//! Make parameter base class for RVC::GEOMETRIC objects.
class GEOMETRIC::MAKEPARMSBASE {
	public:

		//! Default constructor
		MAKEPARMSBASE (
			POINTTYPE PointType
			) :
			m_PointType(PointType),
			m_XScale(1.0),
			m_YScale(1.0),
			m_ZScale(1.0)
			{}

		//! Get vector object extents
		//! @return Extents of the vector object
		const DRECT3D& GetExtents (
			) const {return (m_Extents);}

		//! Obtain point coordinate type for vector creation
		//! @return RVCGEOMETRIC::POINTTYPE coordinate type
		POINTTYPE GetPointType (
			) const {return (m_PointType);}

		//! Get vector x scale value
		//! @return Vector X scale value
		double GetXScale (
			) const {return (m_XScale);}

		//! Get vector y scale value
		//! @return Vector Y scale value
		double GetYScale (
			) const {return (m_YScale);}

		//! Get vector z scale value
		//! @return Vector Z scale value
		double GetZScale (
			) const {return (m_ZScale);}

		//! Set vector object extents
		void SetExtents (
			const DRECT3D& rect
			) {m_Extents = rect;}

		//! Set coordinate point type for vector creation
		void SetPointType (
			POINTTYPE PointType
			) {m_PointType = PointType;}

		//! Set vector x scale value
		void SetXScale (
			double xscale
			) {m_XScale = xscale;}

		//! Set vector y scale value
		void SetYScale (
			double yscale
			) {m_YScale = yscale;}

		//! Set vector x scale value
		void SetZScale (
			double zscale
			) {m_ZScale = zscale;}

	protected:

		DRECT3D m_Extents;
		POINTTYPE m_PointType;
		double m_XScale;
		double m_YScale;
		double m_ZScale;
	};

//---------------------------------------------------------------------------------------------------------------------

//! OBSERVER class for RVC::GEOMETRIC objects.
class GEOMETRIC::OBSERVER : public RVC::OBJECTOBSERVER {
	public:
		enum TYPE {
			TYPE_ExtentsChange = 	0x00000008,
			TYPE_PointTypeChange = 	0x00000010
//!			TYPE_AddElement = 0x00000201
			};

		explicit OBSERVER (
			RVC::GEOMETRIC& object
			) : OBJECTOBSERVER(object) {}

		virtual ~OBSERVER (
			) {}

//!		virtual void OnAddElement (
//!			ELEMTYPE ElemType,
//!			INT32 ElemNum
//!			) { return; }

		//! Called when the geoemtric object extents has changed
		virtual void OnExtentsChanged (
			const DRECT3D&								//!< NewExtents
			) { return; }

		//! Called when the geoemtric object extents has changed
		virtual void OnPointTypeChanged (
			GEOMETRIC::POINTTYPE						//!< NewPointType
			) { return; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual void OnNotifyRaw (UINT32 reason, void *data);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//---------------------------------------------------------------------------------------------------------------------

//! Filter search class for FindClosestElement() method
class GEOMETRIC::SEARCHFILTER {
	public:
		//! Element filter for locating correct elements.
		//! @return True to use element in search, False to not use element in search
		virtual bool Filter (
			const RVC::GEOMETRIC& object,
			ELEMTYPE ElemType,
			INT32 ElemNum
			) = 0;

	protected:
		//! Destructor
		virtual ~SEARCHFILTER () {}
	};

//---------------------------------------------------------------------------------------------------------------------

//! Filter search class for FindClosestElement() method supporting bit-set.
class GEOMETRIC::SEARCHFILTER_BITSET : public GEOMETRIC::SEARCHFILTER {
	public:

		//! Construct from BITSET_UNOWNED.
		SEARCHFILTER_BITSET (
			const BITSET_UNOWNED& bitset
			) : m_bitset(bitset) { }

		//! Construct from bit array and count.
		SEARCHFILTER_BITSET (
			const UINT8 *set,
			UINT32 NumEntries
			) : m_bitset(const_cast<UINT8*>(set),NumEntries) { }

		//! Destructor
		virtual ~SEARCHFILTER_BITSET () {}

	protected:

		//! Element filter for locating correct elements.
		//! @return True if element in set, false if not.
		virtual bool Filter (
			const RVC::GEOMETRIC& object,
			ELEMTYPE ElemType,
			INT32 ElemNum
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		BITSET_UNOWNED m_bitset;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//---------------------------------------------------------------------------------------------------------------------

//! Filter search class for FindClosestElement() method supporting bit-set.
class GEOMETRIC::SEARCHFILTER_UNSIGNEDSET : public GEOMETRIC::SEARCHFILTER {
	public:

		//! Construct from UNSIGNEDSET_UNOWNED.
		SEARCHFILTER_UNSIGNEDSET (
			const UNSIGNEDSET& ElemSet			//!< Element set, must not be compressed
			) : m_ElemSet(ElemSet) { }

		//! Destructor
		virtual ~SEARCHFILTER_UNSIGNEDSET () {}

	protected:

		//! Element filter for locating correct elements.
		//! @return True if element in set, false if not.
		virtual bool Filter (
			const RVC::GEOMETRIC& object,
			ELEMTYPE ElemType,
			INT32 ElemNum
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		const UNSIGNEDSET& m_ElemSet;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//---------------------------------------------------------------------------------------------------------------------

//! Filter search class for FindClosestElement() method supporting bit-set.
class GEOMETRIC::SEARCHFILTER_ELEMTRANSLIST : public GEOMETRIC::SEARCHFILTER {
	public:

		//! Construct from ELEMTRANSLIST
		SEARCHFILTER_ELEMTRANSLIST (
			const GEOMETRIC::ELEMTRANSLIST& ElemTransList
			) : m_ElemTransList(ElemTransList) { }

		//! Destructor
		virtual ~SEARCHFILTER_ELEMTRANSLIST () {}

	protected:

		//! Element filter for locating correct elements.
		//! @return True if element in set, false if not.
		virtual bool Filter (
			const RVC::GEOMETRIC& object,
			ELEMTYPE ElemType,
			INT32 ElemNum
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		GEOMETRIC::ELEMTRANSLIST m_ElemTransList;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//---------------------------------------------------------------------------------------------------------------------

class GEOMETRIC::SEARCHRESULT {
	public:

		class CONST_ITERATOR;
		friend class CONST_ITERATOR;

		class CONST_ITERATOR {
			public:

				//! Default constructor
				CONST_ITERATOR (
					) :
					m_IsRange(false),
					m_CurElemNum(0),
					m_CurIndex(0),
					m_sr(0)
					{}

				//! Internal constructor, used by container.
				CONST_ITERATOR (
					const SEARCHRESULT* sr,
					bool CreateEnd = false
					);

				//! Copy constructor
				CONST_ITERATOR (
					const CONST_ITERATOR& rhs
					) :
					m_IsRange(rhs.m_IsRange),
					m_CurElemNum(rhs.m_CurElemNum),
					m_CurIndex(rhs.m_CurIndex),
					m_sr(rhs.m_sr)
					{}

				//! Dereference operator, returns current element number.
				const INT32& operator* (
					) const { return (m_CurElemNum); }

				//! Pre-increment operator.
				CONST_ITERATOR& operator++ (
					);

				//! Post-increment operator.
				CONST_ITERATOR operator++(int
					) {
					CONST_ITERATOR temp = *this;
					++*this;
					return (temp);
					}

				bool operator== (				//! Equality operator
					const CONST_ITERATOR& rhs
					) const { return (m_CurElemNum == rhs.m_CurElemNum); }

				bool operator!= (				//! Inequality operator
					const CONST_ITERATOR& rhs
					) const { return (!(*this == rhs)); }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				bool m_IsRange;
				INT32 m_CurElemNum;
				INT32 m_CurIndex;
				const SEARCHRESULT* m_sr;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		SEARCHRESULT (
			);

		SEARCHRESULT (
			const SEARCHRESULT& rhs
			);

		~SEARCHRESULT (
			);

		SEARCHRESULT& operator= (
			const SEARCHRESULT& rhs
			);

		CONST_ITERATOR Begin (
			) const {return (CONST_ITERATOR(this));}

		void Clear (
			);

		CONST_ITERATOR End (
			) const {return (CONST_ITERATOR(this, true));}

		ERRVALUE FindElements (
			const DRECT2D& SearchExtents,
			RVC::QSEARCHTREE& QTreeObj,
			INT32 MaxElemNum
			);

		ERRVALUE FindElements (
			ELEMTYPE ElemType,
			const DRECT2D& SearchExtents,
			RVC::SPATIALTREE& STreeObj
			);

		ERRVALUE GetElementSet (
			UNSIGNEDSET& ElemSet
			) const;

		ERRVALUE GetElementSet (
			BITSET& SelectSet
			) const;

		ERRVALUE GetElementSet (
			SIMPLE_ARRAY<INT32>& SelectList
			) const;

		INT32 GetNumItems () const;

		bool IsEmpty () const;

		//! Set search result for the all elements set
		void SetAll (
			INT32 NumElements
			);

		//! Set search result for array passed in
		void SetArray (
			const SIMPLE_ARRAY<INT32>& ElemSet
			);

		//! Sort the elements in the list using the elements extents from smallest to largest
		void SortItemsByExtentArea (
			const RVC::GEOMETRIC& GeoObj,
			ELEMTYPE ElemType
			);

		//! Sort the elements in the list using the elements extents from smallest to largest
		void SortItemsByExtentArea (
			const RVC::GEOMETRIC& GeoObj,
			ELEMTYPE ElemType,
			const DRECT2D& OverlapFilterRect		//!< Filter out those entries that do not overlap this rectangle
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		SIMPLE_ARRAY<INT32> m_list;
		SIMPLE_ARRAY<INT32_RANGE> m_range;
		INT32 m_MaxElemNum;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}		//! End of RVC namespace

#endif 	//!< INC_RVC_GEOMETRC_H
