/**
 * \file rvc/hyperidx.h 
 * \brief interface for the RVC::HYPERINDEX class.
 *
 * \if NODOC
 * $Id: hyperidx.h_v 1.26 2011/12/09 23:04:56 mju Exp $
 *
 * $Log: hyperidx.h_v $
 * Revision 1.26  2011/12/09 23:04:56  mju
 * Remove redundant hdr.
 *
 * Revision 1.25  2010/11/16 16:06:26  mju
 * Add shape::clear.
 *
 * Revision 1.24  2009/02/10 21:30:18  mju
 * Add link::type_smlscript.
 *
 * Revision 1.23  2008/03/11 22:15:17  scowan
 * Added getName method for links.
 *
 * Revision 1.22  2007/12/14 21:02:09  mju
 * Add link::canUseScalePos.
 *
 * Revision 1.21  2007/12/14 20:56:47  mju
 * Add link.setMapScale.
 *
 * Revision 1.20  2007/04/11 20:10:30  mju
 * Add getRefType and get/setRef... methods.
 * Remove get/setFilename/Name methods.
 *
 * Revision 1.17  2006/06/15 16:33:10  scowan
 * Changed from integer id to allocated class instance.
 *
 * Revision 1.16  2006/01/05 15:03:49  mju
 * Add link.getObjType.
 *
 * Revision 1.8  2001/01/03 17:49:53  mju
 * Add new LINKTO types.
 *
 * Revision 1.1  2000/09/29 16:40:26  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_HYPERIDX_H
#define INC_RVC_HYPERIDX_H

#ifndef INC_RVC_OBJECT_H
	#include <rvc/object.h>
#endif

#ifndef INC_MI32_ELEMTYPE_H
	#include <mi32/elemtype.h>
#endif

#ifndef INC_MI32_COLOR_H
	#include <mi32/color.h>
#endif

#ifndef INC_MI32_RECT_H
	#include <mi32/rect.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
struct HYPERLINK;
class POLYLINE;
namespace RVC {
	class DBASE_ELEMENT;
	class ELEMENT;
	class HYPERHANDLE;
	struct HYPERLINK;
	struct HYPERSHAPE;
	}
#endif


namespace RVC {

//! Manages RVC::OBJTYPE_HyperIndex RVC objects.  
//!	
//!	RVC Object Types that this class supports: RVC::OBJTYPE_HyperIndex
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_Vector
//!		RVC::OBJTYPE_TIN
//!		RVC::OBJTYPE_CAD
//!		RVC::OBJTYPE_Raster
//!		RVC::OBJTYPE_RasterSet
//!		RVC::OBJTYPE_Hyperspectral
class HYPERINDEX : public RVC::OBJECT {
	public:

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class LINK;
		class SHAPE;
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Default constructor
		HYPERINDEX (
			);

		//! Copy constructor
		HYPERINDEX (
			const RVC::HYPERINDEX& obj
			);

		//! Destructor
		virtual ~HYPERINDEX (
			);

		//! Assignment
		RVC::HYPERINDEX& operator= (
			const RVC::HYPERINDEX& rhs
			);

		//! Add a link to the rvc hyperindex object, do not use WriteLink() since multiple people 
		//! can be trying to add a link at the same time
		//! @return Index of new link object of error if < 0
		INT32 AddLink (
			const LINK& link								//!< Link to add
			);

		//! Add a link to a hyper shape
		ERRVALUE AddLinkToShape (
			INT32 ShapeNum,								//!< Shape number to add to
			INT32 LinkNum									//!< Stored link to add
			);

		//! Add a shape to the rvc hyperindex object, do not use WriteShape() since multiple people 
		//! can be trying to add a shape at the same time
		//! @return Index of new shape object of error if < 0
		INT32 AddShape (
			const SHAPE& shape								//!< Shape to add
			);

		//! Add a shape to the rvc hyperindex object, do not use WriteShape() since multiple people 
		//! can be trying to add a shape at the same time
		//! @return Index of new shape object of error if < 0
		INT32 AddShape (
			const SHAPE& shape,							//!< Shape parameters to write
			const POLYLINE& Points						//!< Shape boundary
			);

		//! Change the characteristics of a hypershape
		ERRVALUE ChangeShape (
			INT32 ShapeNum,								//!< Number of shape to change
			const SHAPE& shape,							//!< New description of shape
			const POLYLINE& Points						//!< Shape boundary points, if needed
			);
			
		//! Clear a link entry
		//! The entry still exists but is now blank
		ERRVALUE ClearLink (
			INT32 LinkNum									//!< Link index to erase
			);

		//! Clear shape points
		ERRVALUE ClearShapePoints (
			INT32 ShapeNum									//!< Shape points index to erase
			);

		//! Delete given shape from the list of shapes
		//!
		//! This method deletes a shape by swapping the last shape with 'ShapeNum', 
		//! then it resizes the number of shapes by -1.
		ERRVALUE DeleteShape (
			INT32 ShapeNum									//!< Shape number to delete
			);
			
		//! Find hyperindex shape that corresponds with the element given
		//! @return: Shape number if >= 0, -1 if none found, < -1 error.
		INT32 FindShape (
			ELEMTYPE ElemType,							//!< Type of element to find shape for
			INT32 ElemNum									//!< Element number to find shape for
			);

		//! Obtain the extents of the hyperindex rvc object
		ERRVALUE GetExtents (
			DRECT3D& rect									//!< Extents of rvc hyperindex object shapes returned
			);

		//! Obtain the number of links in a rvc hyperindex object
		//! @return number of links
		INT32 GetNumLinks (
			);

		//! Obtain the number of shapes in a rvc hyperindex object
		//! @return number of shapes
		INT32 GetNumShapes (
			);
			
		//! Create a new rvc hyperindex object under a specified parent rvc object with name, description, and source
		ERRVALUE Make (
			const RVC::OBJECT& parent,		//!< Parent object to create style object under
			const RVC::DESCRIPTOR& Descriptor,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Create a new rvc style object under a specified parent rvc object with name, description, and source
		ERRVALUE Make (
			RVC::OBJITEM& ObjItem,	//!< Name, Desc of object to create, Created object location RETURNED
#ifdef RVC_CREDENTIALS_MANDATORY
			const char* Source,			//!< Source of new style object, default is application name
			const CREDENTIALS* Credentials //!< User credentials passed in, NULL for default behavior
#else
			const char* Source = 0,			//!< Source of new style object, default is application name
			const CREDENTIALS* Credentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Read a rvc hyperindex object link element
		ERRVALUE ReadLink (
			INT32 LinkNum,									//!< The link index to read
			LINK& link										//!< The link storage instance
			);

		//! Read a rvc hyperindex object shape element
		ERRVALUE ReadShape (
			INT32 ShapeNum,								//!< Shape to read
			SHAPE& shape									//!< Shape parmeters returned
			);

		//! Read a rvc hyperindex object shape element
		ERRVALUE ReadShape (
			INT32 ShapeNum,								//!< Shape to read
			SHAPE& shape,									//!< Shape parmeters returned
			POLYLINE& Points								//!< Shape points read in
			);
			
		//! Read a rvc hyperindex object shape element
		ERRVALUE ReadShape (
			INT32 ShapeNum,								//!< Shape to read
			SHAPE& shape,									//!< Shape parmeters returned
			SIMPLE_ARRAY<INT32>& Links					//!< HyperLink indexes returned
			);
			
		//! Read a rvc hyperindex object shape element
		ERRVALUE ReadShape (
			INT32 ShapeNum,								//!< Shape to read
			SHAPE& shape,									//!< Shape parmeters returned
			POLYLINE& Points,								//!< Shape points read in
			SIMPLE_ARRAY<INT32>& Links					//!< HyperLink indexes returned
			);
			
		//! Remove a link from a hyper shape
		ERRVALUE RemoveLinkFromShape (
			INT32 ShapeNum,								//!< Shape number to add to
			INT32 LinkNum									//!< Stored link to add
			);

		//! Write a rvc hyperindex object link, use AddLink() to add a link
		ERRVALUE WriteLink (
			INT32 LinkNum,									//!< Link index to write
			const LINK& link								//!< Link to write
			);

		//! Read a rvc hyperindex object shape element
		ERRVALUE WriteShape (
			INT32 ShapeNum,								//!< Shape to write
			const SHAPE& shape							//!< Shape parmeters to write
			);

		//! Write a rvc hyperindex object shape
		ERRVALUE WriteShape (
			INT32 ShapeNum,								//!< Shape index to write
			const SHAPE& shape,							//!< Shape parameters to write
			const POLYLINE& Points						//!< Shape boundary
			);

		//! Write a rvc hyperindex object shape
		ERRVALUE WriteShape (
			INT32 ShapeNum,								//!< Shape index to write
			const SHAPE& shape,							//!< Shape parameters to write
			const SIMPLE_ARRAY<INT32>& Links			//!< Link indexes for this shape to contain
			);

		//! Write a rvc hyperindex object shape
		ERRVALUE WriteShape (
			INT32 ShapeNum,								//!< Shape index to write
			const SHAPE& shape,							//!< Shape parameters to write
			const POLYLINE& Points,						//!< Shape boundary
			const SIMPLE_ARRAY<INT32>& Links			//!< Link indexes for this shape to contain
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! Implementation of RVC::OBJECT virtual methods
		virtual ERRVALUE v_CloseObject ();
		virtual OBJTYPE v_GetDftObjectType () const;
		virtual bool v_IsObjectOpen () const;
		virtual ERRVALUE v_OpenObject (OPENMODE OpenFlags, MDLGPARENT parent, const CREDENTIALS* Credentials);

		HYPERHANDLE* m_HyperHandle;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


class HYPERINDEX::LINK {
	public:

		//! Type of object or file link is to.
		enum TYPE {
			TYPE_DISP2DWINDOW =	0,
//			TYPE_RasterColor				// Obsolete
			TYPE_Raster =			2,
			TYPE_Vector =			3,
			TYPE_CAD =				4,
			TYPE_Layout =			5,
			TYPE_Group =			6,
			TYPE_TIN =				7,
			TYPE_TextFile =		8,
			TYPE_ExternalFile =	9,
			TYPE_URL =				10,
			TYPE_FileByAttrib =	11,
			TYPE_URLByAttrib =	12,
			TYPE_Shape =			13,
			TYPE_SMLScript =		14,
			TYPE_LAST
			};

		//! Reference type indicating what the link uses for reference.
		enum REFTYPE {
			REFTYPE_Invalid = 0,
			REFTYPE_FilePath,			//!< File path, use Get/SetRefFilePath to acccess
			REFTYPE_ObjItem,			//!< Object, use Get/SetRefObjItem to access
			REFTYPE_URL,				//!< URL, use Get/SetRefURL to access
			REFTYPE_TableField		//!< Database table.field, use Get/SetRefTableField to access
			};

		enum POSITION {
			POSITION_Click = 0,		//!< Center around mouse click
			POSITION_Center, 			//!< Center of image
			POSITION_AsSaved,			//!< As saved in layout (layouts only
			POSITION_Max
			};

		//! Values for how to scale view after following link.
		//! These do not numerically correspond to the HYPERLINK_Scale... values.
		enum SCALE {
			SCALE_1X = 0,				//!< Display at 1X zoom
			SCALE_FullView,			//!< Display to fit window
			SCALE_AsSaved,				//!< Display at scale saved with layout
			SCALE_Zoomed,				//!< Display at specified zoom factor
			SCALE_ToMap,				//!< Display at specified "map scale"
			SCALE_Max
			};

		//! Default constructor
		LINK (
			);

		// Default copy constructor, destructor and operator=() are used

		//! Determine if can use scale/position settings.
		bool CanUseScalePos (
			) const;

		//! Clear all entries in the instance
		void Clear (
			);

		const MISTRING& GetDescription (
			) const { return (m_Desc); }

		TYPE GetLinkType (
			) const { return (m_LinkType); }
			
		//! Map scale if m_ScaleType is SCALE_ToMap
		double GetMapScale (
			) const { return (m_MapScale); }

		const MISTRING& GetName (
			) const { return (m_Name); }

		//! Get type of object being linked to.
		//! Will return OBJTYPE_All if not linked to RVC object.
		OBJTYPE GetObjType (
			) const { return (GetObjType(m_LinkType)); }

		//! Get type of object being linked to.
		//! Will return OBJTYPE_All if not linked to RVC object.
		static OBJTYPE GetObjType (
			TYPE Type
			);

		POSITION GetPosition (
			) const { return (m_PosnType); }
			
		//! Get reference file path, only valid if GetRefType returns REFTYPE_FilePath.
		const FILEPATH& GetRefFilePath (
			) const { return (m_FilePath); }

		//! Get reference object item, only valid if GetRefType returns REFTYPE_ObjItem.
		ERRVALUE GetRefObjItem (
			RVC::OBJITEM& ObjItem					//!< Item returned, might not be resolved to actual object
			) const;

		//! Get reference attribute table and field, only valid if GetRefType returns REFTYPE_TableField.
		ERRVALUE GetRefTableField (
			RVC::OBJECTNAME& TableName,			//!< Table name returned
			RVC::OBJECTNAME& FieldName				//!< Field name returned
			) const;

		//! Get reference type.
		REFTYPE GetRefType (
			) const;

		//! Get reference URL, only valid if GetRefType returns REFTYPE_URL.
		const MISTRING& GetRefURL (
			) const { return (m_URL); }

		SCALE GetScaleType (
			) const { return (m_ScaleType); }

		//! Zoom factor if m_ScaleType is SCALE_Zoomed
		double GetZoomFactor (
			) const { return (m_ZoomFactor); }

		//! Determine if link is valid.
		//! Does not necessarily mean that link refers to an existing file/object/url, checks members
		//! for reasonable settings only.
		bool IsValid (
			) const;
			
		//! Obtain link information if the LinkTo is TYPE_FileByAttrib or TYPE_URLByAttrib
		//! @return 'True' if the attribute was read, false if not
		bool ReadLinkFromAttribute (
			const RVC::DBASE_ELEMENT& DBaseObj,
			const RVC::ELEMENT& Element
			);

		void SetDescription (
			const MISTRING& desc
			) { m_Desc = desc; }
		
		//! Set type of object/document to link to
		//! Will clear reference members if link type actually changed.
		void SetLinkType (
			TYPE type
			);

		void SetMapScale (
			double MapScale
			) { m_MapScale = MapScale; }
			
		void SetPosition (
			POSITION posn
			) { m_PosnType = posn; }

		//! Set reference file path, only valid if GetRefType returns REFTYPE_FilePath.
		ERRVALUE SetRefFilePath (
			const FILEPATH& filepath
			);

		//! Set reference object item.
		//! Will adjust link type to be appropriate for object type.
		ERRVALUE SetRefObjItem (
			const RVC::OBJITEM& ObjItem
			);

		//! Set reference attribute table and field, only valid if GetRefType returns REFTYPE_TableField.
		ERRVALUE SetRefTableField (
			const RVC::OBJECTNAME& TableName,
			const RVC::OBJECTNAME& FieldName
			);

		//! Set reference URL, only valid if GetRefType returns REFTYPE_URL.
		ERRVALUE SetRefURL (
			const MISTRING& URL
			);
			
		void SetScaleType (
			SCALE scale
			) { m_ScaleType = scale; }
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FILEPATH m_FilePath;
		MISTRING m_Name;
		MISTRING m_Desc;
		MISTRING m_URL;
		TYPE m_LinkType;				//!< Type of object/document to link to
		POSITION m_PosnType;
		SCALE m_ScaleType;
		double m_ZoomFactor;			//!< Zoom factor if m_ScaleType is SCALE_Zoomed
		double m_MapScale;			//!< Map scale if m_ScaleType is SCALE_ToMap

		void GetHyperLink (HYPERLINK& hlink, const FILEPATH& objfilepath) const;
		void SetHyperLink (const HYPERLINK& hlink, const FILEPATH& objfilepath);

		friend class HYPERINDEX;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

DEFINE_ENUM_OP_INCREMENT(HYPERINDEX::LINK::TYPE)

class HYPERINDEX::SHAPE {
	public:
		enum TYPE {
			TYPE_Deleted =			-1,	//!< Shape has been deleted
			TYPE_Rect =				0,		//!< 2 points, min and max
			TYPE_Circle =			1,  	//!< Center (X,Y) followed by radius in meters (rx,ry)
			TYPE_Polygon =			2,		//!< N points
			TYPE_VectPointAny =	5,		//!< Any (closest) vector point
			TYPE_VectLineAny =	6,		//!< Any (closest) vector line
			TYPE_VectPolyAny =	7,		//!< Any (closest) vector polygon
			TYPE_VectPoint =		8,		//!< Vector point
			TYPE_VectLine =		9,		//!< Vector line
			TYPE_VectPoly =		10,	//!< Vector polygon
			TYPE_CAD =				11,	//!< CAD
			TYPE_CAD_Any =			12,	//!< Any (closest) CAD element
			TYPE_ShapeElem =		13,	//!< Shape object element
			TYPE_ShapeElem_Any =	14,	//!< Any (closest) shape object element
			TYPE_TIN_Node =		15,	//!< TIN node
			TYPE_TIN_Node_Any =	16		//!< Any (closest) TIN node
			};			

		//! Check if shape type matches any element.
		//! @return 'True' if the shape type matches any element, false if not
		static bool IsAnyElem (
			TYPE ShapeType
			);

		//! Default constructor
		SHAPE ();

		// Default copy constructor, destructor and operator=() are used

		void Clear ();

		//! Get color of the shape, if applicable
		//! @return Shape color
		const COLOR& GetColor (
			 ) const { return (m_Color); }
		
		//! Element number for geometric links
		//! @return Element number that the shape is attached to
		INT32 GetElemNum (
			) const { return (m_ElemNum); }
		
		//! Get extents of the shape, if applicable
		//! @return Shape extents
		const DRECT3D& GetExtents (
			) const { return (m_Extents); }
			
		//! Get number of links attached to this shape
		//! @return Number of shape links
		INT32 GetNumLinks (
			) const { return (m_NumLinks); }
			
		//! Get the type of the shape
		//! @return RVC::HYPERINDEX::SHAPE::TYPE
		TYPE GetType (
			) const { return (m_ShapeType); }
			
		//! Determine if the shape has links
		//! @return 'True' if the shape has links, false if not
		bool HasLinks (
			) const { return (m_NumLinks > 0); }

		//! Determine if the shape has points
		//! @return 'True' if the shape has points, false if not
		bool HasPoints (
			) const { return (m_NumPoints > 0); }
			
		//! Check if shape type matches any element.
		//! @return 'True' if the shape type matches any element, false if not
		bool IsAnyElem (
			) const { return (IsAnyElem(m_ShapeType)); }

		//! Set color of the shape, if applicable
		void SetColor (
			const COLOR& color
			) { m_Color = color; }
		
		//! Set element number for geometric links
		void SetElemNum (
			INT32 ElemNum
			) { m_ElemNum = ElemNum; }
		
		//! Set extents of the shape, if applicable
		void SetExtents (
			const DRECT3D& extents
			) { m_Extents = extents; }
			
		//! Set the type of the shape
		void SetType (
			TYPE type
			) { m_ShapeType = type; }
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DRECT3D m_Extents;						// Extents of shape
		COLOR m_Color;								// Color of shape
		INT32 m_ElemNum;							// Element number for geometric links
		TYPE m_ShapeType;							// 0 = square, 1 = Circle, 2 = polygon
		UINT32 m_NumPoints;						// Square = 2, Circle = 2 (center, radius), poly = N
		UINT32 m_NumLinks;						// Number of links

		void SetShape (const HYPERSHAPE& hshape);
		void GetShape (HYPERSHAPE& hshape) const;

		friend class HYPERINDEX;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}		// End of RVC namespace

#endif	// INC_RVC_HYPERIDX_H
