/**
 * \file rvc/metadata.h 
 * \brief interface for the RVC::METADATA class.
 *
 * \if NODOC
 * $Id: metadata.h_v 1.16 2011/12/21 22:17:25 mju Exp $
 *
 * $Log: metadata.h_v $
 * Revision 1.16  2011/12/21 22:17:25  mju
 * Remove redundant hdr.
 *
 * Revision 1.15  2006/07/20 17:37:55  scowan
 * Added new format type.
 *
 * Revision 1.11  2005/02/17 21:03:02  scowan
 * Add format field to metadata information.
 *
 * Revision 1.10  2003/10/29 22:36:27  scowan
 * Added get valid parent set method.
 *
 * Revision 1.5  2001/06/26 16:54:45  scowan
 * Specify ismultiobject to turn off base class setting.
 *
 * Revision 1.1  2000/11/29 16:10:39  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_METADATA_H
#define INC_RVC_METADATA_H

#ifndef INC_RVC_TEXTOBJ_H
	#include <rvc/textobj.h>
#endif

namespace RVC {

//! Manages RVC::OBJTYPE_Metadata RVC objects.  
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_File
//!		- RVC::OBJTYPE_Folder
//!		- RVC::OBJTYPE_Table
//!		- All geospatial types
class METADATA : public RVC::TEXTOBJ {
	public:

		enum FORMAT {
		  	FORMAT_Unknown = 0,
			FORMAT_ISO19115 = 1,
			FORMAT_FGDC_STD_001_1998 = 2,
			FORMAT_ISO19139 = 3
			};

		//! Default constructor
		METADATA (
			);

		//! Copy constructor
		METADATA (
			const RVC::METADATA& obj
			);

		//! Destructor
		virtual ~METADATA (
			);

		//! Assignment
		RVC::METADATA& operator= (
			const RVC::METADATA& rhs
			);

		//! Get format of the metadata, see FORMAT enum for types
		//! @return FORMAT of the metadata
		FORMAT GetMetadataFormat (
			);
			
		//! Get correct OBJTYPESET for all valid METADATA parent OBJTYPE's
		//! @return OBJTYPESET for all valid METADATA parent OBJTYPE's
		static RVC::OBJTYPESET GetValidParentSet (
			);
			
		//! Link to external file with specified encoding
		//! Object name, description, and source are filled in automatically with defaults
		ERRVALUE Link (
			RVC::OBJECT& parent,									//!< Parent rvc object to create metadata object under
			const FILEPATH& LinkPath,							//!< Path to external text file, stores it as a relative path if possible
#ifdef RVC_CREDENTIALS_MANDATORY
			CHAR_ENCODING encoding,
			FORMAT Format,
			const CREDENTIALS* Credentials 
#else
			CHAR_ENCODING encoding = CHAR_ENCODING_ISO_2022,		//!< Encoding of text file
			FORMAT Format = FORMAT_Unknown,
			const CREDENTIALS* Credentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Make a rvc metadata object given parent rvc object and metadata string
		//! Object name, description, and source are filled in automatically with defaults
		ERRVALUE Make (
			RVC::OBJECT& parent,					//!< Parent rvc object to create metadata object under
			const MISTRING& string,
#ifdef RVC_CREDENTIALS_MANDATORY
			FORMAT Format,
			const CREDENTIALS* Credentials 
#else
			FORMAT Format = FORMAT_Unknown,
			const CREDENTIALS* Credentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Read rvc metadata object into a MISTRING
		ERRVALUE Read (
			MISTRING& string					//!< String to read metadata into
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;

		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		//! End of RVC namespace

#endif 	//!< INC_RVC_METADATA_H
