/**
 * \file rvc/shape.h 
 * \brief interface for the RVC::SHAPE class.
 *
 * \if NODOC
 * $Id: shape.h_v 1.63 2012/11/19 15:44:58 bcolombe Exp $
 *
 * $Log: shape.h_v $
 * Revision 1.63  2012/11/19 15:44:58  bcolombe
 * Add LINKPARM_SHPFUSIONTABLE
 *
 * Revision 1.62  2012/10/09 15:03:19  mju
 * Add "separate" to setupElemTransList.
 *
 * Revision 1.61  2012/05/29 15:28:28  mju
 * Add element::getAsRegion.
 * Use elemnum for some methods.
 *
 * Revision 1.60  2011/12/23 13:54:17  mju
 * Remove redundant hdrs.
 *
 * Revision 1.59  2011/11/08 18:24:36  mju
 * Remove getExtentsOptimization as replaced by getExtentsIfKnown.
 *
 * Revision 1.58  2011/11/07 16:52:34  mju
 * Add getExtentsIfKnown.
 *
 * Revision 1.57  2010/05/12 21:10:27  mju
 * Change v_optimize to v_scan.
 *
 * Revision 1.56  2010/05/05 20:40:56  mju
 * Add v_optimizeAutoLink override.
 *
 * Revision 1.55  2010/04/23 13:09:17  mju
 * Add v_findElementSet.
 *
 * Revision 1.54  2009/09/01 13:37:54  mju
 * Add m_recBuffer.
 *
 * Revision 1.53  2008/10/17 17:06:43  scowan
 * Added generate region method.
 *
 * Revision 1.52  2008/10/03 19:41:24  scowan
 * Moved the table extraction method to database class.
 *
 * Revision 1.51  2008/10/02 22:01:08  scowan
 * Added a method to copy database tables based on shape paradigm.
 *
 * Revision 1.50  2008/09/26 20:16:25  scowan
 * Oracle support.
 *
 * Revision 1.49  2008/09/10 15:18:39  scowan
 * Added extractToShape.
 *
 * Revision 1.48  2008/09/09 16:12:45  scowan
 * Added element class methods to access the record member
 *
 * Revision 1.47  2008/09/05 22:34:28  scowan
 * Added a geometric makeparm.
 *
 * Revision 1.46  2008/09/05 17:50:51  scowan
 * Added members to shape and shape element classes.
 *
 * Revision 1.45  2008/08/06 17:07:12  scowan
 * Added linkparms for oo4o oracle.
 *
 * Revision 1.44  2008/08/01 20:34:22  scowan
 * Added method to get connection string.
 *
 * Revision 1.43  2008/07/31 23:02:55  scowan
 * Added method to get the link type.
 *
 * Revision 1.42  2008/06/16 20:10:17  scowan
 * Added shape and table object link to las files.
 *
 * Revision 1.41  2008/05/29 22:11:48  scowan
 * Added getNumComponents method.
 *
 * Revision 1.40  2008/05/29 16:24:21  scowan
 * Added shape element methods.
 *
 * Revision 1.39  2008/04/08 20:04:21  scowan
 * Change default encoding of shape linking to utf8.
 *
 * Revision 1.38  2008/01/25 23:09:43  scowan
 * Added the element type enum for shape object for make method and makeparms.
 *
 * Revision 1.36  2007/08/23 22:03:45  dwilliss
 * added link type enum for oracle via occi
 *
 * Revision 1.35  2007/04/13 15:31:58  scowan
 * Added virtual readRegion method.
 *
 * Revision 1.34  2007/01/16 16:11:48  scowan
 * Added flag for app data maintenance.
 *
 * Revision 1.33  2006/12/29 21:16:13  scowan
 * Added readElement virtual methods.
 *
 * Revision 1.32  2006/11/21 13:50:18  mju
 * Fix make when credentials mandatory.
 *
 * Revision 1.31  2006/11/20 20:26:54  scowan
 * Added make, add, delete and write methods.
 *
 * Revision 1.28  2006/05/11 19:28:07  dwilliss
 * Added MySQL
 *
 * Revision 1.27  2005/12/21 23:09:05  scowan
 * Added support for the readNext virtual methods.
 *
 * Revision 1.24  2005/12/09 17:29:54  scowan
 * Updated to use credentials.
 *
 * Revision 1.23  2005/12/02 23:52:30  dwilliss
 * added PostGIS
 *
 * Revision 1.22  2005/11/10 17:02:31  dwilliss
 * Added link parms for geodatabase
 *
 * Revision 1.19  2004/12/18 00:08:06  scowan
 *  Added component field to findClosestElements.
 *
 * Revision 1.18  2004/08/24 22:26:33  scowan
 * Added virtual methods to get and set total line length.
 *
 * Revision 1.15  2004/05/27 22:06:34  scowan
 * Added methods to extract to cad.
 *
 * Revision 1.14  2004/03/22 16:37:43  scowan
 * Supported find closest elements method.
 *
 * Revision 1.13  2003/11/21 22:17:10  scowan
 * Support new Capabilities system.
 *
 * Revision 1.1  2003/05/07 15:02:46  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_SHAPE_H
#define INC_RVC_SHAPE_H

#ifndef INC_RVC_GEOMETRC_H
	#include <rvc/geometrc.h>
#endif

#ifndef INC_RVC_DBTABLE_H
	#include <rvc/dbtable.h>
#endif

namespace RVC {

#ifndef GENERATING_DOXYGEN_OUTPUT
class SHAPEHANDLE;
#endif // GENERATING_DOXYGEN_OUTPUT

//! Manages RVC::OBJTYPE_Shape RVC objects.
//!	
//!	RVC Object Types that this class supports: RVC::OBJTYPE_Shape.
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_File
//!		- RVC::OBJTYPE_Folder
class SHAPE : public GEOMETRIC {
	public:

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class ELEMITERATOR;
		class LINKPARM_ARCSHAPE;
		class LINKPARM_LAS;
		class LINKPARM_ORACLE;
		class LINKPARM_ORACLE_OO4O;
		class LINKPARM_GEODATABASE;
		class LINKPARM_POSTGIS;
		class LINKPARM_SHPFUSIONTABLE;
		class LINKPARM_MYSQL;
		class LINKPARMS;
		class MAKEPARMS;
		class MAKEPARM_ARCSHAPE;
		class MAKEPARM_ORACLE;
//		class MAKEPARM_GEODATABASE;
//		class MAKEPARM_POSTGIS;
//		class MAKEPARM_MYSQL;
		#endif // GENERATING_DOXYGEN_OUTPUT
		
		// Type of elements stored in the database
		enum ELEMENTTYPE {
			ELEMENTTYPE_Unknown = 0,
			ELEMENTTYPE_Point,
			ELEMENTTYPE_Line,
			ELEMENTTYPE_SubRegion,
			ELEMENTTYPE_MultiPoint,
			ELEMENTTYPE_MultiLine,
			ELEMENTTYPE_Region,
			ELEMENTTYPE_MaxTypes
			};
	
		//! Type of external object
		enum LINKTYPE {
			LINKTYPE_Internal = 0,	//!< TNTmips internal format
			LINKTYPE_Oracle,			//!< Oracle Spatial via OO4O
			LINKTYPE_ArcShape,		//!< Arc/Info shapefile
			LINKTYPE_Geodatabase,	//!< Personal Geodatabase (*.mdb)
			LINKTYPE_PostGIS,			//!< PostgreSQL PostGIS
			LINKTYPE_MySQL,			//!< MySQL Spatial
			LINKTYPE_OracleOCCI,		//!< Oracle Spatial via OCCI
			LINKTYPE_LAS,				//!< LiDAR point data via LAS file
			LINKTYPE_FUSIONTABLE
			};
			
		//! Container for the geometric information pertaining to an element.
		//! The ELEMENT class can represent one or more of lines, points, or polygons
		class ELEMENT {
			public:
			
				//! Default constructor
				ELEMENT (
					);
					
				//! Copy constructor
				ELEMENT (
					const ELEMENT& rhs
					);
					
				//! Destructor
				~ELEMENT (
					);
					
				//! Assignment
				ELEMENT& operator= (
					const ELEMENT& rhs
					);

				//! Clear the element back to its constructed state
				void Clear (
					);

				//! Compute the extents of this element
				void ComputeExtents (
					DRECT3D& Extents
					) const;
					
				//! Convert point coordinates using affine transformation		
				void ConvertForward (
					const TRANS2D_AFFINE& taf
					);
				
				//! Convert point coordinates using generic transformation		
				ERRVALUE ConvertForward (
					const TRANS2D_MAPGEN& tmg
					);

				//! Convert point coordinates using affine transformation		
				void ConvertInverse (
					const TRANS2D_AFFINE& taf
					);
				
				//! Convert point coordinates using generic transformation		
				ERRVALUE ConvertInverse (
					const TRANS2D_MAPGEN& tmg
					);

				//! Get element coordinates as REGION2D.
				void GetAsRegion (
					REGION2D& region
					) const;

				//! Get the DIMENSION of this element
				//! @return DIMENSION_2D or DIMENSION_3D			
				DIMENSION GetDimension (
					) const { return (m_PolyLine.GetDimension()); }

				//! Get the ELEMTYPE of this element
				//! @return ELEMTYPE_Point, ELEMTYPE_Line, ELEMTYPE_Polygon or ELEMTYPE_Empty if no element			
				ELEMTYPE GetElemType (
					) const { return (m_ElemType); }
					
				//! Get the extents of the SHAPE::ELEMENT if known.
				//! @return Pointer to extents if known, NULL if not.
				const DRECT2D* GetExtentsIfKnown (
					) const { return (m_PolyLine.GetExtentsIfKnown()); }

				//! Get the lines if GetElemType() returns ELEMTYPE_Line
				//! If GetElemType() returns something different than ELEMTYPE_Line, 'PolyLineList' will be empty
				void GetLines (
					POLYLINELIST& PolyLineList			//!< List of lines RETURNED
					) const;

				//! Get the number of 'components' for this element.
				//! @return The number of components for this element.  Eg: if 5 lines compose this element, 
				//! then the return value will be 5.
				INT32 GetNumComponents (
					) const;

				//! Get the points if GetElemType() returns ELEMTYPE_Point
				//! @return Array of point elements as a POLYLINE
				const POLYLINE& GetPoints (
					) const { return (m_PolyLine); }
					
				//! Get the polygons if GetElemType() returns ELEMTYPE_Polygon
				//! If GetElemType() returns something different than ELEMTYPE_Polygon, 'region' will be empty
				void GetPolygons (
					REGION2D& region					//!< List of polygons (with possible islands) RETURNED
					) const;

				//! Get the record associated with this element.  
				//! By the definition of the shape object, there will exist a database record for each element.
				//! @return Constant reference to a DBTABLE::RECORD.
				const DBTABLE::RECORD& GetRecord (
					) const { return (m_Record); }

				//! Get the record associated with this element.  
				//! By the definition of the shape object, there will exist a database record for each element.
				//! @return Reference to a DBTABLE::RECORD.
				DBTABLE::RECORD& GetRecord (
					) { return (m_Record); }

				//! Is the element a valid element
				//! @return 'true' for a valid element, 'false' if not
				bool IsValid (
					) const;

				//! Set the points
				void SetPoints (
					const POLYLINE& PointList
					);
					
				//! Set a line
				void SetLine (
					const POLYLINE& PolyLine
					);

				//! Set the lines
				void SetLines (
					const POLYLINELIST& PolyLineList
					);

				//! Set a polygon
				void SetPolygon (
					const POLYLINE& Polygon
					);

				//! Set the polygons
				void SetPolygons (
					const REGION2D& region
					);

				//! Set the record associated with this element.  
				//! By the definition of the shape object, there will exist a database record for each element.
				void SetRecord (
					DBTABLE::RECORD& Record
					) { m_Record = Record; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				ELEMTYPE m_ElemType;		//!< _Point, _Line, _Polygon
				POLYLINE m_PolyLine;		//!< Storage for the shape element
				SIMPLE_ARRAY<INT32> m_Index;
				DBTABLE::RECORD m_Record;
				
				friend class SHAPE;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};


		//! Default constructor
		SHAPE (
			);

		//! Copy constructor
		SHAPE (
			const RVC::SHAPE& rhs
			);

		//! Destructor
		virtual ~SHAPE (
			);

		//! Assignment, will close and re-open object if possible
		RVC::SHAPE& operator= (
			const RVC::SHAPE& rhs
			);

		//! Add an element to the shape object
		//! Valid only if GEOMETRIC::CAPABILITY_CanAddToEnd is true.  See HasCapability() for usage.
		//! @return Number of the element added or <0 if an error occured
		INT32 Add (
			const ELEMENT& Element
			);

		//! Assign the record member of 'Element' to the base table
		ERRVALUE AssignTable (
			ELEMENT& Element
			);
			
		//! Delete an element from the shape object
		//! Valid only if GEOMETRIC::CAPABILITY_CanDeleteAnyElement is true.  See HasCapability() for usage.
		ERRVALUE Delete (
			ELEMNUM ElemNum
			);
			
		//! Obtain an element iterator to the beginning of the element list
		ELEMITERATOR ElementBegin (
			) const;

		//! Obtain an element iterator to the end of the element list
		ELEMITERATOR ElementEnd (
			) const;

		//! Generate a region subobject corresponding to the exterior outline of the shape object.
		//! Only valid if HasCapability(CAPABILITY_CanGenerateRegion) returns true.
		ERRVALUE GenerateRegionSubobject (
			);
			
		//! Obtain the connection string used to connect to the external data.
		ERRVALUE GetConnectionString (
			MISTRING& ConnectString
			);
			
		//! Obtain the element type set for this shape object
		ELEMENTTYPE GetElementType (
			) const;
			
		//! Obtain the LINKTYPE for this shape object
		LINKTYPE GetLinkType (
			) const;

		//! Link to an external shape object
		ERRVALUE Link (
			RVC::OBJECT& parent,
			const DESCRIPTOR& Descriptor,
			const LINKPARMS& LinkParms,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Link to new shape object given RVC::OBJITEM, creation parameters, and source
		ERRVALUE Link (
			const RVC::OBJITEM& ObjItem,
			const LINKPARMS& LinkParms,
			#ifdef RVC_CREDENTIALS_MANDATORY
				const char* Source,
				const CREDENTIALS* Credentials 
			#else
				const char* Source = 0,
				const CREDENTIALS* Credentials = 0				//!< User credentials passed in, NULL for default behavior
			#endif
			);

		//! Create new shape object given parent object, name, description, creation parameters, and source
		ERRVALUE Make (
			const RVC::OBJECT& parent,
			const DESCRIPTOR& Descriptor,
			const MAKEPARMS& MakeParms,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);
		
		//! Create new shape object given RVC::OBJITEM, creation parameters, and source
		ERRVALUE Make (
			RVC::OBJITEM& ObjItem,								//!< Parent object and DESCRIPTOR of object to create, new location RETURNED
			const MAKEPARMS& MakeParms,
			#ifdef RVC_CREDENTIALS_MANDATORY
				const char* Source,
				const CREDENTIALS* Credentials
			#else
				const char* Source = 0,
				const CREDENTIALS* Credentials = 0				//!< User credentials passed in, NULL for default behavior
			#endif
			);

		//! Read an element from the shape object source
		ERRVALUE Read (
			ELEMNUM ElemNum,
			ELEMENT& Element
			) const;
			
		//! Refresh the result set to reflect the current data source
		//! Only useful on systems like Oracle Spatial that can have the data changed out from underneath
		ERRVALUE Refresh (
			);
			
		//! Update / change an element in the shape object
		//! Valid only if GEOMETRIC::CAPABILITY_CanChangeElements is true.  See HasCapability() for usage.
		ERRVALUE Write (
			ELEMNUM ElemNum,
			const ELEMENT& Element
			);
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! Implementation of RVC::OBJECT virtual methods
		virtual ERRVALUE v_CloseObject ();
		virtual OBJTYPE v_GetDftObjectType () const;
		virtual bool v_IsObjectOpen () const;
		virtual ERRVALUE v_OpenObject (OPENMODE OpenMode, MDLGPARENT parent, const CREDENTIALS* Credentials);

		//! Implementation of RVC::GEOSPATIAL virtual methods
		virtual void v_GetExtents (DRECT3D &extents) const;
		virtual void v_GetScale (double& xscale, double& yscale) const;
		virtual double v_GetZOffset () const;
		virtual double v_GetZScale () const;
		virtual bool v_HasCapability (int Capability) const;
		virtual ERRVALUE v_ScanAutoLink ();
		virtual void v_SetScale (double xscale, double yscale);
		virtual void v_SetZOffset (double zoffset);
		virtual void v_SetZScale (double zscale);

		//! Implementation of RVC::GEOMETRIC pure virtual methods
		virtual ERRVALUE v_ExtractTo (RVC::GEOMETRIC& DestObj, RVC::GEOMETRIC::ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg, RVC::GEOMETRIC::EXTRACTFLAGS ExtractFlags) const;
		virtual INT32 v_FindClosestElementLow (ELEMTYPE ElemType, const DPOINT2D& SearchPt, GEOMETRIC::SEARCHFILTER* SearchFilter, double SearchDist, double* DistFromElem) const;
		virtual ERRVALUE v_GetElementExtents (ELEMTYPE ElemType, INT32 ElemNum, DRECT3D& Extents) const;
		virtual INT32 v_GetNextElementID (ELEMTYPE ElemType) const;
		virtual INT32 v_GetNumElements (ELEMTYPE ElemType) const;
		virtual POINTTYPE v_GetPointType () const;
		virtual INT32 v_GetPointSize () const;
		virtual QSEARCHTREE* v_GetSearchTree (ELEMTYPE ElemType) const;
		virtual double v_GetTotalLineLength () const;
		virtual bool v_IsElementDeleted (const RVC::ELEMENT& Element) const;
		virtual bool v_IsEmpty () const;
		virtual ERRVALUE v_SetExtents (const DRECT3D &extents);
		virtual ERRVALUE v_SetNextElementID (ELEMTYPE ElemType, INT32 NextElementID);
		virtual void v_SetSearchTree (ELEMTYPE ElemType, QSEARCHTREE* SearchTree) const;
		virtual ERRVALUE v_SetTotalLineLength (double length);
		virtual int v_SetupElemTransList (const GEOMETRIC::ELEMSELECTPARMS& SelectParms, GEOMETRIC::ELEMTRANSLIST& TransList, GEOMETRIC::ELEMSELECTFILTER *SelectFilter, bool separate) const;
		virtual bool v_TestElementAgainstRegion (const RVC::ELEMENT& Element, const REGION2D& region, TESTCRITERIA TestMode) const;
		virtual ERRVALUE v_UpdateExtents (const DRECT3D &extents);
		
		//! Implementation of RVC::GEOMETRIC virtual methods
		virtual ERRVALUE v_FindClosestElementsLow (const DPOINT2D& SearchPt, double SearchDist, SEARCHFILTER* SearchFilter, SIMPLE_ARRAY<RVC::ELEMENT>& ElementList, COMPONENT Component) const;
		virtual ERRVALUE v_FindElementDrawSet (ELEMTYPE ElemType, const DRECT2D& SearchExtents, SEARCHRESULT& SearchResult) const;
		virtual ERRVALUE v_FindElementSearchSet (ELEMTYPE ElemType, const DRECT2D& SearchExtents, SEARCHRESULT& SearchResult) const;
		virtual ERRVALUE v_FindElementSet (ELEMTYPE ElemType, const DRECT2D& SearchExtents, UNSIGNEDSET& ElemSet, bool BuildIndex) const;
		virtual ERRVALUE v_ReadElement (INT32 ElemNum, POLYLINELIST& LineList, DIMENSION dim) const;
		virtual ERRVALUE v_ReadElement (INT32 ElemNum, POLYLINE& PointList, DIMENSION dim) const;
		virtual ERRVALUE v_ReadElement (INT32 ElemNum, REGION2D& Region, DIMENSION dim) const;
		virtual ERRVALUE v_ReadNextLine (RVC::ELEMENT& Element, POLYLINELIST& LineList, DIMENSION dim) const;
		virtual ERRVALUE v_ReadNextPoint (RVC::ELEMENT& Element, POLYLINE& PointList, DIMENSION dim) const;
		virtual ERRVALUE v_ReadNextRegion (RVC::ELEMENT& Element, REGION2D& Region, DIMENSION dim) const;
		virtual ERRVALUE v_ReadRegion (const BITSET_UNOWNED& ElemSet, REGION2D& Region, DIMENSION dim, bool NoIslands) const;
		
		ERRVALUE ExtractToCAD (RVC::GEOMETRIC& DestObj, RVC::GEOMETRIC::ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg, RVC::GEOMETRIC::EXTRACTFLAGS ExtractFlags) const;
		ERRVALUE ExtractToRegion (RVC::GEOMETRIC& DestObj, RVC::GEOMETRIC::ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg, RVC::GEOMETRIC::EXTRACTFLAGS ExtractFlags) const;
		ERRVALUE ExtractToShape (RVC::GEOMETRIC& DestObj, RVC::GEOMETRIC::ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg, RVC::GEOMETRIC::EXTRACTFLAGS ExtractFlags) const;
		ERRVALUE ExtractToVector (RVC::GEOMETRIC& DestObj, RVC::GEOMETRIC::ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg, RVC::GEOMETRIC::EXTRACTFLAGS ExtractFlags) const;
		
		SHAPEHANDLE *m_ShapeHandle;
		mutable QSEARCHTREE *m_ShapeQTree;
		mutable ELEMENT m_WorkElement;
		mutable bool m_SetInstanceData;		//!< This object set the instance data if true, it needs to remove it on closing
		DBTABLE m_ShapeTable;
		mutable SIMPLE_ARRAY<UINT8> m_RecBuffer;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	

//! Constant iterator class
//!
//! This class is used to iterate through a shape objects elements
//! See methods ElementBegin() and ElementEnd() for obtaining an element iterator
class SHAPE::ELEMITERATOR {
	public:

		//! Default constructor
		ELEMITERATOR (
			) : 
			m_ElemNum(-1),
			m_CurrentObj(0),
			m_Element(0)
			{}

		//! Internal constructor used by ElementEnd() methods
		ELEMITERATOR (
			const RVC::SHAPE* shape,
			INT32 ElemNum
			) : 
			m_ElemNum(ElemNum),
			m_CurrentObj(shape),
			m_Element(0)
			{}

		//! Copy constructor
		ELEMITERATOR (
			const ELEMITERATOR& rhs
			);

		//! Internal constructor used by ElementBegin() methods
		ELEMITERATOR (
			const RVC::SHAPE* shape
			);

		//! Destructor
		virtual ~ELEMITERATOR (
			);

		//! Assignment
		ELEMITERATOR& operator= (
			const ELEMITERATOR& rhs
			);

		//! Dereference
		const SHAPE::ELEMENT& operator* (
			) const { return (*m_Element); }

		//! Dereference arrow
		const SHAPE::ELEMENT* operator-> (
			) const { return (&**this); }

		//! Pre-increment
		ELEMITERATOR& operator++ (
			);

		//! Pre-decrement
		ELEMITERATOR& operator-- (
			);

		//! Comparison for equality
		bool operator== (
			const ELEMITERATOR& rhs
			) const { return (m_ElemNum == rhs.m_ElemNum && m_CurrentObj == rhs.m_CurrentObj); }

		//! Comparison for inequality
		bool operator!= (
			const ELEMITERATOR& rhs
			) const { return (!(*this == rhs)); }

		//! Get the current element number
		//! @return Element number			
		INT32 GetElemNum (
			) const { return (m_ElemNum); }

	protected:
		INT32 m_ElemNum;
		const SHAPE *m_CurrentObj;
		SHAPE::ELEMENT* m_Element;
	};
			
			
//===================================================================================================================
//	SHAPE::LINKPARMS and derived classes

//! Base class definition for link parameters to link to external shape objects
//! An instance of this class cannot be created, must use the derived classes below
class SHAPE::LINKPARMS {
	public:

		//! Destructor
		virtual ~LINKPARMS (
			) {}

		//! Get the encoding of the external table			
		CHAR_ENCODING GetEncoding (
			) const {return (m_Encoding);}

		//! Get the type of external link
		//! @return External link type
		SHAPE::LINKTYPE GetLinkType (
			) const {return (m_LinkType);}

		//! Get the link stirng
		//! @return Link string
		const MISTRING& GetLinkString (
			) const {return (m_LinkString);}

		//! Get whether or not to store the password in the link object.
		//! This is ignored for link types which do not require a password.  It
		//! is used mainly for links to RDBMS-based shape objects such as
		//! PostGIS and Oracle Spatial.
		bool GetStorePasswordInLinkObject (
			) const { return (m_bStorePasswordInLinkObject); }

		//! Get the table name.  Only useful for data source type spatial objects
		//! @return Table name
		const MISTRING& GetTableName (
			) const {return (m_LinkTable);}

	protected:
	
		LINKPARMS (
			const MISTRING& LinkString,
			SHAPE::LINKTYPE LinkType,
			CHAR_ENCODING Encoding,
			bool StorePasswordInLinkObject
			) :
			m_LinkString(LinkString),
			m_LinkType(LinkType),
			m_Encoding(Encoding),
			m_bStorePasswordInLinkObject(StorePasswordInLinkObject)
			{}
			
		void SetTableName (
			const MISTRING& TableName
			) { m_LinkTable = TableName; }
	
	private:	
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MISTRING m_LinkString;
		MISTRING m_LinkTable;
		SHAPE::LINKTYPE m_LinkType;
		CHAR_ENCODING m_Encoding;
		bool 	m_bStorePasswordInLinkObject;
		
		friend class SHAPE;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

	
//! Setup parameters to link to an Arc/Info Shape file
class SHAPE::LINKPARM_ARCSHAPE : public RVC::SHAPE::LINKPARMS {
	public:	
		LINKPARM_ARCSHAPE (
			const FILEPATH& filename,
			CHAR_ENCODING Encoding = CHAR_ENCODING_UTF8
			) : LINKPARMS(filename.GetOSPath(), LINKTYPE_ArcShape, Encoding, false) {}
	};

	
//! Setup parameters to link to a LiDAR LAS file
class SHAPE::LINKPARM_LAS : public RVC::SHAPE::LINKPARMS {
	public:	
		LINKPARM_LAS (
			const FILEPATH& filename,
			CHAR_ENCODING Encoding = CHAR_ENCODING_ASCII
			) : LINKPARMS(filename.GetOSPath(), LINKTYPE_LAS, Encoding, false) {}
	};


//! Setup parameters to link to an Oracle Spatial external database table
class SHAPE::LINKPARM_ORACLE : public RVC::SHAPE::LINKPARMS {
	public:	
		LINKPARM_ORACLE (
			const MISTRING& LinkString,
			const MISTRING& TableName,
			CHAR_ENCODING Encoding = CHAR_ENCODING_ASCII,
			bool StorePasswordInLinkObject = false
			) : 
			LINKPARMS(LinkString, LINKTYPE_OracleOCCI, Encoding, StorePasswordInLinkObject)
			{
			SetTableName(TableName);
			}
	};

//! Setup parameters to link to an Oracle Spatial external database table
class SHAPE::LINKPARM_ORACLE_OO4O : public RVC::SHAPE::LINKPARMS {
	public:	
		LINKPARM_ORACLE_OO4O (
			const MISTRING& LinkString,
			const MISTRING& TableName,
			CHAR_ENCODING Encoding = CHAR_ENCODING_ASCII,
			bool StorePasswordInLinkObject = false
			) : 
			LINKPARMS(LinkString, LINKTYPE_Oracle, Encoding, StorePasswordInLinkObject)
			{
			SetTableName(TableName);
			}
	};

//! Setup parameters to link to an Arc GeoDatabase external database table
class SHAPE::LINKPARM_GEODATABASE : public RVC::SHAPE::LINKPARMS {
	public:	
		LINKPARM_GEODATABASE (
			const MISTRING& LinkString,
			const MISTRING& TableName,
			bool StorePasswordInLinkObject = false
			) : 
			LINKPARMS(LinkString, LINKTYPE_Geodatabase, CHAR_ENCODING_Unicode, StorePasswordInLinkObject)
			{
			SetTableName(TableName);
			}
	};


//! Setup parameters to link to a PostGIS external database table
class SHAPE::LINKPARM_POSTGIS : public RVC::SHAPE::LINKPARMS {
	public:	
		LINKPARM_POSTGIS (
			const MISTRING& LinkString,
			const MISTRING& TableName,
			bool StorePasswordInLinkObject = false
			) : 
			LINKPARMS(LinkString, LINKTYPE_PostGIS, CHAR_ENCODING_Unicode, StorePasswordInLinkObject)
			{
			SetTableName(TableName);
			}
	};

//! Setup parameters to link to a Fusion Table external database table
class SHAPE::LINKPARM_SHPFUSIONTABLE : public RVC::SHAPE::LINKPARMS {
	public:	
		LINKPARM_SHPFUSIONTABLE (
			const MISTRING& LinkString,
			const MISTRING& TableName,
			bool StorePasswordInLinkObject = false
			) : 
			LINKPARMS(LinkString, LINKTYPE_FUSIONTABLE, CHAR_ENCODING_Unicode, StorePasswordInLinkObject)
			{
			SetTableName(TableName);
			}
	};

//! Setup parameters to link to a MySQL external database table
class SHAPE::LINKPARM_MYSQL : public RVC::SHAPE::LINKPARMS {
	public:	
		LINKPARM_MYSQL (
			const MISTRING& LinkString,
			const MISTRING& TableName,
			bool StorePasswordInLinkObject = false
			) : 
			LINKPARMS(LinkString, LINKTYPE_MySQL, CHAR_ENCODING_Unicode, StorePasswordInLinkObject)
			{
			SetTableName(TableName);
			}
	};


//! Base class definition for make parameters to make external shape objects
//! An instance of this class cannot be created, must use the derived classes below
class SHAPE::MAKEPARMS : public GEOMETRIC::MAKEPARMSBASE {
	public:

		//! Destructor
		virtual ~MAKEPARMS (
			) {}
			
		//! Get the connection stirng
		//! @return connection string
		const MISTRING& GetConnectionString (
			) const {return (m_ConnectString);}
			
		const SPATREF::COORDREFSYS& GetCoordRefSys (
			) const { return (m_CoordRefSys); }
			
		//! Get the element type of external link
		//! @return External element type
		ELEMENTTYPE GetElemType (
			) const {return (m_ElemType);}

		const DBTABLE::FIELDINFOARRAY& GetFieldInfo (
			) const { return (m_FieldArray); }
			
		//! Get the type of external link
		//! @return External link type
		SHAPE::LINKTYPE GetLinkType (
			) const {return (m_LinkType);}

		bool IsValidElementType (
			ELEMENTTYPE ElementType
			) const { return (v_IsValidElementType(ElementType)); }
			
	protected:

		//! Default constructor, initialize to 2D by default
		MAKEPARMS (
			LINKTYPE LinkType,
			ELEMENTTYPE ElemType,
			const MISTRING& ConnectString,
			const DBTABLE::FIELDINFOARRAY& FieldArray,
			const SPATREF::COORDREFSYS& CoordRefSys,
			POINTTYPE PointType = POINTTYPE_2DXY
			);

		//! Template constructor
		MAKEPARMS (
			const SHAPE& Template,
			LINKTYPE LinkType,
			const MISTRING& ConnectString,
			const SPATREF::COORDREFSYS* CoordRefSys = 0		//!< If NULL, get COORDREFSYS from 'GeometricObj'
			);
			
		//! From Geometric constructor
		MAKEPARMS (
			const GEOMETRIC& GeometricObj,
			ELEMENTTYPE ElemType,
			LINKTYPE LinkType,
			const MISTRING& ConnectString,
			const DESCRIPTOR& BaseTable,
			const SPATREF::COORDREFSYS* CoordRefSys = 0		//!< If NULL, get COORDREFSYS from 'GeometricObj'
			);
			
	private:
		LINKTYPE m_LinkType;
		ELEMENTTYPE m_ElemType;
		MISTRING m_ConnectString;
		DBTABLE::FIELDINFOARRAY m_FieldArray;
		SPATREF::COORDREFSYS m_CoordRefSys;

		bool v_IsValidElementType (ELEMENTTYPE ElementType) const;
	};


class SHAPE::MAKEPARM_ARCSHAPE : public SHAPE::MAKEPARMS {
	public:	
		MAKEPARM_ARCSHAPE (
			ELEMENTTYPE ElemType,
			const FILEPATH& filename,
			const DBTABLE::FIELDINFOARRAY& FieldArray,
			const SPATREF::COORDREFSYS& CoordRefSys,
			POINTTYPE PointType = POINTTYPE_2DXY
			) : MAKEPARMS(LINKTYPE_ArcShape, ElemType, filename.GetOSPath(), FieldArray, CoordRefSys, PointType) {}
			
		MAKEPARM_ARCSHAPE (
			const SHAPE& Template,
			const FILEPATH& filename,
			const SPATREF::COORDREFSYS* CoordRefSys = 0		//!< If NULL, get COORDREFSYS from 'GeometricObj'
			) : MAKEPARMS(Template, LINKTYPE_ArcShape, filename.GetOSPath(), CoordRefSys) {}
			
		MAKEPARM_ARCSHAPE (
			const GEOMETRIC& GeometricObj,
			ELEMENTTYPE ElemType,
			const FILEPATH& filename,
			const DESCRIPTOR& BaseTable,
			const SPATREF::COORDREFSYS* CoordRefSys = 0		//!< If NULL, get COORDREFSYS from 'GeometricObj'
			) : MAKEPARMS(GeometricObj, ElemType, LINKTYPE_ArcShape, filename.GetOSPath(), BaseTable, CoordRefSys) {}
			
	private:
		bool v_IsValidElementType (ELEMENTTYPE ElementType) const;
	};


class SHAPE::MAKEPARM_ORACLE : public SHAPE::MAKEPARMS {
	public:	
		MAKEPARM_ORACLE (
			ELEMENTTYPE ElemType,
			const MISTRING& ConnectString,
			const DBTABLE::FIELDINFOARRAY& FieldArray,
			const SPATREF::COORDREFSYS& CoordRefSys,
			POINTTYPE PointType = POINTTYPE_2DXY
			) : MAKEPARMS(LINKTYPE_OracleOCCI, ElemType, ConnectString, FieldArray, CoordRefSys, PointType) {}
			
		MAKEPARM_ORACLE (
			const SHAPE& Template,
			const MISTRING& ConnectString,
			const SPATREF::COORDREFSYS* CoordRefSys = 0		//!< If NULL, get COORDREFSYS from 'GeometricObj'
			) : MAKEPARMS(Template, LINKTYPE_OracleOCCI, ConnectString, CoordRefSys) {}
			
		MAKEPARM_ORACLE (
			const GEOMETRIC& GeometricObj,
			ELEMENTTYPE ElemType,
			const MISTRING& ConnectString,
			const DESCRIPTOR& BaseTable,
			const SPATREF::COORDREFSYS* CoordRefSys = 0		//!< If NULL, get COORDREFSYS from 'GeometricObj'
			) : MAKEPARMS(GeometricObj, ElemType, LINKTYPE_OracleOCCI, ConnectString, BaseTable, CoordRefSys) {}
			
	private:
		bool v_IsValidElementType (ELEMENTTYPE ElementType) const;
	};

}		//! End of RVC namespace

#endif 	//!< INC_RVC_SHAPE_H
