/**
 * \file rvc/uiformobjectlist.h
 * \brief RVC::UI::FORM_OBJECT_LIST class definitions
 *
 * \if NODOC
 * $Id: uiformobjectlist.h_v 1.7 2012/07/27 14:03:25 mju Exp $
 *
 * $Log: uiformobjectlist.h_v $
 * Revision 1.7  2012/07/27 14:03:25  mju
 * Add getObjTypeSet.
 *
 * Revision 1.6  2011/12/28 13:56:49  mju
 * Remove redundant hdr.
 *
 * Revision 1.4  2007/05/16 21:38:23  scowan
 * Added setObjectList method that only works before calling create method.
 *
 * Revision 1.1  2006/07/21 21:19:41  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_UIFORMOBJECTLIST_H
#define	INC_RVC_UIFORMOBJECTLIST_H

#ifndef	INC_MGUI_LISTBOX_H
#include <mgui/listbox.h>
#endif

#ifndef	INC_RVC_OBJITEMLIST_H
#include <rvc/objitemlist.h>
#endif

#ifndef	INC_RVC_OTYPESET_H
#include <rvc/otypeset.h>
#endif

#ifndef	INC_RVC_GETOBJFLAGS_H
#include <rvc/getobjflags.h>
#endif

namespace RVC {

class GETOBJFILTER;

namespace UI {
	
//===================================================================================================================
//! Form containing basic object selection list with "select", "remove", and "remove all" buttons.
class FORM_OBJECT_LIST : public MGUI::FORM_COMPOSITE {
	public:
	
		//! Constructor.
		FORM_OBJECT_LIST ();

		//! Destructor.
		virtual ~FORM_OBJECT_LIST ();
			
		//! Clears out all of the selected objects
		void Clear (
			bool notify							//!< Send notifications via virtual methods
			);

		//! Create object list form.
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,				//!< Parent pane
			const MISTRING& promptstr,							//!< Prompt string to use with DlgGetObjects
			const RVC::OBJTYPESET& ObjTypeSet,				//!< Set of object types that are selectable
			GETOBJFLAGS GetObjFlags = GETOBJFLAG_None,	//!< Flags for DlgGetObjects
			RVC::GETOBJFILTER* GetObjFilter = 0,			//!< Filter for DlgGetObjects
			int ListHeight = 5,									//!< Initial list box height
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand
			);

		//! Get the current object, will return an empty OBJITEM if there is nothing in the list
		const RVC::OBJITEM& GetCurrentObject () const;
			
		//! Get the list of objects in the list			
		const RVC::OBJITEMLIST& GetObjectList () const
			{ return (m_ObjItemList); }

		MGUI::LAYOUT_PANE& GetBtnPane (
			) { return (m_BtnPane); }

		//! Get set of allowed object types.
		const RVC::OBJTYPESET& GetObjTypeSet () const
			{ return (m_ObjTypeSet); }

		//! Is the list empty?			
		bool IsEmpty () const
			{ return (m_ObjItemList.IsEmpty()); }

		//! Select objects via form.
		//! Behaves same as when 'Select' button pressed.
		ERRVALUE Select (
			bool notify							//!< Send notifications via virtual methods
			);
			
		//! Set delegate to call for any change to objects in list.
		void SetDelegateOnListChanged (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnListChanged = delegate; }

		//! Set delegate to call for each object added to list.
		void SetDelegateOnObjectAdded (
			FastDelegate<void(const RVC::OBJITEM&, INT32)> delegate	//!< Delegate parameters: object item, item index in list
			) { m_DelegateOnObjectAdded = delegate; }

		//! Set delegate to call when single object removed from list.
		void SetDelegateOnObjectRemoved (
			FastDelegate<void(INT32)> delegate								//!< Delegate parameter: item index in list
			) { m_DelegateOnObjectRemoved = delegate; }

		//! Set delegate to call when object selected (highlighted) in list.
		void SetDelegateOnObjectSelected (
			FastDelegate<void(const RVC::OBJITEM&, INT32)> delegate	//!< Delegate parameters: object item, item index in list
			) { m_DelegateOnObjectSelected = delegate; }

		//! Set delegate to call when objects added to list.
		void SetDelegateOnObjectsAdded (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnObjectsAdded = delegate; }

		//! Set delegate to call when all objects removed from list.
		void SetDelegateOnObjectsAllRemoved (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnObjectsAllRemoved = delegate; }

		//! Set the list of objects in the list
		//! Must be called before Create() method.
		void SetObjectList (
			const RVC::OBJITEMLIST& ObjItemList
			);

	protected:

		//! Called for any change to objects in list.
		//! Default implementation invokes corresponding delegate.
		//! If override call when/if needed to invoke delegate.
		virtual void v_OnListChanged ();

		//! Called for each object added to list.
		//! Default implementation invokes corresponding delegate.
		//! If override call when/if needed to invoke delegate.
		virtual void v_OnObjectAdded (const RVC::OBJITEM& ObjItem, INT32 ItemNum);

		//! Called when single object removed from list.
		//! Default implementation invokes corresponding delegate.
		//! If override call when/if needed to invoke delegate.
		virtual void v_OnObjectRemoved (INT32 ItemNum);

		//! Called when object selected (highlighted) from list.
		//! Default implementation invokes corresponding delegate.
		//! If override call when/if needed to invoke delegate.
		virtual void v_OnObjectSelected (const RVC::OBJITEM& ObjItem, INT32 ItemNum);

		//! Called when objects added to list.
		//! Default implementation invokes corresponding delegate.
		//! If override call when/if needed to invoke delegate.
		virtual void v_OnObjectsAdded ();

		//! Called when all objects removed from list.
		//! Default implementation invokes corresponding delegate.
		//! If override call when/if needed to invoke delegate.
		virtual void v_OnObjectsAllRemoved ();
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		static RVC::OBJITEM *s_pNullItem;
		DELEGATE_VOID_NOPARMS m_DelegateOnListChanged;
		FastDelegate<void(const RVC::OBJITEM&, INT32)> m_DelegateOnObjectAdded;
		FastDelegate<void(INT32)> m_DelegateOnObjectRemoved;
		FastDelegate<void(const RVC::OBJITEM&, INT32)> m_DelegateOnObjectSelected;
		DELEGATE_VOID_NOPARMS m_DelegateOnObjectsAdded;
		DELEGATE_VOID_NOPARMS m_DelegateOnObjectsAllRemoved;
		MGUI::CTRL_LISTBOX m_ListBox;
		MGUI::CTRL_PUSHBUTTON m_BtnSelect;
		MGUI::CTRL_PUSHBUTTON m_BtnRemove;
		MGUI::CTRL_PUSHBUTTON m_BtnRemoveAll;
		MGUI::LAYOUT_PANE m_BtnPane;
		RVC::OBJITEMLIST m_ObjItemList;
		MISTRING m_PromptStr;
		RVC::OBJTYPESET m_ObjTypeSet;
		GETOBJFLAGS m_GetObjFlags;
		RVC::GETOBJFILTER* m_GetObjFilter;

		void OnListSelection ();
		void OnBtnRemove ();
		void OnBtnRemoveAll ();
		void OnBtnSelect ();

		FORM_OBJECT_LIST (const FORM_OBJECT_LIST&);
		FORM_OBJECT_LIST& operator= (const FORM_OBJECT_LIST&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//===================================================================================================================
}		// End namespace UI
}		// End namespace RVC

#endif	// INC_RVC_UIFORMOBJECTLIST_H
