/**
 * \file spatmod/imagefilterstereodual.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_STEREO_DUAL definition.
 *
 * \if NODOC
 * $Id: imagefilterstereodual.h_v 1.3 2012/04/05 21:21:05 mju Exp $
 *
 * $Log: imagefilterstereodual.h_v $
 * Revision 1.3  2012/04/05 21:21:05  mju
 * Support chroma-coding areas of missing elevation.
 *
 * Revision 1.2  2012/04/04 21:03:13  mju
 * Use stereoshift in construction.
 *
 * Revision 1.1  2011/10/31 20:51:07  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERSTEREODUAL_H
#define	INC_SPATMOD_IMAGEFILTERSTEREODUAL_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_SPATMOD_IMAGESECTION_H
#include <spatmod/imagesection.h>
#endif

#ifndef	INC_SPATMOD_IMAGESAMPLEITERATOR_H
#include <spatmod/imagesampleiterator.h>
#endif

#ifndef	INC_SPATMOD_IMAGEHELPERRESAMPLE_H
#include <spatmod/imagehelperresample.h>
#endif

#ifndef	INC_MI32_TRANS2MAPGEN_H
#include <mi32/trans2dmapgen.h>
#endif

#ifndef	INC_MI32_RECT_H
#include <mi32/rect.h>
#endif

#ifndef	INC_MI32_STEREOSHIFT_H
#include <mi32/stereoshift.h>
#endif

namespace SPATMOD {
namespace IMAGE {

class BUFFER;

//=====================================================================================================================

//! Filter to generate stereo image pair.
//! This filter creates a pair of images, with first (left), second (right) or both displaced for
//! subsequent stereo viewing.  The sequence of image sample will be duplicated, thus if the source has 4 samples of
//! RGBA, the result will have have 8 samples of RGBARGBA.  The input must have an "alpha" sample, and the output
//! alpha values will reflect the "validity" such that an output alpha of 0 is considered completely transparent.
//!
//! All samples from the drape stage must have the same data type.
//! The terrain stage must have a single component of any numeric data type.
//!
//! The value for DisplacementScale should be chosen such that the maximum actual displacement is reasonably small,
//! typically no more than 50 pixels, both in order to maintain visual stereo fusion, as well as for processing efficiency.
//! This value effectively controls the "vertical exaggeration" and relative "depth" of the rendering.
//! The value for DisplacementZeroValue must be within the range of the overall terrain value and typically will be
//! near the maximum terrain value for a particular rendering.  This may require implementation of user controls
//! for adjustment, or a preprocessing pass may be made to estimate this value based on the portion of the terrain used
//! in the rendering.  Rendered pixels on the terrain "above" the "zero displacement" will appear "closer" to the viewer
//! than the rendering plane (monitor screen).  Pixels on the terrain far "below" the "zero displacement" level may appear
//! with too much separation.
//!
//! The coordinate transformation between resulting image and input stage is computed using georeference as
//! specified in the constructor, or georeference from the specified reference image, along with the georeference
//! from the input stage.  An optional transformation accuracy may be specified via SetAccuracy() to allow use
//! of efficient affine transformation if appropriate.
//! Note that the source image will not be processed "sequentially", and the sections specified
//! in PullBegin for the source will overlap.  It is recommended that this filter be placed early in the pipeline or
//! that a suitable caching filter be placed ahead of it if the input requires significant computation or sequential
//! iteration.
class FILTER_STEREO_DUAL : public FILTER {
	public:

		//! Construct using specified dimensions and georeference.
		FILTER_STEREO_DUAL (
			STAGE *pStageDrape,						//!< Pointer to stage to "drape" onto terrain
			bool TakeOwnershipDrape,				//!< Filter takes ownership of drape stage and will delete when needed
			STAGE *pStageTerrain,					//!< Pointer to stage for terrain elevation
			bool TakeOwnershipTerrain,				//!< Filter takes ownership of terrain stage and will delete when needed
			const STEREOSHIFT& StereoShift,		//!< Stereo shift settings
			const DIMENSIONS& Dimensions,			//!< Resulting image dimensions
			const GEOREFERENCE& Georeference,	//!< Georeference for resulting image
			RESAMPLEMETHOD ResampleMethod			//!< Resampling method
			);

		//! Construct to match specified reference image.
		FILTER_STEREO_DUAL (
			STAGE *pStageDrape,						//!< Pointer to stage to "drape" onto terrain
			bool TakeOwnershipDrape,				//!< Filter takes ownership of drape stage and will delete when needed
			STAGE *pStageTerrain,					//!< Pointer to stage for terrain elevation
			bool TakeOwnershipTerrain,				//!< Filter takes ownership of terrain stage and will delete when needed
			const STEREOSHIFT& StereoShift,		//!< Stereo shift settings
			const BASE& RefImage,					//!< Reference image to match
			RESAMPLEMETHOD ResampleMethod			//!< Resampling method
			);

		// Destructor.
		virtual ~FILTER_STEREO_DUAL ();

		//! Set transformation accuracy in source cells.
		//! If within the specified accuracy, a fast affine transformation will be used for computation.
		//! If used, must be done before Initialize().
		//! Default accuracy is .5 cells.
		void SetAccuracy (
			double accuracy							//!< Transformation accuracy in source cells.
			) { m_Accuracy = accuracy; }

		//! Set pixel value to use or blend where no terrain is available.
		//! Only non-alpha samples are affected, alpha/opacity samples are left unchanged.
		void SetPixelNoTerrain (
			const PIXEL& pixel,						//!< Pixel value, must have properties matching drape stage
			double BlendFactor						//!< Blend factor (0-1) to control amount used from specified pixel value
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class BLENDER;

		RESAMPLEMETHOD m_ResampleMethodDrape;
		RESAMPLEMETHOD m_ResampleMethodTerrain;
		TRANS2D_MAPGEN m_TransToPrev[2];
		DRECT2D m_RectTotalPrev[2];
		HELPER_RESAMPLE m_DrapeResampler;
		HELPER_RESAMPLE m_TerrainResampler;
		STEREOSHIFT m_StereoShift;
		BLENDER *m_BlenderNoTerrain;
		double m_DisplacementMax;
		double m_Accuracy;
		bool m_PullSinglePixels;

		ERRVALUE ComputePrevMaxSectionSize (
			const SECTION_ITERATOR& TgtIterator,
			int StageIdx,
			DRECT2D& PrevRectAll,
			UINT32& PrevMaxColumns,
			UINT32& PrevMaxRows
			) const;

		ERRVALUE DoPullBegin (
			const SECTION_ITERATOR& TgtIterator,
			int StageIdx
			);

		void ProcessMissingTerrain (
			INT32 TgtRow,
			INT32 TgtColumns,
			BUFFER& TgtFrame,
			const UINT8 *HasTerrain
			) const;

		// BASE/STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// Unimplemented
		FILTER_STEREO_DUAL (const FILTER_STEREO_DUAL&);
		FILTER_STEREO_DUAL& operator= (const FILTER_STEREO_DUAL&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERSTEREODUAL_H

