/**
 * \file spatmod/imagefiltervalidityrange.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_VALIDITYRANGE definition.
 *
 * \if NODOC
 *
 * $Id: imagefiltervalidityrange.h_v 1.1 2009/03/05 18:05:17 mju Exp $
 *
 * $Log: imagefiltervalidityrange.h_v $
 * Revision 1.1  2009/03/05 18:05:17  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERVALIDITYRANGE_H
#define	INC_SPATMOD_IMAGEFILTERVALIDITYRANGE_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to test pixel for containment inside (or outside) range(s).
//! Resulting image will have validity set based on result.
//! All ranges are inclusive - values equal to range limits are included in range.
class FILTER_VALIDITYRANGE : public FILTER {
	public:

		//! Constructor using single range of PIXEL values to test for.
		FILTER_VALIDITYRANGE (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			const PIXEL& MinValue,					//!< Minimum value (inclusive)
			const PIXEL& MaxValue,					//!< Maximum value (inclusive)
			bool ValidOutside = false				//!< True to mark values outside range as valid, false to use values inside
			);

		//! Constructor using single range of 'double' values to test for.
		//! Input image must have single component.
		FILTER_VALIDITYRANGE (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			double MinValue,							//!< Minimum value (inclusive)
			double MaxValue,							//!< Maximum value (inclusive)
			bool ValidOutside = false				//!< True to mark values outside range as valid, false to use values inside
			);

		//! Destructor.
		virtual ~FILTER_VALIDITYRANGE ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		PIXEL m_PixelMin;
		PIXEL m_PixelMax;
		UINT8 **m_TableUINT8;
		bool m_ValidOutside;

		void ProcessUINT8 (UINT16 SampleIdx, UINT8 *pValidity, const UINT8 *pSample, UINT32 NumColumns) const;
		void ProcessUINT16 (UINT16 SampleIdx, UINT8 *pValidity, const UINT16 *pSample, UINT32 NumColumns) const;
		void ProcessSINT16 (UINT16 SampleIdx, UINT8 *pValidity, const INT16 *pSample, UINT32 NumColumns) const;
		void ProcessUINT32 (UINT16 SampleIdx, UINT8 *pValidity, const UINT32 *pSample, UINT32 NumColumns) const;
		void ProcessSINT32 (UINT16 SampleIdx, UINT8 *pValidity, const INT32 *pSample, UINT32 NumColumns) const;
		void ProcessFloat (UINT16 SampleIdx, UINT8 *pValidity, const float *pSample, UINT32 NumColumns) const;
		void ProcessDouble (UINT16 SampleIdx, UINT8 *pValidity, const double *pSample, UINT32 NumColumns) const;

		// STAGE overrides.
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);
		virtual ERRVALUE v_Initialize ();

		// Unimplemented
		FILTER_VALIDITYRANGE (const FILTER_VALIDITYRANGE&);
		FILTER_VALIDITYRANGE& operator= (const FILTER_VALIDITYRANGE&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERVALIDITYRANGE_H

