/**
 * \file <spatmod/imagespans.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::SPANS definition.
 *
 * \if NODOC
 * $Id: imagespans.h_v 1.8 2011/03/31 20:34:20 mju Exp $
 *
 * $Log: imagespans.h_v $
 * Revision 1.8  2011/03/31 20:34:20  mju
 * Add getLogString.
 *
 * Revision 1.7  2009/01/16 17:36:14  mju
 * Add getSpanSizeMaximum/Minimum.
 *
 * Revision 1.6  2008/11/06 15:40:01  mju
 * Add isEqual and comparison operators.
 *
 * Revision 1.5  2007/07/12 14:23:22  mju
 * Add computeEnd.
 *
 * Revision 1.4  2006/12/08 17:17:12  mju
 * Add computeSize/Start.
 *
 * Revision 1.3  2006/11/09 22:44:05  mju
 * Add applyScale.
 *
 * Revision 1.2  2006/11/09 21:26:13  mju
 * Add adjustForRange.
 *
 * Revision 1.1  2006/11/02 15:43:39  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESPANS_H
#define	INC_SPATMOD_IMAGESPANS_H

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Container for defining spans.
class SPANS {
	public:

		//! Constructor
		SPANS ();

		//! Adjust span sizes for specified range.
		//! This may result in number of spans and all sizes changing.
		ERRVALUE AdjustForRange (
			double Start,								//!< New range start relative to existing spans, must be >= 0
			double Size									//!< New size, must be > 0 and <= Start + previous Size
			);

		//! Apply scale factor to all size values.
		//! Does not change number of spans.
		void ApplyScale (
			double zoom									//! Zoom factor to apply
			);

		//! Compute ending value for span.
		//! @return Ending value or error < 0
		double ComputeEnd (
			INT32 span
			) const;

		//! Compute span position.
		//! If value is on span boundary, the span number returned will be the span starting at that value.
		//! @return Span number, -1 if left of first span, "num spans" if right of last span
		INT32 ComputeSpanPos (
			double value,								//! Value to compute for
			bool& IsOnBoundary						//! Return true if position on boundary, false if not
			) const;

		//! Compute size of span.
		//! @return Span size or error < 0.
		double ComputeSize (
			INT32 span
			) const;

		//! Compute starting value for span.
		//! @return Starting value or error < 0
		double ComputeStart (
			INT32 span
			) const;

		//! Get string for use in log.
		MISTRING GetLogString () const;

		//! Get number of spans.
		UINT32 GetNumSpans (
			) const { return (m_NumSpans); }

		//! Get size of first span.
		//! Note that first and last spans may have different sizes than each other or interior spans.
		double GetSpanSizeFirst (
			) const { return (m_SpanSizeFirst); }

		//! Get size of interior span.
		//! If GetNumSpans is less than 3 then will return 0.
		double GetSpanSizeInterior (
			) const { return (m_SpanSizeInterior); }

		//! Get size of last span.
		//! Note that first and last spans may have different sizes than each other or interior spans.
		//! If GetNumSpans is less than 2 then the "last" span will have same size as "first" span.
		double GetSpanSizeLast (
			) const { return (m_SpanSizeLast); }

		//! Get maximum span size.
		double GetSpanSizeMaximum (
			) const;

		//! Get minimum span size.
		double GetSpanSizeMinimum (
			) const;

		//! Get total size of all spans.
		double GetTotalSize (
			) const { return (m_TotalSize); }

		//! Determine if another set of spans with offset and limited size is aligned with this one.
		//! Partial spans are considered "aligned".
		//! @return true if aligned, false if not.
		bool IsAligned (
			const SPANS& testspans,
			double testoffset
			) const;

		//! Test if two sets of SPANS are equal.
		bool IsEqual (
			const SPANS& rhs
			) const;

		//! Set span sizes.
		ERRVALUE SetSpanSizes (
			double SpanSize,
			double SpanSizeFirst = 0,
			double SpanSizeLast = 0
			);

		//! Set total size.
		//! Will reset span dimensions to match total size (result in single span).
		ERRVALUE SetTotalSize (
			double TotalSize
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		double m_TotalSize;
		double m_SpanSizeFirst;
		double m_SpanSizeInterior;
		double m_SpanSizeLast;
		UINT32 m_NumSpans;

		#endif	// GENERATING_DOXYGEN_OUTPUT

	};

inline bool operator== (
	const SPANS& lhs,
	const SPANS& rhs
	) { return (lhs.IsEqual(rhs)); }

inline bool operator!= (
	const SPANS& lhs,
	const SPANS& rhs
	) { return (!lhs.IsEqual(rhs)); }

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	 // INC_SPATMOD_IMAGESPANS_H
