/**
 * \file spatmod/imagetargetrvcbase.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::TARGET_RVC_BASE definition.
 *
 * \if NODOC
 * $Id: imagetargetrvcbase.h_v 1.3 2008/02/28 16:45:13 mju Exp $
 *
 * $Log: imagetargetrvcbase.h_v $
 * Revision 1.3  2008/02/28 16:45:13  mju
 * Add set/getUsage methods.
 *
 * Revision 1.2  2007/05/08 13:06:04  mju
 * Use objitemlist hdr.
 *
 * Revision 1.1  2006/12/08 18:57:00  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGETARGETRVCBASE_H
#define	INC_SPATMOD_IMAGETARGETRVCBASE_H

#ifndef	INC_SPATMOD_IMAGETARGET_H
#include <spatmod/imagetarget.h>
#endif

#ifndef	INC_RVC_OBJITEMLIST_H
#include <rvc/objitemlist.h>
#endif

#ifndef	INC_RVC_RASTER_H
#include <rvc/raster.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Base class for RVC image target implementations.
class TARGET_RVC_BASE : public TARGET {
	public:

		//! Histogram modes.
		enum HISTOGRAM {
			HISTOGRAM_None,					//!< Don't compute histograms
			HISTOGRAM_Default,				//!< Default sample/full based on user preference setting
			HISTOGRAM_AutoSample,			//!< Auto-sample using pyramid tier if available
			HISTOGRAM_Full						//!< Compute full histogram
			};

		//! Destructor.
		virtual ~TARGET_RVC_BASE ();

		//! Get object item list.
		//! Can be used after object creation to determine the actual locations of the objects created.
		const RVC::OBJITEMLIST& GetObjItemList (
			) const { return (m_ObjItemList); }

		//! Set histogram creation mode.
		//! Histogram will be created at end of Process() only if entire extent of target image is specified.
		//! If used, must be called before Process().
		void SetHistogramMode (
			HISTOGRAM HistogramMode
			) { m_HistogramMode = HistogramMode; }

		//! Set pyramiding mode.
		//! Pyramiding will be done at end of Process() only if entire extent of target image is specified.
		//! If used, must be called before Process().
		//! By default pyramids are generated with user's preferred Full/Partial setting and pixel sampling.
		//! Using PYRAMID_Disable will also disable embedded tier creation for JPEG2000 compression.
		void SetPyramidMode (
			RVC::IMAGE::PYRAMID PyramidMode
			) { m_PyramidMode = PyramidMode; }

		//! Set object usage.
		//! If used, must be called before Process().
		void SetUsage (
			RVC::RASTER::USAGE Usage
			);

	protected:

		//! Construct target with single output object.
		//! If source image cannot be written to single object an error will occur when Initialize() is
		//! called, use constructor with OBJITEMLIST if multiple objects are to be created.
		TARGET_RVC_BASE (
			STAGE *pInputStage,								//!< Stage in pipeline to obtain image data from
			const RVC::OBJITEM& ObjItem,					//!< Object location and descriptor
			const RVC::CREDENTIALS *pCredentials = 0	//!< Optional credentials
			);

		//! Construct target with multiple output objects (one per sample).
		//! Number of items objects in list must be appropriate for input image or an error
		//! will be returned from Initialize().
		TARGET_RVC_BASE (
			STAGE *pInputStage,								//!< Stage in pipeline to obtain image data from
			const RVC::OBJITEMLIST& ObjItemList,		//!< Object locations and descriptors
			const RVC::CREDENTIALS *pCredentials = 0	//!< Optional credentials
			);

		//! Get credentials pointer, if any specified in construction.
		const RVC::CREDENTIALS* GetCredentials (
			) const { return (m_pCredentials); }

		//! Get object item list (non-const).
		//! Should only be used during object creation to specify actual locations of objects created.
		RVC::OBJITEMLIST& GetObjItemList (
			) { return (m_ObjItemList); }

		//! Get usage value to store with created raster.
		RVC::RASTER::USAGE GetUsage (
			) const { return (m_Usage); }

		//! Write georeference for each raster if don't already have.
		ERRVALUE WriteGeoreference (
			) const;

		// TARGET overrides.

		//! Derived class should call base method before performing own processing.
		virtual ERRVALUE v_PushBegin (const SECTION_ITERATOR& iterator);

		//! Derived class should call base method after performing own processing.
		//! Will create pyramids and histogram if specified and entire image was processed.
		virtual ERRVALUE v_PushEnd ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		RVC::OBJITEMLIST m_ObjItemList;
		RVC::CREDENTIALS *m_pCredentials;
		RVC::RASTER::USAGE m_Usage;
		HISTOGRAM m_HistogramMode;
		RVC::IMAGE::PYRAMID m_PyramidMode;
		bool m_ProcessedFullImage;

		// Unimplemented
		TARGET_RVC_BASE (const TARGET_RVC_BASE&);
		TARGET_RVC_BASE& operator= (const TARGET_RVC_BASE&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGETARGETRVCBASE_H

